@@ -1,5 +1,5 @@
 #!/bin/mksh
-# $MirOS: contrib/code/jupp/Make-w32.sh,v 1.14 2017/01/11 21:23:52 tg Exp $
+# $MirOS: contrib/code/jupp/Make-w32.sh,v 1.23 2018/02/01 02:26:45 tg Exp $
 
 usage() {
 	print -ru2 "Usage: $0 [-bCgn]"
@@ -24,12 +24,8 @@ done
 
 extrawarnings="-Wall -Wextra"
 (( debug )) && extrawarnings="$extrawarnings -g3"
-extrawarnings="$extrawarnings -Wno-unused-parameter"
-echo "N: gcc-3.4.4-999 does not support -Wno-missing-field-initializers"
-echo "N: expect warnings about those, they are known, do not report them"
-extrawarnings="$extrawarnings -Wno-old-style-definition -Wno-strict-prototypes"
-extrawarnings="$extrawarnings -Wno-cast-qual"
-extrawarnings="$extrawarnings -Wno-missing-prototypes -Wno-missing-declarations"
+extrawarnings="$extrawarnings -Wno-unused-parameter -Wno-cast-qual"
+extrawarnings="$extrawarnings -Wno-strict-prototypes"
 
 export LC_ALL=C
 set -ex
@@ -50,23 +46,24 @@ typeset -u jWIN=$jwin
 if (( contb )); then
 	[[ -s mkw32/build/Makefile ]]
 else
-	rm -rf mkw32
+	rm -rf mkw32 JWIN31$jWIN.*
 	mkdir mkw32{,/{build,$jtop}}
 fi
+date >>JWIN31$jWIN.log
 cd mkw32/build
 export CFLAGS='-Os -march=i486 -mtune=pentium-mmx'
 export CPPFLAGS='-DJUPPRC_BUILTIN_NAME=\"jupp32rc\"'
 (( contb )) || mksh ../../configure \
+    --build=i486-pc-cygwin --host=i486-pc-cygwin \
     --prefix=c:/windows/system32 \
     --sysconfdir=c:/windows/system32 \
     --disable-dependency-tracking \
-    --disable-curses \
-    --disable-termcap \
+    --disable-terminfo \
     --disable-search-libs \
     --disable-getpwnam \
     --disable-termidx \
-    --enable-win32reloc
-make AM_CFLAGS="$extrawarnings"
+    --enable-win32reloc | tee -a ../../JWIN31$jWIN.log
+make AM_CFLAGS="$extrawarnings" | tee -a ../../JWIN31$jWIN.log
 if (( nopkg )); then
 	ln -f joe.exe jupp.exe
 	ln -sf ../../jupprc .
@@ -98,13 +95,22 @@ sed -b -e "s!@jwin@!$jupp!g" -e "s!@ts@!$(date -u +%m/%d/%Y)!g" \
 		print -r -- "$line"
 	fi
 done >setup.inf
-sed -bi "/^setup.inf=1,,/s/^.*\$/$(stat -c '%n=1,,%s
-' setup.inf)/" setup.inf
-sed -bi "/^setup.inf=1,,/s/^.*\$/$(stat -c '%n=1,,%s
-' setup.inf)/" setup.inf
+sz=$(stat -c '%n=1,,%s
+' setup.inf)
+sed -bi "/^setup.inf=1,,/s/^.*\$/$sz/" setup.inf
+sz=$(stat -c '%n=1,,%s
+' setup.inf)
+sed -bi "/^setup.inf=1,,/s/^.*\$/$sz/" setup.inf
+if [[ $sz != "$(stat -c '%n=1,,%s
+' setup.inf)" ]]; then
+	print -rnu2 "Size of SETUP.INF destabilises between $sz and "
+	stat -c '%n=1,,%s
+' setup.inf
+	exit 1
+fi
 chmod 444 *
 cd ..
 zip -D -X -9 -k ../JWIN31$jWIN.ZIP $jtop/*
 cd ..
-ls -l JWIN31$jWIN.ZIP
+ls -l JWIN31$jWIN.*
 (( nocln )) || rm -rf mkw32
