@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/b.c,v 1.15 2016/10/08 17:42:12 tg Exp $ */
 /*
  *	Editor engine
  *	Copyright
@@ -6,17 +5,16 @@
  *
  *	This file is part of JOE (Joe's Own Editor)
  */
+#define EXTERN_B_C
 #include "config.h"
 #include "types.h"
 
+__RCSID("$MirOS: contrib/code/jupp/b.c,v 1.35 2018/01/08 02:01:18 tg Exp $");
+
 #include <unistd.h>
-#include <stdio.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#ifdef HAVE_PWD_H
 #include <pwd.h>
-#endif
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
@@ -37,7 +35,6 @@
 #include "va.h"
 #include "vfile.h"
 #include "vs.h"
-#include "utf8.h"
 #include "charmap.h"
 #include "w.h"
 
@@ -59,13 +56,13 @@ int force = 0;
 VFILE *vmem;
 
 const unsigned char *msgs[] = {
-	US "No error",
-	US "New File",
-	US "Error reading file",
-	US "Error seeking file",
-	US "Error opening file",
-	US "Error writing file",
-	US "File on disk is newer"
+	UC "No error",
+	UC "New File",
+	UC "Error reading file",
+	UC "Error seeking file",
+	UC "Error opening file",
+	UC "Error writing file",
+	UC "File on disk is newer"
 };
 
 /* Get size of gap (amount of free space) */
@@ -93,7 +90,7 @@ static void gstgap(H *hdr, unsigned char *ptr, int ofst)
 }
 
 /* Insert a block */
-static void ginsm(H *hdr, unsigned char *ptr, int ofst, unsigned char *blk, int size)
+static void ginsm(H *hdr, unsigned char *ptr, int ofst, const unsigned char *blk, int size)
 {
 	if (ofst != hdr->hole)
 		gstgap(hdr, ptr, ofst);
@@ -116,8 +113,8 @@ static void grmem(H *hdr, unsigned char *ptr, int ofst, unsigned char *blk, int
 }
 
 
-static H nhdrs = { {&nhdrs, &nhdrs} };
-static H ohdrs = { {&ohdrs, &ohdrs} };
+static H nhdrs = { {&nhdrs, &nhdrs}, 0, 0, 0, 0 };
+static H ohdrs = { {&ohdrs, &ohdrs}, 0, 0, 0, 0 };
 
 /* Header allocation */
 static H *halloc(void)
@@ -147,7 +144,7 @@ static void hfreechn(H *h)
 }
 
 
-static P frptrs = { {&frptrs, &frptrs} };
+static P frptrs = { {&frptrs, &frptrs}, NULL, 0, 0, NULL, 0, 0, 0, 0, 0, 0, NULL };
 
 /* Pointer allocation */
 static P *palloc(void)
@@ -161,8 +158,8 @@ static void pfree(P *p)
 }
 
 /* Doubly linked list of buffers and free buffer structures */
-static B bufs = { {&bufs, &bufs} };
-static B frebufs = { {&frebufs, &frebufs} };
+static B bufs = { {&bufs, &bufs}, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL, { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, { NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, 0, 0 }, NULL, NULL, 0, 0, 0, 0, 0, 0 };
+static B frebufs = { {&frebufs, &frebufs}, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL, { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }, { NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, 0, 0 }, NULL, NULL, 0, 0, 0, 0, 0, 0 };
 
 B *bnext(void)
 {
@@ -264,7 +261,7 @@ void brm(B *b)
 			prm(b->bof->link.next);
 		prm(b->bof);
 		if (b->name)
-			joe_free(b->name);
+			free(b->name);
 		demote(B, link, &frebufs, b);
 	}
 }
@@ -429,7 +426,7 @@ int pisbow(P *p)
 	int d = prgetc(q);
 
 	prm(q);
-	if (joe_isalnum_(p->b->o.charmap,c) && (!joe_isalnum_(p->b->o.charmap,d) || pisbof(p)))
+	if (joe_isalnux(p->b->o.charmap,c) && (!joe_isalnux(p->b->o.charmap,d) || pisbof(p)))
 		return 1;
 	else
 		return 0;
@@ -443,7 +440,7 @@ int piseow(P *p)
 	int c = prgetc(q);
 
 	prm(q);
-	if (joe_isalnum_(p->b->o.charmap,c) && (!joe_isalnum_(p->b->o.charmap,d) || piseof(p)))
+	if (joe_isalnux(p->b->o.charmap,c) && (!joe_isalnux(p->b->o.charmap,d) || piseof(p)))
 		return 1;
 	else
 		return 0;
@@ -506,7 +503,7 @@ int pispure(P *p,int c)
 		if (pgetc(q)!=c) {
 			prm(q);
 			return 0;
-                }
+		}
 	prm(q);
 	return 1;
 }
@@ -619,7 +616,7 @@ int pgetc(P *p)
 				c = d == NO_MORE_DATA ? 0x1000FFFF : 0x1000FFFE;
 				wid = 1;
 			} else if (val)
-				wid = joe_wcwidth(1,c);
+				wid = joe_wcwidth(c);
 		} else {
 			wid = 1;
 		}
@@ -783,9 +780,9 @@ int prgetc(P *p)
 
 		if (val && c!='\t' && c!='\n') {
 			p->valcol = 1;
-			p->col -= joe_wcwidth(1,d);
+			p->col -= joe_wcwidth(d);
 		}
-		
+
 		return d;
 #endif
 	}
@@ -987,7 +984,7 @@ P *pcol(P *p, long goalcol)
 			if (c == '\t')
 				wid = p->b->o.tab - p->col % p->b->o.tab;
 			else
-				wid = joe_wcwidth(1,c);
+				wid = joe_wcwidth(c);
 
 			if (p->col + wid > goalcol)
 				break;
@@ -1784,7 +1781,7 @@ static void bsplit(P *p)
 }
 
 /* Make a chain out of a block of memory (the block must not be empty) */
-static H *bldchn(unsigned char *blk, int size, long *nlines)
+static H *bldchn(const unsigned char *blk, int size, long *nlines)
 {
 	H anchor, *l;
 
@@ -1903,7 +1900,7 @@ P *binsb(P *p, B *b)
 }
 
 /* insert memory block 'blk' at 'p' */
-P *binsm(P *p, unsigned char *blk, int amnt)
+P *binsm(P *p, const unsigned char *blk, int amnt)
 {
 	long nlines;
 	H *h = NULL;
@@ -1937,7 +1934,7 @@ P *binsm(P *p, unsigned char *blk, int amnt)
 P *binsbyte(P *p, unsigned char c)
 {
 	if (p->b->o.crlf && c == '\n')
-		return binsm(p, US "\r\n", 2);
+		return binsm(p, UC "\r\n", 2);
 	else
 		return binsm(p, &c, 1);
 }
@@ -1952,7 +1949,7 @@ P *binsc(P *p, int c)
 	} else {
 		unsigned char ch = c;
 		if (p->b->o.crlf && c == '\n')
-			return binsm(p, US "\r\n", 2);
+			return binsm(p, UC "\r\n", 2);
 		else
 			return binsm(p, &ch, 1);
 	}
@@ -2021,39 +2018,30 @@ B *bread(int fi, long int max)
  *
  * Returns new variable length string.
  */
-unsigned char *parsens(unsigned char *s, long int *skip, long int *amnt)
+unsigned char *
+parsens(const unsigned char *s, long int *skip, long int *amnt)
 {
 	unsigned char *n = vsncpy(NULL, 0, sz(s));
-	int x;
+	size_t x;
 
 	*skip = 0;
-	*amnt = MAXLONG;
-	for (x = sLEN(n) - 1; x > 0 && ((n[x] >= '0' && n[x] <= '9') || n[x] == 'x' || n[x] == 'X'); --x) ;
+	*amnt = LONG_MAX;
+	for (x = sLEN(n) - 1; x > 0 && ((n[x] >= '0' && n[x] <= '9') || (n[x] | 0x20) == 'x'); --x)
+		/* nothing */;
 	if (n[x] == ',') {
+		void *vp;
+
 		n[x] = 0;
-		if (n[x + 1] == 'x' || n[x + 1] == 'X')
-			sscanf((char *)(n + x + 2), "%lx", skip);
-		else if (n[x + 1] == '0' && (n[x + 2] == 'x' || n[x + 2] == 'X'))
-			sscanf((char *)(n + x + 3), "%lx", skip);
-		else if (n[x + 1] == '0')
-			sscanf((char *)(n + x + 1), "%lo", skip);
-		else
-			sscanf((char *)(n + x + 1), "%ld", skip);
-		for (--x; x > 0 && ((n[x] >= '0' && n[x] <= '9') || n[x] == 'x' || n[x] == 'X'); --x) ;
+		*skip = ustol(n + x + 1, &vp, USTOL_EOS);
+		for (--x; x > 0 && ((n[x] >= '0' && n[x] <= '9') || (n[x] | 0x20) == 'x'); --x)
+			/* nothing */;
 		if (n[x] == ',') {
 			n[x] = 0;
-			*amnt = *skip;
-			if (n[x + 1] == 'x' || n[x + 1] == 'X')
-				sscanf((char *)(n + x + 2), "%lx", skip);
-			else if (n[x + 1] == '0' && (n[x + 2] == 'x' || n[x + 2] == 'X'))
-				sscanf((char *)(n + x + 3), "%lx", skip);
-			else if (n[x + 1] == '0')
-				sscanf((char *)(n + x + 1), "%lo", skip);
-			else
-				sscanf((char *)(n + x + 1), "%ld", skip);
+			if (vp != NULL)
+				*amnt = *skip;
+			*skip = ustol(n + x + 1, NULL, USTOL_EOS);
 		}
 	}
-#ifndef __MSDOS__
 	if (n[0] == '~') {
 		for (x = 1; n[x] && n[x] != '/'; ++x) ;
 		if (n[x] == '/') {
@@ -2074,7 +2062,7 @@ unsigned char *parsens(unsigned char *s, long int *skip, long int *amnt)
 				n[x] = '/';
 				if (passwd) {
 					unsigned char *z = vsncpy(NULL, 0,
-							 sz((unsigned char *)(passwd->pw_dir)));
+					    sz((unsigned char *)(passwd->pw_dir)));
 
 					z = vsncpy(z, sLEN(z), sz(n + x));
 					vsrm(n);
@@ -2084,7 +2072,6 @@ unsigned char *parsens(unsigned char *s, long int *skip, long int *amnt)
 			}
 		}
 	}
-#endif
 	return n;
 }
 
@@ -2095,7 +2082,8 @@ unsigned char *parsens(unsigned char *s, long int *skip, long int *amnt)
  * -3 for seek error
  * -4 for open error
  */
-B *bload(unsigned char *s)
+B *
+bload(const unsigned char *s)
 {
 	unsigned char buffer[SEGSIZ];
 	FILE *fi;
@@ -2111,7 +2099,7 @@ B *bload(unsigned char *s)
 	if (!s || !s[0]) {
 		error = -1;
 		b = bmk(NULL);
-		setopt(b,US "");
+		setopt(b, UC "");
 		b->rdonly = b->o.readonly;
 		b->er = error;
 		return b;
@@ -2120,13 +2108,11 @@ B *bload(unsigned char *s)
 	n = parsens(s, &skip, &amnt);
 
 	/* Open file or stream */
-#ifndef __MSDOS__
-	if (n[0] == '!') {
+	if (s[0] == '!') {
 		nescape(maint->t);
 		ttclsn();
 		fi = popen((char *)(n + 1), "r");
 	} else
-#endif
 	if (!strcmp(n, "-"))
 		fi = stdin;
 	else {
@@ -2153,9 +2139,6 @@ B *bload(unsigned char *s)
 			error = -1;
 		else
 			error = -4;
-		b = bmk(NULL);
-		setopt(b,n);
-		b->rdonly = b->o.readonly;
 		goto opnerr;
 	}
 
@@ -2185,16 +2168,20 @@ B *bload(unsigned char *s)
 	b->rdonly = b->o.readonly;
 
 	/* Close stream */
-err:
-#ifndef __MSDOS__
+ err:
 	if (s[0] == '!')
 		pclose(fi);
-	else
-#endif
-	if (strcmp(n, "-"))
+	else if (strcmp(n, "-"))
 		fclose(fi);
 
-opnerr:
+ opnerr:
+	if (!b) {
+		/* error case */
+		b = bmk(NULL);
+		setopt(b,n);
+		b->rdonly = b->o.readonly;
+	}
+
 	if (s[0] == '!') {
 		ttopnn();
 		nreturn(maint->t);
@@ -2204,7 +2191,7 @@ opnerr:
 	b->name = joesep((unsigned char *)strdup(s));
 
 	/* Set flags */
-	if (error || s[0] == '!' || skip || amnt != MAXLONG) {
+	if (error || s[0] == '!' || skip || amnt != LONG_MAX) {
 		b->backup = 1;
 		b->changed = 0;
 	} else if (!strcmp(n, "-")) {
@@ -2267,14 +2254,15 @@ opnerr:
 }
 
 /* Find already loaded buffer or load file into new buffer */
-B *bfind(unsigned char *s)
+B *
+bfind(const unsigned char *s)
 {
 	B *b;
 
 	if (!s || !s[0]) {
 		error = -1;
 		b = bmk(NULL);
-		setopt(b,US "");
+		setopt(b, UC "");
 		b->rdonly = b->o.readonly;
 		b->internal = 0;
 		b->er = error;
@@ -2296,14 +2284,15 @@ B *bfind(unsigned char *s)
 }
 
 /* Find already loaded buffer or load file into new buffer */
-B *bfind_scratch(unsigned char *s)
+B *
+bfind_scratch(const unsigned char *s)
 {
 	B *b;
 
 	if (!s || !s[0]) {
 		error = -1;
 		b = bmk(NULL);
-		setopt(b,US "");
+		setopt(b, UC "");
 		b->rdonly = b->o.readonly;
 		b->internal = 0;
 		b->er = error;
@@ -2330,7 +2319,8 @@ B *bfind_scratch(unsigned char *s)
 	return b;
 }
 
-B *bfind_reload(unsigned char *s)
+B *
+bfind_reload(const unsigned char *s)
 {
 	B *b;
 	b = bload(s);
@@ -2338,7 +2328,8 @@ B *bfind_reload(unsigned char *s)
 	return b;
 }
 
-B *bcheck_loaded(unsigned char *s)
+B *
+bcheck_loaded(const unsigned char *s)
 {
 	B *b;
 
@@ -2416,7 +2407,7 @@ int bsavefd(P *p, int fd, long int size)
 	}
 	prm(np);
 	return error = 0;
-err:
+ err:
 	prm(np);
 	return error = 5;
 }
@@ -2440,20 +2431,17 @@ int bsave(P *p, unsigned char *s, long int size, int flag)
 	if (amnt < size)
 		size = amnt;
 
-#ifndef __MSDOS__
 	if (s[0] == '!') {
 		nescape(maint->t);
 		ttclsn();
 		f = popen((char *)(s + 1), "w");
-	} else
-#endif
-	if (s[0] == '>' && s[1] == '>')
+	} else if (s[0] == '>' && s[1] == '>')
 		f = fopen((char *)(s + 2), "a");
 	else if (!strcmp(s, "-")) {
 		nescape(maint->t);
 		ttclsn();
 		f = stdout;
-	} else if (skip || amnt != MAXLONG)
+	} else if (skip || amnt != LONG_MAX)
 		f = fopen((char *)s, "r+");
 	else {
 		f = fopen((char *)s, "w");
@@ -2476,7 +2464,7 @@ int bsave(P *p, unsigned char *s, long int size, int flag)
 
 	bsavefd(p, fileno(f), size);
 
-	if (!error && force && size && !skip && amnt == MAXLONG) {
+	if (!error && force && size && !skip && amnt == LONG_MAX) {
 		P *q = pdup(p);
 		unsigned char nl = '\n';
 
@@ -2486,13 +2474,10 @@ int bsave(P *p, unsigned char *s, long int size, int flag)
 		prm(q);
 	}
 
-err:
-#ifndef __MSDOS__
+ err:
 	if (s[0] == '!')
 		pclose(f);
-	else
-#endif
-	if (strcmp(s, "-"))
+	else if (strcmp(s, "-"))
 		fclose(f);
 	else
 		fflush(f);
@@ -2504,7 +2489,7 @@ err:
 			p->b->mod_time = sbuf.st_mtime;
 	}
 
-opnerr:
+ opnerr:
 	if (s[0] == '!' || !strcmp(s, "-")) {
 		ttopnn();
 		nreturn(maint->t);
@@ -2556,7 +2541,7 @@ unsigned char *brmem(P *p, unsigned char *blk, int size)
 
 unsigned char *brs(P *p, int size)
 {
-	unsigned char *s = (unsigned char *) joe_malloc(size + 1);
+	unsigned char *s = malloc(size + 1);
 
 	s[size] = 0;
 	return brmem(p, s, size);
@@ -2585,7 +2570,15 @@ unsigned char *brzs(P *p, unsigned char *buf, int size)
 
 /* Save edit buffers when editor dies */
 
-RETSIGTYPE ttsig(int sig)
+RETSIGTYPE
+ttsig(int sig)
+{
+	ttabrt(sig, NULL);
+	_exit(1);
+}
+
+void
+ttabrt(int sig, const char *msg)
 {
 	time_t tim = time(NULL);
 	B *b;
@@ -2593,20 +2586,24 @@ RETSIGTYPE ttsig(int sig)
 	int tmpfd;
 	struct stat sbuf;
 
+	if (msg) {
+		fprintf(stderr, "\r\n*** Aborting JOE because: %s ***\r\n", msg);
+		fflush(stderr);
+	}
+
 	if ((tmpfd = open("DEADJOE", O_RDWR | O_EXCL | O_CREAT, 0600)) < 0) {
+		struct stat cbuf;
+
 		if (lstat("DEADJOE", &sbuf) < 0)
 			_exit(-1);
 		if (!S_ISREG(sbuf.st_mode) || sbuf.st_uid != geteuid())
 			_exit(-1);
-		/*
-		   A race condition still exists between the lstat() and the open()
-		   systemcall, which leads to a possible denial-of-service attack
-		   by setting the file access mode to 600 for every file the
-		   user executing joe has permissions to.
-		   This can't be fixed w/o breacking the behavior of the orig. joe!
-		 */
 		if ((tmpfd = open("DEADJOE", O_RDWR | O_APPEND)) < 0)
 			_exit(-1);
+		/* https://stackoverflow.com/a/2917482/2171120 */
+		if (fstat(tmpfd, &cbuf) < 0 ||
+		    cbuf.st_dev != sbuf.st_dev || cbuf.st_ino != sbuf.st_ino)
+			_exit(-1);
 		if (fchmod(tmpfd, S_IRUSR | S_IWUSR) < 0)
 			_exit(-1);
 	}
@@ -2614,7 +2611,9 @@ RETSIGTYPE ttsig(int sig)
 		_exit(-1);
 
 	fprintf(f, "\n*** Modified files in JOE when it aborted on %s", ctime(&tim));
-	if (sig)
+	if (msg)
+		fprintf(f, "*** JOE was aborted: %s\n", msg);
+	else if (sig)
 		fprintf(f, "*** JOE was aborted by signal %d\n", sig);
 	else
 		fprintf(f, "*** JOE was aborted because the terminal closed\n");
@@ -2628,7 +2627,7 @@ RETSIGTYPE ttsig(int sig)
 			fflush(f);
 			bsavefd(b->bof, fileno(f), b->eof->byte);
 		}
+	fclose(f);
 	if (sig)
 		ttclsn();
-	_exit(1);
 }
