@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/builtin.c,v 1.5 2017/01/10 19:16:27 tg Exp $ */
 /*
  *	Built-in config files
  *	Copyright
@@ -9,7 +8,8 @@
 
 #include "config.h"
 
-#include <stdio.h>
+__RCSID("$MirOS: contrib/code/jupp/builtin.c,v 1.10 2018/01/08 00:48:06 tg Exp $");
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -19,34 +19,35 @@
 
 #define zcmp(a,b) strcmp((char *)(a), (char *)(b))
 
-JFILE *jfopen(unsigned char *name, const char *mode)
+JFILE *
+jfopen(const unsigned char *name, const char *mode)
 {
 	if (name[0] == '*') {
 		int x;
-		char *xname;
+		char *xname, *cp;
 
-		xname = strdup((char *)name + 1);
-		name = (void *)xname;
-		while ((x = *name++)) {
+		xname = strdup((const char *)name + 1);
+		cp = xname;
+		while ((x = *cp++)) {
 			if (x >= 'A' && x <= 'Z')
-				name[-1] = x - 'A' + 'a';
+				cp[-1] = x - 'A' + 'a';
 		}
 
 		for (x = 0; builtins[x]; x += 2) {
 			if (!zcmp(builtins[x], xname)) {
-				JFILE *j = (JFILE *)joe_malloc(sizeof(JFILE));
+				JFILE *j = malloc(sizeof(JFILE));
 				j->f = 0;
 				j->p = builtins[x + 1];
-				joe_free(xname);
+				free(xname);
 				return j;
 			}
 		}
-		joe_free(xname);
+		free(xname);
 		return 0;
 	} else {
-		FILE *f = fopen((char *)name, (char *)mode);
+		FILE *f = fopen((const char *)name, (const char *)mode);
 		if (f) {
-			JFILE *j = (JFILE *)joe_malloc(sizeof(JFILE));
+			JFILE *j = malloc(sizeof(JFILE));
 			j->f = f;
 			j->p = 0;
 			return j;
@@ -61,11 +62,13 @@ int jfclose(JFILE *f)
 	int rtn = 0;
 	if (f->f)
 		rtn = fclose(f->f);
-	joe_free(f);
+	free(f);
 	return rtn;
 }
 
-unsigned char *jfgets(unsigned char *buf,int len,JFILE *f)
+/*XXX fails to honour len (= 1024, in practice) for builtins */
+unsigned char *
+jfgets(unsigned char *buf, int len, JFILE *f)
 {
 	if (f->f)
 		return (unsigned char *)fgets((char *)buf, len, f->f);
