@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/bw.c,v 1.22 2017/07/09 01:15:51 tg Exp $ */
 /*
  *	Edit buffer window generation
  *	Copyright
@@ -9,11 +8,10 @@
 #include "config.h"
 #include "types.h"
 
+__RCSID("$MirOS: contrib/code/jupp/bw.c,v 1.37 2018/01/08 02:01:19 tg Exp $");
+
 #include <string.h>
-#include <stdio.h>
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
 
 #ifdef HAVE_BSD_STRING_H
 #include <bsd/string.h>
@@ -28,7 +26,6 @@
 #include "ublock.h"
 #include "utils.h"
 #include "syntax.h"
-#include "utf8.h"
 #include "charmap.h"
 #include "w.h"
 
@@ -43,7 +40,7 @@ static P *getto(P *p, P *cur, P *top, long int line)
 
 	if (p == NULL) {
 		P *best = cur;
-		long dist = MAXLONG;
+		long dist = LONG_MAX;
 		long d;
 
 		d = (line >= cur->line ? line - cur->line : cur->line - line);
@@ -76,8 +73,11 @@ int mid = 0;
 
 /* For hex */
 
-static void bwfllwh(BW *w)
+static void
+bwfllwh(BW *w)
 {
+	long dofs = 0;
+
 	/* Top must be a muliple of 16 bytes */
 	if (w->top->byte%16) {
 		pbkwd(w->top,w->top->byte%16);
@@ -116,11 +116,10 @@ static void bwfllwh(BW *w)
 	}
 
 	/* Adjust scroll offset */
-	if (w->cursor->byte%16+60 < w->offset) {
-		w->offset = w->cursor->byte%16+60;
-		msetI(w->t->t->updtab + w->y, 1, w->h);
-	} else if (w->cursor->byte%16+60 >= w->offset + w->w) {
-		w->offset = w->cursor->byte%16+60 - (w->w - 1);
+	if (w->cursor->byte % 16 + 60 >= w->w)
+		dofs = w->cursor->byte % 16 + 60 - (w->w - 1);
+	if (w->offset != dofs) {
+		w->offset = dofs;
 		msetI(w->t->t->updtab + w->y, 1, w->h);
 	}
 }
@@ -162,7 +161,7 @@ static void bwfllwt(BW *w)
 		prm(newtop);
 	}
 
-/* Adjust column */
+	/* Adjust column */
 	if (w->cursor->xcol < w->offset) {
 		w->offset = w->cursor->xcol;
 		msetI(w->t->t->updtab + w->y, 1, w->h);
@@ -174,8 +173,9 @@ static void bwfllwt(BW *w)
 
 /* For either */
 
-void bwfllw(BW *w)
+void bwfllw(jobject jO)
 {
+	BW *w = jO.bw;
 	if (w->o.hex)
 		bwfllwh(w);
 	else
@@ -186,7 +186,8 @@ void bwfllw(BW *w)
    If the state is not known, it is computed and the state for all
    of the remaining lines of the window are also recalculated. */
 
-int get_highlight_state(BW *w,int line)
+static int
+get_highlight_state(BW *w, int line)
 {
 	P *tmp = 0;
 	int state;
@@ -310,14 +311,20 @@ void bwdel(BW *w, long int l, long int n, int flg)
 
 /* Update a single line */
 
-static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long int scr, long int from, long int to,int st,BW *bw)
-        
-      
-            			/* Screen line address */
-      				/* Window */
-     				/* Buffer pointer */
-         			/* Starting column to display */
-              			/* Range for marked block */
+static int
+lgen(SCRN *t, int y,
+    /* screen line address */
+    int *screen,
+    int *attr, int x,
+    /* window */
+    int w,
+    /* buffer pointer */
+    P *p,
+    /* starting column to display */
+    long int scr,
+    /* range for marked block */
+    long int from, long int to,
+    int st, BW *bw)
 {
 	int ox = x;
 	int tach, tach1;
@@ -332,10 +339,10 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 
 	struct utf8_sm utf8_sm;
 
-        int *syn = NULL;
-        P *tmp;
-        int idx=0;
-        int atr = 0;
+	int *syn = NULL;
+	P *tmp;
+	int idx=0;
+	int atr = 0;
 
 	utf8_init(&utf8_sm);
 
@@ -358,7 +365,8 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 
 	if (col == scr)
 		goto loop;
-      lp:			/* Display next character */
+ lp:
+	/* Display next character */
 	if (amnt)
 		do {
 			if (ungetit== -1)
@@ -372,7 +380,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 			if (p->b->o.crlf && bc == '\r') {
 				++byte;
 				if (!--amnt) {
-				      pppl:
+ pppl:
 					if (bp == p->ptr + SEGSIZ) {
 						if (pnext(p)) {
 							bp = p->ptr;
@@ -392,7 +400,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 					++amnt;
 					goto eobl;
 				}
-			      nnnl:
+ nnnl:
 				--byte;
 				++amnt;
 			}
@@ -427,11 +435,13 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 				goto eobl;
 			else {
 				int wid = 1;
-				if (p->b->o.charmap->type) {
+
+				/* should be p->b->o.charmap->type but for Coverity; they are identical here */
+				if (bw->b->o.charmap->type) {
 					c = utf8_decode(&utf8_sm,bc);
 
 					if (c>=0) /* Normal decoded character */
-						wid = joe_wcwidth(1,c);
+						wid = joe_wcwidth(c);
 					else if(c== -1) /* Character taken */
 						wid = -1;
 					else if(c== -2) { /* Incomplete sequence */
@@ -470,8 +480,8 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 		goto lp;
 	}
 	goto eof;
-
-      loop:			/* Display next character */
+ loop:
+	/* Display next character */
 	if (amnt)
 		do {
 			if (ungetit== -1)
@@ -485,7 +495,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 			if (p->b->o.crlf && bc == '\r') {
 				++byte;
 				if (!--amnt) {
-				      ppl:
+ ppl:
 					if (bp == p->ptr + SEGSIZ) {
 						if (pnext(p)) {
 							bp = p->ptr;
@@ -505,7 +515,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 					++amnt;
 					goto eobl;
 				}
-			      nnl:
+ nnl:
 				--byte;
 				++amnt;
 			}
@@ -536,7 +546,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 				do {
 					outatr(utf8_map, t, screen + x, attr + x, x, y, tach, c1|atr);
 					tach = tach1;
-					if (ifhave)
+					if (have)
 						goto bye;
 					if (++x == w)
 						goto eosl;
@@ -548,15 +558,20 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 			} else {
 				int wid = -1;
 				int utf8_char;
-				if (p->b->o.charmap->type) { /* UTF-8 */
 
+				/* should be p->b->o.charmap but it’s identical */
+				if (bw->b->o.charmap->type) {
+					/* UTF-8 */
 					utf8_char = utf8_decode(&utf8_sm,bc);
 
-					if (utf8_char >= 0) { /* Normal decoded character */
-						wid = joe_wcwidth(1,utf8_char);
-					} else if(utf8_char== -1) { /* Character taken */
+					if (utf8_char >= 0) {
+						/* Normal decoded character */
+						wid = joe_wcwidth(utf8_char);
+					} else if (utf8_char == -1) {
+						/* Character taken */
 						wid = -1;
-					} else if(utf8_char== -2) { /* Incomplete sequence (FIXME: do something better here) */
+					} else if (utf8_char == -2) {
+						/* Incomplete sequence (FIXME: do something better here) */
  unget_cch:
 						ungetit = bc;
 						++amnt;
@@ -564,7 +579,8 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 						utf8_char = 0x1000FFFE;
 						wid = utf8_sm.ptr;
 						utf8_init(&utf8_sm);
-					} else if(utf8_char== -3) { /* Invalid UTF-8 start character 128-191, 254, 255 */
+					} else if (utf8_char == -3) {
+						/* Invalid UTF-8 start character 128-191, 254, 255 */
 						/* Show as control character */
 						wid = 1;
 						utf8_char = 0x1000FFFE;
@@ -594,7 +610,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 				} else
 					--idx;
 
-				if (ifhave)
+				if (have)
 					goto bye;
 				if (x >= w)
 					goto eosl;
@@ -612,17 +628,17 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 		goto loop;
 	}
 	goto eof;
-
-      eobl:			/* End of buffer line found.  Erase to end of screen line */
+ eobl:
+	/* End of buffer line found.  Erase to end of screen line */
 	++p->line;
-      eof:
+ eof:
 	if (x != w)
 		done = eraeol(t, x, y);
 	else
 		done = 0;
 
 /* Set p to bp/amnt */
-      bye:
+ bye:
 	if (bp - p->ptr <= p->hdr->hole)
 		p->ofst = bp - p->ptr;
 	else
@@ -630,7 +646,7 @@ static int lgen(SCRN *t, int y, int *screen, int *attr, int x, int w, P *p, long
 	p->byte = byte;
 	return done;
 
-      eosl:
+ eosl:
 	if (bp - p->ptr <= p->hdr->hole)
 		p->ofst = bp - p->ptr;
 	else
@@ -652,7 +668,7 @@ static void gennum(BW *w, int *screen, int *attr, SCRN *t, int y, int *comp)
 		strlcpy((char *)buf, "      ",12);
 	for (z = 0; buf[z]; ++z) {
 		outatr(w->b->o.charmap, t, screen + z, attr + z, z, y, buf[z], 0);
-		if (ifhave)
+		if (have)
 			return;
 		comp[z] = buf[z];
 	}
@@ -682,7 +698,7 @@ void bwgenh(BW *w)
 			from = markb->byte;
 			to = markk->byte;
 		}
-	else if (marking && w == (BW *)maint->curwin->object && markb && markb->b == w->b && w->cursor->byte != markb->byte && !from) {
+	else if (marking && w == maint->curwin->object.bw && markb && markb->b == w->b && w->cursor->byte != markb->byte && !from) {
 		if (square) {
 			from = long_min(w->cursor->xcol, markb->xcol);
 			to = long_max(w->cursor->xcol, markb->xcol);
@@ -693,7 +709,7 @@ void bwgenh(BW *w)
 		}
 	}
 
-	if (marking && w == (BW *)maint->curwin->object)
+	if (marking && w == maint->curwin->object.bw)
 		msetI(t->updtab + w->y, 1, w->h);
 
 	if (dosquare) {
@@ -796,7 +812,7 @@ void bwgen(BW *w, int linums)
 			from = markb->byte;
 			to = markk->byte;
 		}
-	else if (marking && w==maint->curwin->object && markb && markb->b == w->b && w->cursor->byte != markb->byte && !from) {
+	else if (marking && w == maint->curwin->object.bw && markb && markb->b == w->b && w->cursor->byte != markb->byte && !from) {
 		if (square) {
 			from = long_min(w->cursor->xcol, markb->xcol);
 			to = long_max(w->cursor->xcol, markb->xcol);
@@ -809,13 +825,13 @@ void bwgen(BW *w, int linums)
 		}
 	}
 
-	if (marking && w==maint->curwin->object)
+	if (marking && w == maint->curwin->object.bw)
 		msetI(t->updtab + w->y, 1, w->h);
 
 	y = w->cursor->line - w->top->line + w->y;
 	attr = t->attr + y*w->t->w;
 	for (screen = t->scrn + y * w->t->w; y != bot; ++y, (screen += w->t->w), (attr += w->t->w)) {
-		if (ifhave && !linums)
+		if (have && !linums)
 			break;
 		if (linums)
 			gennum(w, screen, attr, t, y, t->compose);
@@ -845,7 +861,7 @@ void bwgen(BW *w, int linums)
 	y = w->y;
 	attr = t->attr + w->y * w->t->w;
 	for (screen = t->scrn + w->y * w->t->w; y != w->y + w->cursor->line - w->top->line; ++y, (screen += w->t->w), (attr += w->t->w)) {
-		if (ifhave && !linums)
+		if (have && !linums)
 			break;
 		if (linums)
 			gennum(w, screen, attr, t, y, t->compose);
@@ -894,7 +910,7 @@ void bwresz(BW *w, int wi, int he)
 
 BW *bwmk(W *window, B *b, int prompt)
 {
-	BW *w = (BW *) joe_malloc(sizeof(BW));
+	BW *w = malloc(sizeof(BW));
 
 	w->parent = window;
 	w->b = b;
@@ -942,18 +958,26 @@ void bwrm(BW *w)
 	prm(w->top);
 	prm(w->cursor);
 	brm(w->b);
-	joe_free(w);
+	free(w);
 }
 
-int ustat(BW *bw)
+int ustat_j(BW *bw)
 {
 	static unsigned char buf[80];
 	int c = brch(bw->cursor);
 
 	if (c == NO_MORE_DATA)
-		joe_snprintf_4((char *)buf, sizeof(buf), "** Line %ld  Col %ld  Offset %ld(0x%lX) **", bw->cursor->line + 1, piscol(bw->cursor) + 1, bw->cursor->byte, bw->cursor->byte);
+		joe_snprintf_4((char *)buf, sizeof(buf),
+		    "** Line %ld  Col %ld  Offset %ld(0x%lX) **",
+		    bw->cursor->line + 1, piscol(bw->cursor) + 1,
+		    bw->cursor->byte, bw->cursor->byte);
 	else
-		joe_snprintf_9((char *)buf, sizeof(buf), "** Line %ld  Col %ld  Offset %ld(0x%lX)  %s %d(0%o/0x%X) Width %d **", bw->cursor->line + 1, piscol(bw->cursor) + 1, bw->cursor->byte, bw->cursor->byte, bw->b->o.charmap->name, c, c, c, joe_wcwidth(bw->o.charmap->type,c));
+		joe_snprintf_9((char *)buf, sizeof(buf),
+		    "** Line %ld  Col %ld  Offset %ld(0x%lX)  %s %d(0%o/0x%X) Width %d **",
+		    bw->cursor->line + 1, piscol(bw->cursor) + 1,
+		    bw->cursor->byte, bw->cursor->byte,
+		    bw->b->o.charmap->name, c, c, c,
+		    bw->o.charmap->type ? joe_wcwidth(c) : 1);
 	msgnw(bw->parent, buf);
 	return 0;
 }
