@@ -1,7 +1,6 @@
 #if 0
 .if "0" == "1"
 #endif
-/* $MirOS: contrib/code/jupp/charmap.c,v 1.19 2017/01/11 22:04:31 tg Exp $ */
 /*
  *	UNICODE/ISO-10646 conversion utilities
  *	Copyright
@@ -11,14 +10,15 @@
  */
 
 #include "config.h"
+#include "types.h"
+
+__RCSID("$MirOS: contrib/code/jupp/charmap.c,v 1.30 2018/01/06 00:28:30 tg Exp $");
 
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "va.h"
 #include "utils.h"
-#include "i18n.h"
 #include "path.h"
 #include "charmap.h"
 
@@ -61,57 +61,57 @@ static const struct {
 	const unsigned char *alias;
 	const unsigned char *builtin;
 } alias_table[] = {
-	{ US "c", US "ascii" },
-	{ US "posix", US "ascii" },
-	{ US "iso646", US "ascii" },
-	{ US "8859-1", US "iso-8859-1" },
-	{ US "8859-2", US "iso-8859-2" },
-	{ US "8859-3", US "iso-8859-3" },
-	{ US "8859-4", US "iso-8859-4" },
-	{ US "8859-5", US "iso-8859-5" },
-	{ US "8859-6", US "iso-8859-6" },
-	{ US "8859-7", US "iso-8859-7" },
-	{ US "8859-8", US "iso-8859-8" },
-	{ US "8859-9", US "iso-8859-9" },
-	{ US "8859-10", US "iso-8859-10" },
-	{ US "8859-11", US "iso-8859-11" },
-	{ US "8859-13", US "iso-8859-13" },
-	{ US "8859-14", US "iso-8859-14" },
-	{ US "8859-15", US "iso-8859-15" },
-	{ US "8859-16", US "iso-8859-16" },
-	{ US "oem", US "cp437" },
-	{ US "ansi", US "cp1252" },
-	{ US "latin1", US "iso-8859-1" },
-	{ US "latin2", US "iso-8859-2" },
-	{ US "latin3", US "iso-8859-3" },
-	{ US "latin4", US "iso-8859-4" },
-	{ US "cyrillic", US "iso-8859-5" },
-	{ US "arabic", US "iso-8859-6" },
-	{ US "greek", US "iso-8859-7" },
-	{ US "hebrew", US "iso-8859-8" },
-	{ US "latin5", US "iso-8859-9" },
-	{ US "turkish", US "iso-8859-9" },
-	{ US "latin6", US "iso-8859-10" },
-	{ US "nordic", US "iso-8859-10" },
-	{ US "thai", US "iso-8859-11" },
-	{ US "latin7", US "iso-8859-13" },
-	{ US "latin8", US "iso-8859-14" },
-	{ US "latin9", US "iso-8859-15" },
-	{ US "cp20127", US "ascii" },
-	{ US "cp20866", US "koi8-r" },
-	{ US "cp21866", US "koi8-u" },
-	{ US "cp28591", US "iso-8859-1" },
-	{ US "cp28592", US "iso-8859-2" },
-	{ US "cp28593", US "iso-8859-3" },
-	{ US "cp28594", US "iso-8859-4" },
-	{ US "cp28595", US "iso-8859-5" },
-	{ US "cp28596", US "iso-8859-6" },
-	{ US "cp28597", US "iso-8859-7" },
-	{ US "cp28598", US "iso-8859-8" },
-	{ US "cp28599", US "iso-8859-9" },
-	{ US "cp28603", US "iso-8859-13" },
-	{ US "cp28605", US "iso-8859-15" },
-	{ US "cp65001", US "utf-8" },
+	{ UC "c", UC "ascii" },
+	{ UC "posix", UC "ascii" },
+	{ UC "iso646", UC "ascii" },
+	{ UC "8859-1", UC "iso-8859-1" },
+	{ UC "8859-2", UC "iso-8859-2" },
+	{ UC "8859-3", UC "iso-8859-3" },
+	{ UC "8859-4", UC "iso-8859-4" },
+	{ UC "8859-5", UC "iso-8859-5" },
+	{ UC "8859-6", UC "iso-8859-6" },
+	{ UC "8859-7", UC "iso-8859-7" },
+	{ UC "8859-8", UC "iso-8859-8" },
+	{ UC "8859-9", UC "iso-8859-9" },
+	{ UC "8859-10", UC "iso-8859-10" },
+	{ UC "8859-11", UC "iso-8859-11" },
+	{ UC "8859-13", UC "iso-8859-13" },
+	{ UC "8859-14", UC "iso-8859-14" },
+	{ UC "8859-15", UC "iso-8859-15" },
+	{ UC "8859-16", UC "iso-8859-16" },
+	{ UC "oem", UC "cp437" },
+	{ UC "ansi", UC "cp1252" },
+	{ UC "latin1", UC "iso-8859-1" },
+	{ UC "latin2", UC "iso-8859-2" },
+	{ UC "latin3", UC "iso-8859-3" },
+	{ UC "latin4", UC "iso-8859-4" },
+	{ UC "cyrillic", UC "iso-8859-5" },
+	{ UC "arabic", UC "iso-8859-6" },
+	{ UC "greek", UC "iso-8859-7" },
+	{ UC "hebrew", UC "iso-8859-8" },
+	{ UC "latin5", UC "iso-8859-9" },
+	{ UC "turkish", UC "iso-8859-9" },
+	{ UC "latin6", UC "iso-8859-10" },
+	{ UC "nordic", UC "iso-8859-10" },
+	{ UC "thai", UC "iso-8859-11" },
+	{ UC "latin7", UC "iso-8859-13" },
+	{ UC "latin8", UC "iso-8859-14" },
+	{ UC "latin9", UC "iso-8859-15" },
+	{ UC "cp20127", UC "ascii" },
+	{ UC "cp20866", UC "koi8-r" },
+	{ UC "cp21866", UC "koi8-u" },
+	{ UC "cp28591", UC "iso-8859-1" },
+	{ UC "cp28592", UC "iso-8859-2" },
+	{ UC "cp28593", UC "iso-8859-3" },
+	{ UC "cp28594", UC "iso-8859-4" },
+	{ UC "cp28595", UC "iso-8859-5" },
+	{ UC "cp28596", UC "iso-8859-6" },
+	{ UC "cp28597", UC "iso-8859-7" },
+	{ UC "cp28598", UC "iso-8859-8" },
+	{ UC "cp28599", UC "iso-8859-9" },
+	{ UC "cp28603", UC "iso-8859-13" },
+	{ UC "cp28605", UC "iso-8859-15" },
+	{ UC "cp65001", UC "utf-8" },
 	{ 0, 0 }
 };
 
@@ -120,7 +120,7 @@ static const struct builtin_charmap {
 	int extra_b7, extra_2192;
 	int to_uni[256];
 } builtin_charmaps[] = {
-	{ US "ascii", 0x20, 0x20, {
+	{ UC "ascii", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -153,7 +153,7 @@ static const struct builtin_charmap {
 	    -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 	    -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1,
 	    -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1 }},
-	{ US "iso-8859-1", 0xB7, 0xAC, {
+	{ UC "iso-8859-1", 0xB7, 0xAC, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -186,7 +186,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	0x00f0, 0x00f1, 0x00f2, 0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x00f7,
 	0x00f8, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x00fd, 0x00fe, 0x00ff }},
-	{ US "iso-8859-2", 0x20, 0x20, {
+	{ UC "iso-8859-2", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -219,7 +219,7 @@ static const struct builtin_charmap {
 	0x010d, 0x00e9, 0x0119, 0x00eb, 0x011b, 0x00ed, 0x00ee, 0x010f,
 	0x0111, 0x0144, 0x0148, 0x00f3, 0x00f4, 0x0151, 0x00f6, 0x00f7,
 	0x0159, 0x016f, 0x00fa, 0x0171, 0x00fc, 0x00fd, 0x0163, 0x02d9 }},
-	{ US "iso-8859-3", 0x20, 0x20, {
+	{ UC "iso-8859-3", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -252,7 +252,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	    -1, 0x00f1, 0x00f2, 0x00f3, 0x00f4, 0x0121, 0x00f6, 0x00f7,
 	0x011d, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x016d, 0x015d, 0x02d9 }},
-	{ US "iso-8859-4", 0x20, 0x20, {
+	{ UC "iso-8859-4", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -285,7 +285,7 @@ static const struct builtin_charmap {
 	0x010d, 0x00e9, 0x0119, 0x00eb, 0x0117, 0x00ed, 0x00ee, 0x012b,
 	0x0111, 0x0146, 0x014d, 0x0137, 0x00f4, 0x00f5, 0x00f6, 0x00f7,
 	0x00f8, 0x0173, 0x00fa, 0x00fb, 0x00fc, 0x0169, 0x016b, 0x02d9 }},
-	{ US "iso-8859-5", 0x20, 0x20, {
+	{ UC "iso-8859-5", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -318,7 +318,7 @@ static const struct builtin_charmap {
 	0x0448, 0x0449, 0x044a, 0x044b, 0x044c, 0x044d, 0x044e, 0x044f,
 	0x2116, 0x0451, 0x0452, 0x0453, 0x0454, 0x0455, 0x0456, 0x0457,
 	0x0458, 0x0459, 0x045a, 0x045b, 0x045c, 0x00a7, 0x045e, 0x045f }},
-	{ US "iso-8859-6", 0x20, 0x20, {
+	{ UC "iso-8859-6", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -351,7 +351,7 @@ static const struct builtin_charmap {
 	0x0648, 0x0649, 0x064a, 0x064b, 0x064c, 0x064d, 0x064e, 0x064f,
 	0x0650, 0x0651, 0x0652,     -1,     -1,     -1,     -1,     -1,
 	    -1,     -1,     -1,     -1,     -1,     -1,     -1,     -1 }},
-	{ US "iso-8859-7", 0x20, 0x20, {
+	{ UC "iso-8859-7", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -384,7 +384,7 @@ static const struct builtin_charmap {
 	0x03b8, 0x03b9, 0x03ba, 0x03bb, 0x03bc, 0x03bd, 0x03be, 0x03bf,
 	0x03c0, 0x03c1, 0x03c2, 0x03c3, 0x03c4, 0x03c5, 0x03c6, 0x03c7,
 	0x03c8, 0x03c9, 0x03ca, 0x03cb, 0x03cc, 0x03cd, 0x03ce,     -1 }},
-	{ US "iso-8859-8", 0x20, 0x20, {
+	{ UC "iso-8859-8", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -417,7 +417,7 @@ static const struct builtin_charmap {
 	0x05d8, 0x05d9, 0x05da, 0x05db, 0x05dc, 0x05dd, 0x05de, 0x05df,
 	0x05e0, 0x05e1, 0x05e2, 0x05e3, 0x05e4, 0x05e5, 0x05e6, 0x05e7,
 	0x05e8, 0x05e9, 0x05ea,     -1,     -1, 0x200e, 0x200f,     -1 }},
-	{ US "iso-8859-9", 0x20, 0x20, {
+	{ UC "iso-8859-9", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -450,7 +450,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	0x011f, 0x00f1, 0x00f2, 0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x00f7,
 	0x00f8, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x0131, 0x015f, 0x00ff }},
-	{ US "iso-8859-10", 0x20, 0x20, {
+	{ UC "iso-8859-10", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -483,7 +483,7 @@ static const struct builtin_charmap {
 	0x010d, 0x00e9, 0x0119, 0x00eb, 0x0117, 0x00ed, 0x00ee, 0x00ef,
 	0x00f0, 0x0146, 0x014d, 0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x0169,
 	0x00f8, 0x0173, 0x00fa, 0x00fb, 0x00fc, 0x00fd, 0x00fe, 0x0138 }},
-	{ US "iso-8859-11", 0x20, 0x20, {
+	{ UC "iso-8859-11", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -516,7 +516,7 @@ static const struct builtin_charmap {
 	0x0e48, 0x0e49, 0x0e4a, 0x0e4b, 0x0e4c, 0x0e4d, 0x0e4e, 0x0e4f,
 	0x0e50, 0x0e51, 0x0e52, 0x0e53, 0x0e54, 0x0e55, 0x0e56, 0x0e57,
 	0x0e58, 0x0e59, 0x0e5a, 0x0e5b,     -1,     -1,     -1,     -1 }},
-	{ US "iso-8859-13", 0x20, 0x20, {
+	{ UC "iso-8859-13", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -549,7 +549,7 @@ static const struct builtin_charmap {
 	0x010d, 0x00e9, 0x017a, 0x0117, 0x0123, 0x0137, 0x012b, 0x013c,
 	0x0161, 0x0144, 0x0146, 0x00f3, 0x014d, 0x00f5, 0x00f6, 0x00f7,
 	0x0173, 0x0142, 0x015b, 0x016b, 0x00fc, 0x017c, 0x017e, 0x2019 }},
-	{ US "iso-8859-14", 0x20, 0x20, {
+	{ UC "iso-8859-14", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -582,7 +582,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	0x0175, 0x00f1, 0x00f2, 0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x1e6b,
 	0x00f8, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x00fd, 0x0177, 0x00ff }},
-	{ US "iso-8859-15", 0x20, 0x20, {
+	{ UC "iso-8859-15", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -615,7 +615,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	0x00f0, 0x00f1, 0x00f2, 0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x00f7,
 	0x00f8, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x00fd, 0x00fe, 0x00ff }},
-	{ US "iso-8859-16", 0x20, 0x20, {
+	{ UC "iso-8859-16", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -648,7 +648,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	0x0111, 0x0144, 0x00f2, 0x00f3, 0x00f4, 0x0151, 0x00f6, 0x015b,
 	0x0171, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x0119, 0x021b, 0x00ff }},
-	{ US "koi-8", 0x20, 0x20, {
+	{ UC "koi-8", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -681,7 +681,7 @@ static const struct builtin_charmap {
 	0x0425, 0x0418, 0x0419, 0x041a, 0x041b, 0x041c, 0x041d, 0x041e,
 	0x041f, 0x042f, 0x0420, 0x0421, 0x0422, 0x0423, 0x0416, 0x0412,
 	0x042c, 0x042b, 0x0417, 0x0428, 0x042d, 0x0429, 0x0427,     -1 }},
-	{ US "koi8-r", 0x20, 0x20, {
+	{ UC "koi8-r", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -714,7 +714,7 @@ static const struct builtin_charmap {
 	0x0425, 0x0418, 0x0419, 0x041a, 0x041b, 0x041c, 0x041d, 0x041e,
 	0x041f, 0x042f, 0x0420, 0x0421, 0x0422, 0x0423, 0x0416, 0x0412,
 	0x042c, 0x042b, 0x0417, 0x0428, 0x042d, 0x0429, 0x0427, 0x042a }},
-	{ US "koi8-t", 0x20, 0x20, {
+	{ UC "koi8-t", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -747,7 +747,7 @@ static const struct builtin_charmap {
 	0x0425, 0x0418, 0x0419, 0x041a, 0x041b, 0x041c, 0x041d, 0x041e,
 	0x041f, 0x042f, 0x0420, 0x0421, 0x0422, 0x0423, 0x0416, 0x0412,
 	0x042c, 0x042b, 0x0417, 0x0428, 0x042d, 0x0429, 0x0427, 0x042a }},
-	{ US "koi8-u", 0x20, 0x20, {
+	{ UC "koi8-u", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -780,7 +780,7 @@ static const struct builtin_charmap {
 	0x0425, 0x0418, 0x0419, 0x041a, 0x041b, 0x041c, 0x041d, 0x041e,
 	0x041f, 0x042f, 0x0420, 0x0421, 0x0422, 0x0423, 0x0416, 0x0412,
 	0x042c, 0x042b, 0x0417, 0x0428, 0x042d, 0x0429, 0x0427, 0x042a }},
-	{ US "cp437", 0xF9, 0xAA, {
+	{ UC "cp437", 0xF9, 0xAA, {
 	/* From: src/share/i18n/csmapper/CP/CP437%UCS.src,v 1.2 2014/06/26 18:58:50 tg Exp $ */
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000A, 0x000B, 0x000C, 0x000D, 0x000E, 0x000F,
@@ -814,7 +814,7 @@ static const struct builtin_charmap {
 	0x03A6, 0x0398, 0x03A9, 0x03B4, 0x221E, 0x03C6, 0x03B5, 0x2229,
 	0x2261, 0x00B1, 0x2265, 0x2264, 0x2320, 0x2321, 0x00F7, 0x2248,
 	0x00B0, 0x2219, 0x00B7, 0x221A, 0x207F, 0x00B2, 0x25A0, 0x00A0 }},
-	{ US "cp1251", 0x20, 0x20, {
+	{ UC "cp1251", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -847,7 +847,7 @@ static const struct builtin_charmap {
 	0x0438, 0x0439, 0x043a, 0x043b, 0x043c, 0x043d, 0x043e, 0x043f,
 	0x0440, 0x0441, 0x0442, 0x0443, 0x0444, 0x0445, 0x0446, 0x0447,
 	0x0448, 0x0449, 0x044a, 0x044b, 0x044c, 0x044d, 0x044e, 0x044f }},
-	{ US "cp1252", 0xB7, 0xAC, {
+	{ UC "cp1252", 0xB7, 0xAC, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -880,7 +880,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x00ec, 0x00ed, 0x00ee, 0x00ef,
 	0x00f0, 0x00f1, 0x00f2, 0x00f3, 0x00f4, 0x00f5, 0x00f6, 0x00f7,
 	0x00f8, 0x00f9, 0x00fa, 0x00fb, 0x00fc, 0x00fd, 0x00fe, 0x00ff }},
-	{ US "cp1255", 0x20, 0x20, {
+	{ UC "cp1255", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -913,7 +913,7 @@ static const struct builtin_charmap {
 	0x05d8, 0x05d9, 0x05da, 0x05db, 0x05dc, 0x05dd, 0x05de, 0x05df,
 	0x05e0, 0x05e1, 0x05e2, 0x05e3, 0x05e4, 0x05e5, 0x05e6, 0x05e7,
 	0x05e8, 0x05e9, 0x05ea,     -1,     -1, 0x200e, 0x200f,     -1 }},
-	{ US "cp1256", 0x20, 0x20, {
+	{ UC "cp1256", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -946,7 +946,7 @@ static const struct builtin_charmap {
 	0x00e8, 0x00e9, 0x00ea, 0x00eb, 0x0649, 0x064a, 0x00ee, 0x00ef,
 	0x064b, 0x064c, 0x064d, 0x064e, 0x00f4, 0x064f, 0x0650, 0x00f7,
 	0x0651, 0x00f9, 0x0652, 0x00fb, 0x00fc, 0x200e, 0x200f, 0x06d2 }},
-	{ US "armscii-8", 0x20, 0x20, {
+	{ UC "armscii-8", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -979,7 +979,7 @@ static const struct builtin_charmap {
 	0x054c, 0x057c, 0x054d, 0x057d, 0x054e, 0x057e, 0x054f, 0x057f,
 	0x0550, 0x0580, 0x0551, 0x0581, 0x0552, 0x0582, 0x0553, 0x0583,
 	0x0554, 0x0584, 0x0555, 0x0585, 0x0556, 0x0586, 0x055a,     -1 }},
-	{ US "tis-620", 0x20, 0x20, {
+	{ UC "tis-620", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -1012,7 +1012,7 @@ static const struct builtin_charmap {
 	0x0e48, 0x0e49, 0x0e4a, 0x0e4b, 0x0e4c, 0x0e4d, 0x0e4e, 0x0e4f,
 	0x0e50, 0x0e51, 0x0e52, 0x0e53, 0x0e54, 0x0e55, 0x0e56, 0x0e57,
 	0x0e58, 0x0e59, 0x0e5a, 0x0e5b,     -1,     -1,     -1,     -1 }},
-	{ US "georgian-ps", 0x20, 0x20, {
+	{ UC "georgian-ps", 0x20, 0x20, {
 	0x0000, 0x0001, 0x0002, 0x0003, 0x0004, 0x0005, 0x0006, 0x0007,
 	0x0008, 0x0009, 0x000a, 0x000b, 0x000c, 0x000d, 0x000e, 0x000f,
 	0x0010, 0x0011, 0x0012, 0x0013, 0x0014, 0x0015, 0x0016, 0x0017,
@@ -1062,7 +1062,7 @@ pair_cmp(struct pair *a, struct pair *b)
 
 /* Predicate and conversion functions for byte-oriented charmaps */
 
-int
+static int
 byte_ispunct(struct charmap *map, int c)
 {
 	int ofst = (c >> 3);
@@ -1071,10 +1071,10 @@ byte_ispunct(struct charmap *map, int c)
 	if (c < 0 || c > 255)
 		return (0);
 	return (((map->print_map[ofst] & bitn) != 0) &&
-	    ((map->alnum__map[ofst] & bitn) == 0));
+	    ((map->alnux_map[ofst] & bitn) == 0));
 }
 
-int
+static int
 byte_isprint(struct charmap *map, int c)
 {
 	int ofst = (c >> 3);
@@ -1085,41 +1085,41 @@ byte_isprint(struct charmap *map, int c)
 	return ((map->print_map[ofst] & bitn) != 0);
 }
 
-int
+static int
 byte_isspace(struct charmap *map, int c)
 {
 	return (c == 32 || (c >= 9 && c <= 13));
 }
 
-int
-byte_isalpha_(struct charmap *map, int c)
+static int
+byte_isalphx(struct charmap *map, int c)
 {
 	int ofst = (c >> 3);
 	int bitn = (1 << (c & 7));
 
 	if (c < 0 || c > 255)
 		return (0);
-	return ((map->alpha__map[ofst] & bitn) != 0);
+	return ((map->alphx_map[ofst] & bitn) != 0);
 }
 
-int
-byte_isalnum_(struct charmap *map, int c)
+static int
+byte_isalnux(struct charmap *map, int c)
 {
 	int ofst = (c >> 3);
 	int bitn = (1 << (c & 7));
 
 	if (c < 0 || c > 255)
 		return (0);
-	return ((map->alnum__map[ofst] & bitn) != 0);
+	return ((map->alnux_map[ofst] & bitn) != 0);
 }
 
-int
+static int
 byte_tolower(struct charmap *map, int c)
 {
 	return ((c < 0 || c > 255) ? c : map->lower_map[c]);
 }
 
-int
+static int
 byte_toupper(struct charmap *map, int c)
 {
 	return ((c < 0 || c > 255) ? c : map->upper_map[c]);
@@ -1155,13 +1155,13 @@ process_builtin(const struct builtin_charmap *builtin)
 	extra_b7 = builtin->extra_b7;
 	extra_2192 = builtin->extra_2192;
 	map = malloc(sizeof(struct charmap));
-	map->name = (unsigned char *)strdup((const char *)builtin->name);
+	map->name = builtin->name;
 	map->type = 0;
 	map->is_punct = byte_ispunct;
 	map->is_print = byte_isprint;
 	map->is_space = byte_isspace;
-	map->is_alpha_ = byte_isalpha_;
-	map->is_alnum_ = byte_isalnum_;
+	map->is_alphx = byte_isalphx;
+	map->is_alnux = byte_isalnux;
 	map->to_lower = byte_tolower;
 	map->to_upper = byte_toupper;
 	map->to_uni = to_uni;
@@ -1195,8 +1195,8 @@ process_builtin(const struct builtin_charmap *builtin)
 
 	for (x = 0; x != 32; ++x) {
 		map->print_map[x] = 0;
-		map->alpha__map[x] = 0;
-		map->alnum__map[x] = 0;
+		map->alphx_map[x] = 0;
+		map->alnux_map[x] = 0;
 	}
 
 	for (x = 0; x != 256; ++x) {
@@ -1208,8 +1208,8 @@ process_builtin(const struct builtin_charmap *builtin)
 			if (joe_iswprint(NULL, c))
 				set_bit(map->print_map, x);
 			if (joe_iswalpha(NULL, c)) {
-				set_bit(map->alpha__map, x);
-				set_bit(map->alnum__map, x);
+				set_bit(map->alphx_map, x);
+				set_bit(map->alnux_map, x);
 			}
 
 			y = joe_towlower(NULL, c);
@@ -1225,15 +1225,15 @@ process_builtin(const struct builtin_charmap *builtin)
 	/* Set underbar <U+005F> */
 
 	if ((c = from_uni(map, 0x5F)) != -1) {
-		set_bit(map->alpha__map, c);
-		set_bit(map->alnum__map, c);
+		set_bit(map->alphx_map, c);
+		set_bit(map->alnux_map, c);
 	}
 
 	/* Put digits into alnum map */
 
 	for (x = 0x30; x != 0x3A; ++x) {
 		if ((c = from_uni(map, x)) != -1)
-			set_bit(map->alnum__map, c);
+			set_bit(map->alnux_map, c);
 	}
 
 	map->next = charmaps;
@@ -1249,15 +1249,15 @@ load_builtins(void)
 
 	/* install UTF-8 map (ties into i18n module) */
 	map = malloc(sizeof(struct charmap));
-	map->name = US "utf-8";
+	map->name = UC "utf-8";
 	map->type = 1;
 	map->to_uni = rtn_arg;
 	map->from_uni = rtn_arg;
 	map->is_punct = joe_iswpunct;
 	map->is_print = joe_iswprint;
 	map->is_space = joe_iswspace;
-	map->is_alpha_ = joe_iswalpha;
-	map->is_alnum_ = joe_iswalnum;
+	map->is_alphx = joe_iswalpha;
+	map->is_alnux = joe_iswalnum;
 	map->to_lower = joe_towlower;
 	map->to_upper = joe_towupper;
 	map->next = charmaps;
@@ -1306,10 +1306,10 @@ parse_charmap(const unsigned char *name, FILE *f)
 			int uni;
 			int byt;
 
-			sscanf((char *)bf1+2, "%x", &uni);
+			uni = ustolb(bf1 + 2, NULL, 0, 0x10FFFF, USTOL_HEX);
 			parse_ws(&p, comment_char);
 			parse_tows(&p, bf1);
-			sscanf((char *)bf1+2, "%x", &byt);
+			byt = ustolb(bf1 + 2, NULL, 0, 255, USTOL_HEX);
 			b->to_uni[byt] = uni;
 		}
 	}
@@ -1332,7 +1332,7 @@ map_up(int c)
 	return ((c >= 'a' && c <= 'z') ? (c + 'A' - 'a') : c);
 }
 
-int
+static int
 map_name_cmp(const unsigned char *a, const unsigned char *b)
 {
 	unsigned char ca, cb;
@@ -1346,7 +1346,7 @@ map_name_cmp(const unsigned char *a, const unsigned char *b)
  into_the_loop:
 	while ((ca = *a) == '-' || ca == '_')
 		++a;
-	while ((cb = *b) == '-' || ca == '_')
+	while ((cb = *b) == '-' || cb == '_')
 		++b;
 	goto loop_beg;
  loop_end:
@@ -1400,8 +1400,11 @@ find_charmap(const unsigned char *name)
 	}
 
 	/* Parse and install character map from file */
-	if (f && (b = parse_charmap(name, f)))
-		return (process_builtin(b));
+	if (f && (b = parse_charmap(name, f))) {
+		m = process_builtin(b);
+		free(b);
+		return (m);
+	}
 
 	/* Check builtin sets */
 	for (y = 0; y != NELEM(builtin_charmaps); ++y)
@@ -1425,7 +1428,7 @@ main(int argc, char *argv[])
 		printf("Not found\n");
 		return (1);
 	}
-	sscanf(argv[2], "%x", &u);
+	u = ustol(argv[2], NULL, USTOL_TRIM | USTOL_EOS);
 	printf("Unicode=%X\n", uni = to_uni(map, u));
 	printf("Local=%X\n", from_uni(map, uni));
 	return (0);
@@ -1466,7 +1469,7 @@ get_encodings(void)
 
 	if ((p = (unsigned char *)getenv("HOME"))) {
 		joe_snprintf_1((char *)buf, sizeof(buf), "%s/.jupp/charmaps", p);
-		if (!chpwd(buf) && (t = rexpnd(US "*"))) {
+		if (!chpwd(buf) && (t = rexpnd(UC "*"))) {
 			for (x = 0; x != aLEN(t); ++x)
 				if (strcmp(t[x], "..")) {
 					for (y = 0; y != aLEN(encodings); ++y)
@@ -1481,7 +1484,7 @@ get_encodings(void)
 		}
 	}
 
-	if (!chJpwd(US ("charmaps")) && (t = rexpnd(US "*"))) {
+	if (!chJpwd(UC "charmaps") && (t = rexpnd(UC "*"))) {
 		for (x = 0; x != aLEN(t); ++x)
 			if (strcmp(t[x], "..")) {
 				for (y = 0; y != aLEN(encodings); ++y)
@@ -1529,7 +1532,7 @@ joe_strtolower(unsigned char *s)
 
 PROG=		charmap
 SRCS=		charmap.c
-SRCS+=		i18n.c path.c utf8.c utils.c va.c vs.c
+SRCS+=		compat.c i18n.c path.c utf8.c utils.c va.c vs.c
 NOMAN=		Yes
 CPPFLAGS+=	-DTEST -DTEST_CHARMAP
 CPPFLAGS+=	-DJUPP_WIN32RELOC=0 -D'JOERC="/etc/joe"'
@@ -1537,10 +1540,8 @@ CPPFLAGS+=	-DJUPP_WIN32RELOC=0 -D'JOERC="/etc/joe"'
 .include <bsd.own.mk>
 
 .ifdef __CRAZY
-COPTS+=		-Wno-unused-parameter \
-		-Wno-strict-prototypes \
-		-Wno-cast-qual \
-		-Wno-missing-prototypes -Wno-missing-declarations
+COPTS+=		-Wno-unused-parameter -Wno-cast-qual
+COPTS+=		-Wno-strict-prototypes
 .endif
 
 .include <bsd.prog.mk>
