@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/cmd.h,v 1.4 2014/06/09 16:06:46 tg Exp $ */
 /*
  *	Command execution
  *	Copyright
@@ -9,8 +8,9 @@
 #ifndef _JOE_CMD_H
 #define _JOE_CMD_H 1
 
-#include "config.h"
-#include "types.h"
+#ifdef EXTERN_CMD_C
+__IDSTRING(rcsid_cmd_h, "$MirOS: contrib/code/jupp/cmd.h,v 1.9 2018/01/07 17:24:48 tg Exp $");
+#endif
 
 extern CMD cmds[];		/* Built-in commands */
 extern int dobeep;
@@ -27,14 +27,15 @@ extern int dobeep;
 #define EMOD		128	/* Not allowed on readonly files */
 /* These use same bits as TYPE* in types.h */
 #define EBLOCK		0x4000	/* Finish block selection */
+#define ECHK0COL	0x8000	/* Reposition screen from leftmost */
 
 /* CMD *findcmd(char *s);
  * Return command address for given name
  */
-CMD *findcmd PARAMS((unsigned char *s));
-void addcmd PARAMS((unsigned char *s, MACRO *m));
+CMD *findcmd(const unsigned char *s);
+void addcmd(const unsigned char *s, MACRO *m);
 
 /* Execute a command.  Returns return value of command */
-int execmd PARAMS((CMD *cmd, int k));
+int execmd(CMD *cmd, int k);
 
 #endif
