@@ -1,11 +1,14 @@
-/* $MirOS: contrib/code/jupp/config.h,v 1.7 2014/06/26 18:15:17 tg Exp $ */
-
 #ifndef _JOE_CONFIG_H
 #define _JOE_CONFIG_H
 
+/* see bottom for RCSID on this one */
+
 #ifndef TEST
 #include "autoconf.h"
 #else
+#define HAVE_CTIME 1
+#define HAVE_STRLCAT 1
+#define HAVE_STRLCPY 1
 #define HAVE_DECL_STRLCAT 1
 #define HAVE_DECL_STRLCPY 1
 #define HAVE_GETCWD 1
@@ -19,7 +22,10 @@
 #define HAVE_STDLIB_H 1
 #define HAVE_UNISTD_H 1
 #define RETSIGTYPE void
-#define PARAMS(protos) protos
+#endif
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
 #endif
 
 #ifdef HAVE_SNPRINTF
@@ -52,34 +58,12 @@
 
 #endif
 
-#define MAXINT  ((((unsigned int)-1)/2)-1)
-#define MAXLONG ((((unsigned long)-1L)/2)-1)
-
 #include <stdio.h>
 #ifndef EOF
-#define EOF -1
+#define EOF (-1)
 #endif
 #define NO_MORE_DATA EOF
 
-#if defined __MSDOS__ && SIZEOF_INT == 2 /* real mode ms-dos compilers */
-#if SIZEOF_VOID_P == 4 /* real mode ms-dos compilers with 'far' memory model or something like that */
-#define physical(a)  (((unsigned long)(a)&0xFFFF)+(((unsigned long)(a)&0xFFFF0000)>>12))
-#define normalize(a) ((void *)(((unsigned long)(a)&0xFFFF000F)+(((unsigned long)(a)&0x0000FFF0)<<12)))
-#else
-#define physical(a) ((unsigned long)(a))
-#define normalize(a) (a)
-#endif /* sizeof(void *) == 4 */
-
-#define SEGSIZ 1024
-#define PGSIZE 1024
-#define LPGSIZE 10
-#define ILIMIT (PGSIZE*96L)
-#define HTSIZE 128
-
-#else /* not real mode ms-dos */
-
-#define physical(a) ((unsigned long)(a))
-#define normalize(a) (a)
 #ifdef PAGE_SIZE
 #define PGSIZE PAGE_SIZE
 #else
@@ -90,14 +74,10 @@
 #define ILIMIT (PGSIZE*1024)
 #define HTSIZE 2048
 
-#endif /* real mode ms-dos */
-
 #if !defined(__GNUC__) && !defined(__attribute__)
 #define __attribute__(p) /* nothing */
 #endif
 
-#define ATTR_UNUSED	__attribute__((__unused__))
-
 #ifdef HAVE_GCC_ATTRIBUTE_BOUNDED
 #define ATTR_BOUNDED(p)	__attribute__((__bounded__ p))
 #else
@@ -113,4 +93,50 @@ size_t strlcpy(char *, const char *, size_t)
     ATTR_BOUNDED((__string__, 1, 3));
 #endif
 
+/* from mksh */
+
+#define BIT(i)		(1U << (i))
+#define NELEM(a)	(sizeof(a) / sizeof((a)[0]))
+
+#if defined(MirBSD) && (MirBSD >= 0x09A1) && \
+    defined(__ELF__) && defined(__GNUC__) && \
+    !defined(__llvm__) && !defined(__NWCC__)
+/*
+ * We got usable __IDSTRING __COPYRIGHT __RCSID __SCCSID macros
+ * which work for all cases; no need to redefine them using the
+ * "portable" macros from below when we might have the "better"
+ * gcc+ELF specific macros or other system dependent ones.
+ */
+#else
+#undef __IDSTRING
+#undef __IDSTRING_CONCAT
+#undef __IDSTRING_EXPAND
+#undef __COPYRIGHT
+#undef __RCSID
+#undef __SCCSID
+#define __IDSTRING_CONCAT(l,p)		__LINTED__ ## l ## _ ## p
+#define __IDSTRING_EXPAND(l,p)		__IDSTRING_CONCAT(l,p)
+#ifdef MKSH_DONT_EMIT_IDSTRING
+#define __IDSTRING(prefix, string)	/* nothing */
+#elif defined(__ELF__) && defined(__GNUC__) && \
+    !defined(__llvm__) && !defined(__NWCC__) && !defined(NO_ASM)
+#define __IDSTRING(prefix, string)				\
+	__asm__(".section .comment"				\
+	"\n	.ascii	\"@(\"\"#)" #prefix ": \""		\
+	"\n	.asciz	\"" string "\""				\
+	"\n	.previous")
+#else
+#define __IDSTRING(prefix, string)				\
+	static const char __IDSTRING_EXPAND(__LINE__,prefix) []	\
+	    __attribute__((__used__)) = "@(""#)" #prefix ": " string
+#endif
+#define __COPYRIGHT(x)		__IDSTRING(copyright,x)
+#define __RCSID(x)		__IDSTRING(rcsid,x)
+#define __SCCSID(x)		__IDSTRING(sccsid,x)
+#endif
+
+#ifdef EXTERN
+__IDSTRING(rcsid_config_h, "$MirOS: contrib/code/jupp/config.h,v 1.14 2017/12/08 02:17:21 tg Exp $");
+#endif
+
 #endif /* ifndef _JOE_CONFIG_H */
