@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/main.h,v 1.3 2012/12/22 00:06:12 tg Exp $ */
 /*
  *	Editor startup and edit loop
  *	Copyright
@@ -9,17 +8,18 @@
 #ifndef _JOE_MAIN_H
 #define _JOE_MAIN_H 1
 
-#include "config.h"
-#include "types.h"
+#ifdef EXTERN_B_C
+__IDSTRING(rcsid_main_h, "$MirOS: contrib/code/jupp/main.h,v 1.6 2017/12/06 21:16:58 tg Exp $");
+#endif
 
 extern const char null[];
 
 extern unsigned char *exmsg;	/* Exit message */
 extern int help;		/* Set to start with help on */
 extern SCREEN *maint;		/* Primary screen */
-void nungetc PARAMS((int c));
-void dofollows PARAMS((void));
-int edloop PARAMS((int flg));
-void edupd PARAMS((int flg));
+void nungetc(int c);
+void dofollows(void);
+int edloop(int flg);
+void edupd(int flg);
 
 #endif
