@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/queue.c,v 1.3 2017/01/10 19:16:27 tg Exp $ */
 /*
  * 	Doubly linked list primitives
  *	Copyright
@@ -9,6 +8,8 @@
 #include "config.h"
 #include "types.h"
 
+__RCSID("$MirOS: contrib/code/jupp/queue.c,v 1.5 2017/12/08 02:00:40 tg Exp $");
+
 #include <stdlib.h>
 
 #include "queue.h"
@@ -23,15 +24,15 @@ void *alitem(void *list, int itemsize)
 	STDITEM	*freelist = (STDITEM *)list;
 
 	if (qempty(STDITEM, link, freelist)) {
-		STDITEM *i = (STDITEM *) joe_malloc(itemsize * 16);
-		STDITEM *z = (STDITEM *) ((unsigned char *) i + itemsize * 16);
+		STDITEM *i = malloc(itemsize * 16);
+		STDITEM *z = (STDITEM *)((unsigned char *)i + itemsize * 16);
 
 		while (i != z) {
 			enquef(STDITEM, link, freelist, i);
-			i = (STDITEM *) ((unsigned char *) i + itemsize);
+			i = (STDITEM *)((unsigned char *)i + itemsize);
 		}
 	}
-	return (void *) deque_f(STDITEM, link, freelist->link.prev);
+	return (void *)deque_f(STDITEM, link, freelist->link.prev);
 }
 
 void frchn(void *list, void *ch)
