@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/qw.c,v 1.5 2017/01/10 19:16:27 tg Exp $ */
 /*
  *	Query windows
  *	Copyright
@@ -9,17 +8,20 @@
 #include "config.h"
 #include "types.h"
 
+__RCSID("$MirOS: contrib/code/jupp/qw.c,v 1.12 2018/01/06 00:28:32 tg Exp $");
+
 #include <stdlib.h>
 
 #include "utils.h"
 #include "vs.h"
 #include "charmap.h"
+#include "qw.h"
 #include "scrn.h"
-#include "utf8.h"
 #include "w.h"
 
-static void dispqw(QW *qw)
+static void dispqw(jobject jO, int flg __attribute__((__unused__)))
 {
+	QW *qw = jO.qw;
 	W *w = qw->parent;
 
 	/* Scroll buffer and position prompt */
@@ -35,21 +37,22 @@ static void dispqw(QW *qw)
 	/* Generate prompt */
 	w->t->t->updtab[w->y] = 1;
 	genfield(w->t->t,
-	         w->t->t->scrn + w->y * w->t->t->co + w->x,
-	         w->t->t->attr + w->y * w->t->t->co + w->x,
-	         w->x,
-	         w->y,
-	         qw->promptofst,
-	         qw->prompt,
-	         qw->promptlen,
-	         0,
-	         w->w - w->x,
-	         1,
-		 NULL);
+	    w->t->t->scrn + w->y * w->t->t->co + w->x,
+	    w->t->t->attr + w->y * w->t->t->co + w->x,
+	    w->x,
+	    w->y,
+	    qw->promptofst,
+	    qw->prompt,
+	    qw->promptlen,
+	    0,
+	    w->w - w->x,
+	    1,
+	    NULL);
 }
 
-static void dispqwn(QW *qw)
+static void dispqwn(jobject jO, int flg __attribute__((__unused__)))
 {
+	QW *qw = jO.qw;
 	W *w = qw->parent;
 
 	/* Scroll buffer and position prompt */
@@ -59,39 +62,40 @@ static void dispqwn(QW *qw)
 		qw->promptofst = 0;
 
 	/* Set cursor position */
-	if (w->win->watom->follow && w->win->object)
+	if (w->win->watom->follow && w->win->object.base)
 		w->win->watom->follow(w->win->object);
-	if (w->win->watom->disp && w->win->object)
-		w->win->watom->disp(w->win->object);
+	if (w->win->watom->disp && w->win->object.base)
+		w->win->watom->disp(w->win->object, 1);
 	w->curx = w->win->curx;
 	w->cury = w->win->cury + w->win->y - w->y;
 
 	/* Generate prompt */
 	w->t->t->updtab[w->y] = 1;
 	genfield(w->t->t,
-	         w->t->t->scrn + w->y * w->t->t->co + w->x,
-	         w->t->t->attr + w->y * w->t->t->co + w->x,
-	         w->x,
-	         w->y,
-	         qw->promptofst,
-	         qw->prompt,
-	         qw->promptlen,
-	         0,
-	         w->w - w->x,
-	         1,
-		 NULL);
+	    w->t->t->scrn + w->y * w->t->t->co + w->x,
+	    w->t->t->attr + w->y * w->t->t->co + w->x,
+	    w->x,
+	    w->y,
+	    qw->promptofst,
+	    qw->prompt,
+	    qw->promptlen,
+	    0,
+	    w->w - w->x,
+	    1,
+	    NULL);
 }
 
 /* When user hits a key in a query window */
 
 struct utf8_sm qw_sm;
 
-static int utypeqw(QW *qw, int c)
+static int utypeqw(jobject jO, int c)
 {
+	QW *qw = jO.qw;
 	W *win;
 	W *w = qw->parent;
 	int *notify = w->notify;
-	int (*func) ();
+	jpoly_int *func;
 	void *object = qw->object;
 
 	if (locale_map->type) {
@@ -103,8 +107,8 @@ static int utypeqw(QW *qw, int c)
 	win = qw->parent->win;
 	func = qw->func;
 	vsrm(qw->prompt);
-	joe_free(qw);
-	w->object = NULL;
+	free(qw);
+	w->object.base = NULL;
 	w->notify = NULL;
 	wabort(w);
 	if (func)
@@ -112,14 +116,15 @@ static int utypeqw(QW *qw, int c)
 	return -1;
 }
 
-static int abortqw(QW *qw)
+static int abortqw(jobject jO)
 {
+	QW *qw = jO.qw;
 	W *win = qw->parent->win;
 	void *object = qw->object;
-	int (*abrt) () = qw->abrt;
+	jpoly_int *abrt = qw->abrt;
 
 	vsrm(qw->prompt);
-	joe_free(qw);
+	free(qw);
 	if (abrt)
 		return abrt(win->object, object);
 	else
@@ -127,7 +132,7 @@ static int abortqw(QW *qw)
 }
 
 static WATOM watomqw = {
-	US "query",
+	UC "query",
 	dispqw,
 	NULL,
 	abortqw,
@@ -141,7 +146,7 @@ static WATOM watomqw = {
 };
 
 static WATOM watqwn = {
-	US "querya",
+	UC "querya",
 	dispqwn,
 	NULL,
 	abortqw,
@@ -155,7 +160,7 @@ static WATOM watqwn = {
 };
 
 static WATOM watqwsr = {
-	US "querysr",
+	UC "querysr",
 	dispqwn,
 	NULL,
 	abortqw,
@@ -170,7 +175,9 @@ static WATOM watqwsr = {
 
 /* Create a query window */
 
-QW *mkqw(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*abrt) (/* ??? */), void *object, int *notify)
+QW *
+mkqw(W *w, const unsigned char *prompt, int len,
+    jpoly_int *func, jpoly_int *abrt, void *object, int *notify)
 {
 	W *new;
 	QW *qw;
@@ -182,7 +189,7 @@ QW *mkqw(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*ab
 		return NULL;
 	}
 	wfit(new->t);
-	new->object = (void *) (qw = (QW *) joe_malloc(sizeof(QW)));
+	new->object.qw = qw = malloc(sizeof(QW));
 	qw->parent = new;
 	qw->prompt = vsncpy(NULL, 0, prompt, len);
 	qw->promptlen = len;
@@ -197,7 +204,9 @@ QW *mkqw(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*ab
 /* Same as above, but cursor is left in original window */
 /* For Ctrl-Meta thing */
 
-QW *mkqwna(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*abrt) (/* ??? */), void *object, int *notify)
+QW *
+mkqwna(W *w, const unsigned char *prompt, int len,
+    jpoly_int *func, jpoly_int *abrt, void *object, int *notify)
 {
 	W *new;
 	QW *qw;
@@ -209,7 +218,7 @@ QW *mkqwna(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*
 		return NULL;
 	}
 	wfit(new->t);
-	new->object = (void *) (qw = (QW *) joe_malloc(sizeof(QW)));
+	new->object.qw = qw = malloc(sizeof(QW));
 	qw->parent = new;
 	qw->prompt = vsncpy(NULL, 0, prompt, len);
 	qw->promptlen = len;
@@ -224,7 +233,9 @@ QW *mkqwna(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*
 /* Same as above, but cursor is left in original window */
 /* For search and replace thing */
 
-QW *mkqwnsr(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (*abrt) (/* ??? */), void *object, int *notify)
+QW *
+mkqwnsr(W *w, const unsigned char *prompt, int len,
+    jpoly_int *func, jpoly_int *abrt, void *object, int *notify)
 {
 	W *new;
 	QW *qw;
@@ -236,7 +247,7 @@ QW *mkqwnsr(W *w, unsigned char *prompt, int len, int (*func) (/* ??? */), int (
 		return NULL;
 	}
 	wfit(new->t);
-	new->object = (void *) (qw = (QW *) joe_malloc(sizeof(QW)));
+	new->object.qw = qw = malloc(sizeof(QW));
 	qw->parent = new;
 	qw->prompt = vsncpy(NULL, 0, prompt, len);
 	qw->promptlen = len;
