@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/regex.c,v 1.5 2012/12/22 00:06:13 tg Exp $ */
 /*
  *	Regular expression subroutines
  *	Copyright
@@ -9,18 +8,20 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h> 
+__RCSID("$MirOS: contrib/code/jupp/regex.c,v 1.10 2017/12/06 23:02:04 tg Exp $");
 
 #include "b.h"
-#include "utf8.h"
 #include "charmap.h"
+#include "regex.h"
+#include "utils.h"
 #include "vs.h"
 
-int escape(int utf8_,unsigned char **a, int *b)
+int
+escape(int isutf8, unsigned char **a, int *b)
 {
 	int c;
 	unsigned char *s = *a;
-	int l = *b;
+	int l = *b, z;
 
 	if (*s == '\\' && l >= 2) {
 		++s; --l;
@@ -69,45 +70,18 @@ int escape(int utf8_,unsigned char **a, int *b)
 		case '5':
 		case '6':
 		case '7':
-			c = *s - '0';
-			++s; --l;
-			if (l > 0 && *s >= '0' && *s <= '7') {
-				c = c * 8 + s[1] - '0';
-				++s; --l;
-			}
-			if (l > 0 && *s >= '0' && *s <= '7') {
-				c = c * 8 + s[1] - '0';
-				++s; --l;
-			}
+			z = ustoc_oct(s, &c, l);
+			s += z;
+			l -= z;
 			break;
 		case 'x':
 		case 'X':
-			c = 0;
-			++s; --l;
-			if (l > 0 && *s >= '0' && *s <= '9') {
-				c = c * 16 + *s - '0';
-				++s; --l;
-			} else if (l > 0 && *s >= 'A' && *s <= 'F') {
-				c = c * 16 + *s - 'A' + 10;
-				++s; --l;
-			} else if (l > 0 && *s >= 'a' && *s <= 'f') {
-				c = c * 16 + *s - 'a' + 10;
-				++s; --l;
-			}
-
-			if (l > 0 && *s >= '0' && *s <= '9') {
-				c = c * 16 + *s - '0';
-				++s; --l;
-			} else if (l > 0 && *s >= 'A' && *s <= 'F') {
-				c = c * 16 + *s - 'A' + 10;
-				++s; --l;
-			} else if (l > 0 && *s >= 'a' && *s <= 'f') {
-				c = c * 16 + *s - 'a' + 10;
-				++s; --l;
-			}
+			z = ustoc_hex(s, &c, l);
+			s += z;
+			l -= z;
 			break;
 		default:
-			if (utf8_)
+			if (isutf8)
 				c = utf8_decode_fwrd(&s, &l);
 			else {
 				c = *s++;
@@ -115,7 +89,7 @@ int escape(int utf8_,unsigned char **a, int *b)
 			}
 			break;
 		}
-	} else if (utf8_) {
+	} else if (isutf8) {
 		c = utf8_decode_fwrd(&s,&l);
 	} else {
 		c = *s++;
@@ -126,7 +100,7 @@ int escape(int utf8_,unsigned char **a, int *b)
 	return c;
 }
 
-static int brack(int utf8_,unsigned char **a, int *la, int c)
+static int brack(int isutf8,unsigned char **a, int *la, int c)
 {
 	int inverse = 0;
 	int flag = 0;
@@ -154,12 +128,12 @@ static int brack(int utf8_,unsigned char **a, int *la, int c)
 		} else {
 			int cl, cr;
 
-			cl = escape(utf8_, &s, &l);
+			cl = escape(isutf8, &s, &l);
 
 			if (l >= 2 && s[0] == '-' && s[1] != ']') {
 				--l;
 				++s;
-				cr = escape(utf8_, &s, &l);
+				cr = escape(isutf8, &s, &l);
 				if (c >= cl && c <= cr)
 					flag = 1;
 			} else if (c == cl)
@@ -173,13 +147,13 @@ static int brack(int utf8_,unsigned char **a, int *la, int c)
 		return flag;
 }
 
-static void savec(int utf8_,unsigned char **pieces, int n, int c)
+static void savec(int isutf8,unsigned char **pieces, int n, int c)
 {
 	unsigned char buf[16];
 	int len;
 	unsigned char *s = NULL;
 
-	if (utf8_)
+	if (isutf8)
 		len = utf8_encode(buf,c);
 	else {
 		buf[0] = c;
@@ -245,7 +219,7 @@ static int skip_special(P *p)
 		goto skip;
 	case '{':
 		to = '}';
-skip:
+ skip:
 		do {
 			s = skip_special(p);
 		} while (s != to && s != NO_MORE_DATA);
@@ -275,14 +249,14 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 	int c, d;
 	P *q = pdup(p);
 	P *o = NULL;
-	int utf8_ = p->b->o.charmap->type;
+	int isutf8 = p->b->o.charmap->type;
 	struct charmap *map = p->b->o.charmap;
 	struct utf8_sm sm;
 
 	utf8_init(&sm);
 
 	while (len) {
-		if (utf8_) {
+		if (isutf8) {
 			do {
 				c = utf8_decode(&sm,*regex++);
 				--len;
@@ -303,7 +277,7 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 				d = pgetc(p);
 				if (d == NO_MORE_DATA)
 					goto fail;
-				savec(utf8_, pieces, n++, d);
+				savec(isutf8, pieces, n++, d);
 				break;
 			case 'n':
 			case 'r':
@@ -326,7 +300,7 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 			case '9':
 				regex -= 2;
 				len += 2;
-				if (pgetc(p) != escape(utf8_, &regex, &len))
+				if (pgetc(p) != escape(isutf8, &regex, &len))
 					goto fail;
 				break;
 			case '*':
@@ -357,9 +331,9 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 				d = pgetc(p);
 				if (d == NO_MORE_DATA)
 					goto fail;
-				if (!brack(utf8_, &regex, &len, d))
+				if (!brack(isutf8, &regex, &len, d))
 					goto fail;
-				savec(utf8_, pieces, n++, d);
+				savec(isutf8, pieces, n++, d);
 				break;
 			case '+':
 				{
@@ -383,13 +357,13 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 						if (regex[1] == '[') {
 							regex += 2;
 							len -= 2;
-							brack(utf8_, &regex, &len, 0);
+							brack(isutf8, &regex, &len, 0);
 						} else {
-							d = escape(utf8_, &regex, &len);
+							d = escape(isutf8, &regex, &len);
 							if (icase)
 								d = joe_tolower(map,d);
 						}
-					} else if (utf8_) {
+					} else if (isutf8) {
 						if ((d = utf8_decode_fwrd(&regex, &len)) < 0)
 							goto done;
 						else if (icase)
@@ -425,7 +399,7 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 							if (oregex[1] == '[') {
 								tregex += 2;
 								tlen -= 2;
-								match = brack(utf8_, &tregex, &tlen, c);
+								match = brack(isutf8, &tregex, &tlen, c);
 							} else
 								match = (d == c);
 						} else {
@@ -435,8 +409,7 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 								match = (c == d);
 						}
 					} while (c != NO_MORE_DATA && match);
-
-				      done:
+ done:
 					if (r) {
 						pset(p, r);
 						prm(r);
@@ -482,13 +455,13 @@ int pmatch(unsigned char **pieces, unsigned char *regex, int len, P *p, int n, i
 			}
 		}
 	}
-succeed:
+ succeed:
 	if (o)
 		prm(o);
 	prm(q);
 	return 1;
 
-fail:
+ fail:
 	if (o)
 		prm(o);
 	pset(p, q);
