@@ -1,6 +1,5 @@
-/* $MirOS: contrib/code/jupp/scrn.c,v 1.26 2017/08/08 16:09:43 tg Exp $ */
 /*
- *	Device independant TTY interface for JOE
+ *	device-independent TTY interface for JOE
  *	Copyright
  *		(C) 1992 Joseph H. Allen
  *
@@ -9,10 +8,9 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h>
-#ifdef HAVE_STDLIB_H
+__RCSID("$MirOS: contrib/code/jupp/scrn.c,v 1.44 2018/01/08 04:07:56 tg Exp $");
+
 #include <stdlib.h>
-#endif
 #include <string.h>
 
 #include "bw.h"
@@ -20,7 +18,6 @@
 #include "scrn.h"
 #include "termcap.h"
 #include "charmap.h"
-#include "utf8.h"
 #include "utils.h"
 
 int skiptop = 0;
@@ -31,104 +28,24 @@ int pastetite = 0;
 int usetabs = 0;
 int assume_color = 0;
 
-/* How to display characters (especially the control ones) */
-/* here are characters ... */
-static const unsigned char xlatc[256] = {
-	 64,  65,  66,  67,  68,  69,  70,  71,			/*   8 */
-	 72,  73,  74,  75,  76,  77,  78,  79,			/*  16 */
-	 80,  81,  82,  83,  84,  85,  86,  87,			/*  24 */
-	 88,  89,  90,  91,  92,  93,  94,  95,			/*  32 */
-	 32,  33,  34,  35,  36,  37,  38,  39,			/*  40 */
-	 40,  41,  42,  43,  44,  45,  46,  47,			/*  48 */
-	 48,  49,  50,  51,  52,  53,  54,  55,			/*  56 */
-	 56,  57,  58,  59,  60,  61,  62,  63,			/*  64 */
+static int
+xlat(int chkasis, int c, int *ap, struct charmap *map)
+{
+	if (joe_isprint(map, c))
+		return (c);
+	if (chkasis && dspasis && c >= 128)
+		return (c);
 
-	 64,  65,  66,  67,  68,  69,  70,  71,			/*  72 */
-	 72,  73,  74,  75,  76,  77,  78,  79,			/*  80 */
-	 80,  81,  82,  83,  84,  85,  86,  87,			/*  88 */
-	 88,  89,  90,  91,  92,  93,  94,  95,			/*  96 */
-	 96,  97,  98,  99, 100, 101, 102, 103,			/* 104 */
-	104, 105, 106, 107, 108, 109, 110, 111,			/* 112 */
-	112, 113, 114, 115, 116, 117, 118, 119,			/* 120 */
-	120, 121, 122, 123, 124, 125, 126,  63,			/* 128 */
-
-	 64,  65,  66,  67,  68,  69,  70,  71,			/* 136 */
-	 72,  73,  74,  75,  76,  77,  78,  79,			/* 144 */
-	 80,  81,  82,  83,  84,  85,  86,  87,			/* 152 */
-	 88,  89,  90,  91,  92,  93,  94,  95,			/* 160 */
-	 32,  33,  34,  35,  36,  37,  38,  39,			/* 168 */
-	 40,  41,  42,  43,  44,  45,  46,  47,			/* 176 */
-	 48,  49,  50,  51,  52,  53,  54,  55,			/* 184 */
-	 56,  57,  58,  59,  60,  61,  62,  63,			/* 192 */
-
-	 64,  65,  66,  67,  68,  69,  70,  71,			/* 200 */
-	 72,  73,  74,  75,  76,  77,  78,  79,			/* 208 */
-	 80,  81,  82,  83,  84,  85,  86,  87,			/* 216 */
-	 88,  89,  90,  91,  92,  93,  94,  95,			/* 224 */
-	 96,  97,  98,  99, 100, 101, 102, 103,			/* 232 */
-	104, 105, 106, 107, 108, 109, 110, 111,			/* 240 */
-	112, 113, 114, 115, 116, 117, 118, 119,			/* 248 */
-	120, 121, 122, 123, 124, 125, 126,  63			/* 256 */
-};
-/* ... and here their attributes */
-static const unsigned short xlata[256] = {
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*   4 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*   8 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  12 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  16 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  20 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  24 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  28 */
-	UNDERLINE, UNDERLINE, UNDERLINE, UNDERLINE,		/*  32 */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  48 */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  64 */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  80 */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/*  96 */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 112 */
-	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, UNDERLINE,	/* 128 */
-
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 130 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 132 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 134 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 136 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 138 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 140 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 142 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 144 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 146 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 148 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 150 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 152 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 154 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 156 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 158 */
-	INVERSE + UNDERLINE, INVERSE + UNDERLINE,		/* 160 */
-
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 164 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 168 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 172 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 176 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 180 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 184 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 188 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 192 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 196 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 200 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 204 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 208 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 212 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 216 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 220 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 224 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 228 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 232 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 236 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 240 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 244 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 248 */
-	INVERSE, INVERSE, INVERSE, INVERSE,			/* 252 */
-	INVERSE, INVERSE, INVERSE, INVERSE + UNDERLINE		/* 256 */
-};
+	if (c & 0x80) {
+		c &= 0x7F;
+		*ap ^= INVERSE;
+	}
+	if (c < 0x20 || c == 0x7F) {
+		c ^= 0x40;
+		*ap ^= UNDERLINE;
+	}
+	return (c);
+}
 
 /* Set attributes */
 
@@ -189,27 +106,28 @@ int set_attr(SCRN *t, int c)
 
 /* Output character with attributes */
 
-void outatr_help(SCRN *t,int *scrn,int *attrf,int xx,int yy,int c,int a)
+void
+outatr_help(SCRN *t, int *scrn, int *attrf, int xx, int yy, int c, int a)
 {
 	/* kludge for help_display() only */
-	if (locale_map->type && !joe_isprint(locale_map,c)) {
-		a ^= xlata[c];
-		c = xlatc[c];
-	}
+	if (locale_map->type)
+		c = xlat(0, c, &a, locale_map);
 	outatr(locale_map, t, scrn, attrf, xx, yy, c, a);
 }
 
-void outatr(struct charmap *map,SCRN *t,int *scrn,int *attrf,int xx,int yy,int c,int a)
+void
+outatr(struct charmap *map, SCRN *t, int *scrn, int *attrf, int xx, int yy, int c, int a)
 {
-	if(map->type)
-		if(locale_map->type) {
-			/* UTF-8 char to UTF-8 terminal */
-			int wid;
+	int wid = 1;
 
+	if (locale_map->type) {
+		/* to UTF-8 terminal */
+		if (map->type) {
+			/* from UTF-8 file */
 			switch ((wid = unictrl(c))) {
 			case 0:
 				/* not a control character */
-				wid = joe_wcwidth(1, c);
+				wid = joe_wcwidth(c);
 				break;
 			case 1:
 				c ^= 0x40;
@@ -218,128 +136,62 @@ void outatr(struct charmap *map,SCRN *t,int *scrn,int *attrf,int xx,int yy,int c
 				a ^= UNDERLINE;
 				break;
 			}
-
-			if(*scrn==c && *attrf==a)
-				return;
-
-			*scrn = c;
-			*attrf = a;
-			if(t->ins)
-				clrins(t);
-			if(t->x != xx || t->y != yy)
-				cpos(t, xx, yy);
-			if(t->attrib != a)
-				set_attr(t, a);
-			if (*unictrlbuf) {
-				ttputs(unictrlbuf);
-			} else {
-				unsigned char buf[7];
-
-				utf8_encode(buf,c);
-				ttputs(buf);
-				if (wid == 0 && xx > 0)
-					attrf[-1] |= HAS_COMBINING;
-			}
-			t->x+=wid;
-			while (wid>1) {
-				*++scrn= -1;
-				*++attrf= 0;
-				--wid;
-			}
 		} else {
-			/* UTF-8 char to non-UTF-8 terminal */
-			/* Don't convert control chars below 256 */
-			if ((c>=32 && c<=126) || c>=160) {
-				if (unictrl(c))
-					a ^= UNDERLINE;
-				c = from_uni(locale_map,c);
-				if (c==-1)
-					c = '?';
-			}
-
-			/* Deal with control characters */
-			if (!joe_isprint(locale_map,c) && !(dspasis && c>=128)) {
-				a ^= xlata[c];
-				c = xlatc[c];
-			}
-
-			if(*scrn==c && *attrf==a)
-				return;
-
-			*scrn = c;
-			*attrf = a;
-			if(t->ins)
-				clrins(t);
-			if(t->x != xx || t->y != yy)
-				cpos(t,xx,yy);
-			if(t->attrib != a)
-				set_attr(t,a);
-			ttputc(c);
-			t->x++;
-		}
-	else
-		if (!locale_map->type) {
-			/* Non UTF-8 char to non UTF-8 terminal */
-			/* Byte-byte Translate? */
-
-			/* Deal with control characters */
-			if (!joe_isprint(locale_map,c) && !(dspasis && c>=128)) {
-				a ^= xlata[c];
-				c = xlatc[c];
-			}
-
-			if (*scrn==c && *attrf==a)
-				return;
-
-			*scrn = c;
-			*attrf = a;
-
-			if(t->ins)
-				clrins(t);
-			if(t->x != xx || t->y != yy)
-				cpos(t,xx,yy);
-			if(t->attrib != a)
-				set_attr(t,a);
-			ttputc(c);
-			t->x++;
-		} else {
-			/* Non UTF-8 char to UTF-8 terminal */
-			unsigned char buf[7];
-			int wid;
-
-			/* Deal with control characters */
-			if (!(dspasis && c>=128) && !joe_isprint(map,c)) {
-				a ^= xlata[c];
-				c = xlatc[c];
-			}
-
-			c = to_uni(map,c);
+			/* from SBCS file */
+			c = to_uni(map, xlat(1, c, &a, map));
 			if (c < 32 || (c >= 0x7F && c < 0xA0)) {
 				c = 0x1000FFFE;
 				a = (a | UNDERLINE) ^ INVERSE;
 			}
-			utf8_encode(buf,c);
+		}
+	} else {
+		/* to SBCS terminal */
+		if (map->type) {
+			/* from UTF-8 file */
 
-			if (*scrn == c && *attrf == a)
-				return;
-
-			wid = joe_wcwidth(0,c);
-			*scrn = c;
-			*attrf = a;
-			if(t->ins)
-				clrins(t);
-			if(t->x != xx || t->y != yy)
-				cpos(t, xx, yy);
-			if(t->attrib != a)
-				set_attr(t, a);
-			ttputs(buf);
-			t->x+=wid;
-			while(wid>1) {
-				*++scrn= -1;
-				*++attrf= 0;
-				--wid;
+			/* don't convert control chars below 256 */
+			if ((c >= 0x20 && c < 0x7F) || c >= 0xA0) {
+				if (unictrl(c))
+					a ^= UNDERLINE;
+				if ((c = from_uni(locale_map, c)) == -1) {
+					c = '!';
+					a |= UNDERLINE;
+				}
 			}
 		}
+		c = xlat(1, c, &a, locale_map);
+	}
+
+	if (*scrn == c && *attrf == a)
+		return;
+
+	*scrn = c;
+	*attrf = a;
+	if (t->ins)
+		clrins(t);
+	if (t->x != xx || t->y != yy)
+		cpos(t, xx, yy);
+	if (t->attrib != a)
+		set_attr(t, a);
+	if (!locale_map->type) {
+		/* SBCS terminal */
+		ttputc(c);
+	} else if (map->type && *unictrlbuf) {
+		/* UTF-8 control char, masked */
+		ttputs(unictrlbuf);
+	} else {
+		unsigned char buf[7];
+
+		utf8_encode(buf, c);
+		ttputs(buf);
+		if (wid == 0 && xx > 0)
+			attrf[-1] |= HAS_COMBINING;
+	}
+	t->x += wid;
+	while (wid-- > 1) {
+		*++scrn = -1;
+		*++attrf = 0;
+	}
 }
 
 /* Set scrolling region */
@@ -444,10 +296,10 @@ SCRN *nopen(CAP *cap)
 	t->cap = cap;
 	setcap(cap, baud, out, NULL);
 
-	t->li = getnum(t->cap,US "li");
+	t->li = getnum(t->cap, UC "li");
 	if (t->li < 1)
 		t->li = 24;
-	t->co = getnum(t->cap,US "co");
+	t->co = getnum(t->cap, UC "co");
 	if (t->co < 2)
 		t->co = 80;
 	x = y = 0;
@@ -457,35 +309,35 @@ SCRN *nopen(CAP *cap)
 		t->co = x;
 	}
 
-	t->haz = getflag(t->cap,US "hz");
-	t->os = getflag(t->cap,US "os");
-	t->eo = getflag(t->cap,US "eo");
-	if (getflag(t->cap,US "hc"))
+	t->haz = getflag(t->cap, UC "hz");
+	t->os = getflag(t->cap, UC "os");
+	t->eo = getflag(t->cap, UC "eo");
+	if (getflag(t->cap, UC "hc"))
 		t->os = 1;
-	if (t->os || getflag(t->cap,US "ul"))
+	if (t->os || getflag(t->cap, UC "ul"))
 		t->ul = 1;
 
-	t->xn = getflag(t->cap,US "xn");
-	t->am = getflag(t->cap,US "am");
+	t->xn = getflag(t->cap, UC "xn");
+	t->am = getflag(t->cap, UC "am");
 
-	t->cl = jgetstr(t->cap,US "cl");
-	t->cd = jgetstr(t->cap,US "cd");
+	t->cl = jgetstr(t->cap, UC "cl");
+	t->cd = jgetstr(t->cap, UC "cd");
 
 	if (!notite) {
-		t->ti = jgetstr(t->cap,US "ti");
-		t->te = jgetstr(t->cap,US "te");
+		t->ti = jgetstr(t->cap, UC "ti");
+		t->te = jgetstr(t->cap, UC "te");
 	}
 	if (pastetite && t->cap->paste_on && t->cap->paste_off) {
 		if (notite) {
-			t->ti = (void *)strdup(t->cap->paste_on);
-			t->te = (void *)strdup(t->cap->paste_off);
+			t->ti = t->cap->paste_on;
+			t->te = t->cap->paste_off;
 		} else {
 			size_t n1, n2;
 			char *cp;
 
 			n1 = t->ti ? strlen(t->ti) : 0;
 			n2 = strlen(t->cap->paste_on);
-			cp = joe_malloc(n1 + n2 + 1);
+			cp = malloc(n1 + n2 + 1);
 			if (t->ti)
 				memcpy(cp, t->ti, n1);
 			memcpy(cp + n1, t->cap->paste_on, n2 + 1);
@@ -493,7 +345,7 @@ SCRN *nopen(CAP *cap)
 
 			n1 = t->te ? strlen(t->te) : 0;
 			n2 = strlen(t->cap->paste_off);
-			cp = joe_malloc(n1 + n2 + 1);
+			cp = malloc(n1 + n2 + 1);
 			memcpy(cp, t->cap->paste_off, n2 + 1);
 			if (t->te)
 				memcpy(cp + n2, t->te, n1 + 1);
@@ -501,123 +353,121 @@ SCRN *nopen(CAP *cap)
 		}
 	}
 
-	t->ut = getflag(t->cap,US "ut");
-	t->Sb = jgetstr(t->cap,US "AB");
-	if (!t->Sb) t->Sb = jgetstr(t->cap,US "Sb");
-	t->Sf = jgetstr(t->cap,US "AF");
-	if (!t->Sf) t->Sf = jgetstr(t->cap,US "Sf");
+	t->ut = getflag(t->cap, UC "ut");
+	t->Sb = jgetstr(t->cap, UC "AB");
+	if (!t->Sb) t->Sb = jgetstr(t->cap, UC "Sb");
+	t->Sf = jgetstr(t->cap, UC "AF");
+	if (!t->Sf) t->Sf = jgetstr(t->cap, UC "Sf");
 
-	if (!(t->me = jgetstr(t->cap,US "me")))
+	if (!(t->me = jgetstr(t->cap, UC "me")))
 		goto oops;
-	if ((t->mb = jgetstr(t->cap,US "mb")))
+	if ((t->mb = jgetstr(t->cap, UC "mb")))
 		t->avattr |= BLINK;
-	if ((t->md = jgetstr(t->cap,US "md")))
+	if ((t->md = jgetstr(t->cap, UC "md")))
 		t->avattr |= BOLD;
-	if ((t->mh = jgetstr(t->cap,US "mh")))
+	if ((t->mh = jgetstr(t->cap, UC "mh")))
 		t->avattr |= DIM;
-	if ((t->mr = jgetstr(t->cap,US "mr")))
+	if ((t->mr = jgetstr(t->cap, UC "mr")))
 		t->avattr |= INVERSE;
  oops:
 
-
-	if (assume_color) {
-		/* Install color support if it looks like an ansi terminal (it has bold which begins with ESC [) */
-#ifndef TERMINFO
-		if (!t->Sf && t->md && t->md[0]=='\\' && t->md[1]=='E' && t->md[2]=='[') {
+	if (assume_color && !t->Sf && t->md) {
+		/*
+		 * Install colour support if this looks like an ANSI
+		 * terminal — that is, it’s got bold with ESC ‘[’…
+		 */
+		if (t->md[0] == '\\' && t->md[1] == 'E' && t->md[2] == '[') {
 			t->ut = 1;
-			t->Sf =US "\\E[3%dm";
-			t->Sb =US "\\E[4%dm";
-		}
-#else
-		if (!t->Sf && t->md && t->md[0]=='\033' && t->md[1]=='[') {
+			t->Sf = UC "\\E[3%dm";
+			t->Sb = UC "\\E[4%dm";
+		} else if (t->md[0] == '\033' && t->md[1] == '[') {
 			t->ut = 1;
-			t->Sf =US "\033[3%p1%dm";
-			t->Sb =US "\033[4%p1%dm";
+			t->Sf = UC "\033[3%p1%dm";
+			t->Sb = UC "\033[4%p1%dm";
 		}
-#endif
 	}
 
-	if (getnum(t->cap,US "sg") <= 0 && !t->mr && jgetstr(t->cap,US "se")) {
-		if ((t->so = jgetstr(t->cap,US "so")) != NULL)
+	if (getnum(t->cap, UC "sg") <= 0 && !t->mr && jgetstr(t->cap, UC "se")) {
+		if ((t->so = jgetstr(t->cap, UC "so")) != NULL)
 			t->avattr |= INVERSE;
-		t->se = jgetstr(t->cap,US "se");
+		t->se = jgetstr(t->cap, UC "se");
 	}
-	if (getflag(t->cap,US "xs") || getflag(t->cap,US "xt"))
+	if (getflag(t->cap, UC "xs") || getflag(t->cap, UC "xt"))
 		t->so = NULL;
 
-	if (getnum(t->cap,US "ug") <= 0 && jgetstr(t->cap,US "ue")) {
-		if ((t->us = jgetstr(t->cap,US "us")) != NULL)
+	if (getnum(t->cap, UC "ug") <= 0 && jgetstr(t->cap, UC "ue")) {
+		if ((t->us = jgetstr(t->cap, UC "us")) != NULL)
 			t->avattr |= UNDERLINE;
-		t->ue = jgetstr(t->cap,US "ue");
+		t->ue = jgetstr(t->cap, UC "ue");
 	}
 
-	if (!(t->uc = jgetstr(t->cap,US "uc")))
+	if (!(t->uc = jgetstr(t->cap, UC "uc")))
 		if (t->ul)
-			t->uc =US "_";
+			t->uc = UC "_";
 	if (t->uc)
 		t->avattr |= UNDERLINE;
 
-	t->ms = getflag(t->cap,US "ms");
+	t->ms = getflag(t->cap, UC "ms");
 
-	t->da = getflag(t->cap,US "da");
-	t->db = getflag(t->cap,US "db");
-	t->cs = jgetstr(t->cap,US "cs");
-	t->rr = getflag(t->cap,US "rr");
-	t->sf = jgetstr(t->cap,US "sf");
-	t->sr = jgetstr(t->cap,US "sr");
-	t->SF = jgetstr(t->cap,US "SF");
-	t->SR = jgetstr(t->cap,US "SR");
-	t->al = jgetstr(t->cap,US "al");
-	t->dl = jgetstr(t->cap,US "dl");
-	t->AL = jgetstr(t->cap,US "AL");
-	t->DL = jgetstr(t->cap,US "DL");
-	if (!getflag(t->cap,US "ns") && !t->sf)
-		t->sf =US "\12";
+	t->da = getflag(t->cap, UC "da");
+	t->db = getflag(t->cap, UC "db");
+	t->cs = jgetstr(t->cap, UC "cs");
+	t->rr = getflag(t->cap, UC "rr");
+	t->sf = jgetstr(t->cap, UC "sf");
+	t->sr = jgetstr(t->cap, UC "sr");
+	t->SF = jgetstr(t->cap, UC "SF");
+	t->SR = jgetstr(t->cap, UC "SR");
+	t->al = jgetstr(t->cap, UC "al");
+	t->dl = jgetstr(t->cap, UC "dl");
+	t->AL = jgetstr(t->cap, UC "AL");
+	t->DL = jgetstr(t->cap, UC "DL");
+	if (!getflag(t->cap, UC "ns") && !t->sf)
+		t->sf = UC "\12";
 
-	if (!getflag(t->cap,US "in") && baud < 38400) {
-		t->dc = jgetstr(t->cap,US "dc");
-		t->DC = jgetstr(t->cap,US "DC");
-		t->dm = jgetstr(t->cap,US "dm");
-		t->ed = jgetstr(t->cap,US "ed");
+	if (!getflag(t->cap, UC "in") && baud < 38400) {
+		t->dc = jgetstr(t->cap, UC "dc");
+		t->DC = jgetstr(t->cap, UC "DC");
+		t->dm = jgetstr(t->cap, UC "dm");
+		t->ed = jgetstr(t->cap, UC "ed");
 
-		t->im = jgetstr(t->cap,US "im");
-		t->ei = jgetstr(t->cap,US "ei");
-		t->ic = jgetstr(t->cap,US "ic");
-		t->IC = jgetstr(t->cap,US "IC");
-		t->ip = jgetstr(t->cap,US "ip");
-		t->mi = getflag(t->cap,US "mi");
+		t->im = jgetstr(t->cap, UC "im");
+		t->ei = jgetstr(t->cap, UC "ei");
+		t->ic = jgetstr(t->cap, UC "ic");
+		t->IC = jgetstr(t->cap, UC "IC");
+		t->ip = jgetstr(t->cap, UC "ip");
+		t->mi = getflag(t->cap, UC "mi");
 	}
 
-	if (jgetstr(t->cap,US "bc"))
-		t->bs = jgetstr(t->cap,US "bc");
-	else if (jgetstr(t->cap,US "le"))
-		t->bs = jgetstr(t->cap,US "le");
-	if (getflag(t->cap,US "bs"))
-		t->bs =US "\10";
+	if (jgetstr(t->cap, UC "bc"))
+		t->bs = jgetstr(t->cap, UC "bc");
+	else if (jgetstr(t->cap, UC "le"))
+		t->bs = jgetstr(t->cap, UC "le");
+	if (getflag(t->cap, UC "bs"))
+		t->bs = UC "\10";
 
 	t->cbs = tcost(t->cap, t->bs, 1, 2, 2, 0, 0);
 
-	t->lf =US "\12";
-	if (jgetstr(t->cap,US "do"))
-		t->lf = jgetstr(t->cap,US "do");
+	t->lf = UC "\12";
+	if (jgetstr(t->cap, UC "do"))
+		t->lf = jgetstr(t->cap, UC "do");
 	t->clf = tcost(t->cap, t->lf, 1, 2, 2, 0, 0);
 
-	t->up = jgetstr(t->cap,US "up");
+	t->up = jgetstr(t->cap, UC "up");
 	t->cup = tcost(t->cap, t->up, 1, 2, 2, 0, 0);
 
-	t->nd = jgetstr(t->cap,US "nd");
+	t->nd = jgetstr(t->cap, UC "nd");
 
 	t->tw = 8;
-	if (getnum(t->cap,US "it") > 0)
-		t->tw = getnum(t->cap,US "it");
-	else if (getnum(t->cap,US "tw") > 0)
-		t->tw = getnum(t->cap,US "tw");
+	if (getnum(t->cap, UC "it") > 0)
+		t->tw = getnum(t->cap, UC "it");
+	else if (getnum(t->cap, UC "tw") > 0)
+		t->tw = getnum(t->cap, UC "tw");
 
-	if (!(t->ta = jgetstr(t->cap,US "ta")))
-		if (getflag(t->cap,US "pt"))
-			t->ta =US "\11";
-	t->bt = jgetstr(t->cap,US "bt");
-	if (getflag(t->cap,US "xt") || !usetabs) {
+	if (!(t->ta = jgetstr(t->cap, UC "ta")))
+		if (getflag(t->cap, UC "pt"))
+			t->ta = UC "\11";
+	t->bt = jgetstr(t->cap, UC "bt");
+	if (getflag(t->cap, UC "xt") || !usetabs) {
 		t->ta = NULL;
 		t->bt = NULL;
 	}
@@ -625,30 +475,30 @@ SCRN *nopen(CAP *cap)
 	t->cta = tcost(t->cap, t->ta, 1, 2, 2, 0, 0);
 	t->cbt = tcost(t->cap, t->bt, 1, 2, 2, 0, 0);
 
-	t->ho = jgetstr(t->cap,US "ho");
+	t->ho = jgetstr(t->cap, UC "ho");
 	t->cho = tcost(t->cap, t->ho, 1, 2, 2, 0, 0);
-	t->ll = jgetstr(t->cap,US "ll");
+	t->ll = jgetstr(t->cap, UC "ll");
 	t->cll = tcost(t->cap, t->ll, 1, 2, 2, 0, 0);
 
-	t->cr =US "\15";
-	if (jgetstr(t->cap,US "cr"))
-		t->cr = jgetstr(t->cap,US "cr");
-	if (getflag(t->cap,US "nc") || getflag(t->cap,US "xr"))
+	t->cr = UC "\15";
+	if (jgetstr(t->cap, UC "cr"))
+		t->cr = jgetstr(t->cap, UC "cr");
+	if (getflag(t->cap, UC "nc") || getflag(t->cap, UC "xr"))
 		t->cr = NULL;
 	t->ccr = tcost(t->cap, t->cr, 1, 2, 2, 0, 0);
 
-	t->cRI = tcost(t->cap, t->RI = jgetstr(t->cap,US "RI"), 1, 2, 2, 0, 0);
-	t->cLE = tcost(t->cap, t->LE = jgetstr(t->cap,US "LE"), 1, 2, 2, 0, 0);
-	t->cUP = tcost(t->cap, t->UP = jgetstr(t->cap,US "UP"), 1, 2, 2, 0, 0);
-	t->cDO = tcost(t->cap, t->DO = jgetstr(t->cap,US "DO"), 1, 2, 2, 0, 0);
-	t->cch = tcost(t->cap, t->ch = jgetstr(t->cap,US "ch"), 1, 2, 2, 0, 0);
-	t->ccv = tcost(t->cap, t->cv = jgetstr(t->cap,US "cv"), 1, 2, 2, 0, 0);
-	t->ccV = tcost(t->cap, t->cV = jgetstr(t->cap,US "cV"), 1, 2, 2, 0, 0);
-	t->ccm = tcost(t->cap, t->cm = jgetstr(t->cap,US "cm"), 1, 2, 2, 0, 0);
+	t->cRI = tcost(t->cap, t->RI = jgetstr(t->cap, UC "RI"), 1, 2, 2, 0, 0);
+	t->cLE = tcost(t->cap, t->LE = jgetstr(t->cap, UC "LE"), 1, 2, 2, 0, 0);
+	t->cUP = tcost(t->cap, t->UP = jgetstr(t->cap, UC "UP"), 1, 2, 2, 0, 0);
+	t->cDO = tcost(t->cap, t->DO = jgetstr(t->cap, UC "DO"), 1, 2, 2, 0, 0);
+	t->cch = tcost(t->cap, t->ch = jgetstr(t->cap, UC "ch"), 1, 2, 2, 0, 0);
+	t->ccv = tcost(t->cap, t->cv = jgetstr(t->cap, UC "cv"), 1, 2, 2, 0, 0);
+	t->ccV = tcost(t->cap, t->cV = jgetstr(t->cap, UC "cV"), 1, 2, 2, 0, 0);
+	t->ccm = tcost(t->cap, t->cm = jgetstr(t->cap, UC "cm"), 1, 2, 2, 0, 0);
 
-	t->cce = tcost(t->cap, t->ce = jgetstr(t->cap,US "ce"), 1, 2, 2, 0, 0);
+	t->cce = tcost(t->cap, t->ce = jgetstr(t->cap, UC "ce"), 1, 2, 2, 0, 0);
 
-/* Make sure terminal can do absolute positioning */
+	/* Make sure terminal can do absolute positioning */
 	if (t->cm)
 		goto ok;
 	if (t->ch && t->cv)
@@ -661,40 +511,36 @@ SCRN *nopen(CAP *cap)
 		goto ok;
 	leave = 1;
 	ttclose();
-	signrm();
-#ifdef	DEBUG
-	/* these are strings, but I do not know if %s is appropriate -mirabilos */
-        fprintf(stderr,"cm=%d ch=%d cv=%d ho=%d lf=%d DO=%d ll=%d up=%d UP=%d cr=%d\n",
-                       t->cm, t->ch, t->cv, t->ho, t->lf, t->DO, t->ll, t->up, t->UP, t->cr);
-#endif
+	signrm(0);
+	free(t);
 	fprintf(stderr,"Sorry, your terminal can't do absolute cursor positioning.\nIt's broken\n");
 	return NULL;
  ok:
 
-/* Determine if we can scroll */
+	/* Determine if we can scroll */
 	if (((t->sr || t->SR) && (t->sf || t->SF) && t->cs) || ((t->al || t->AL) && (t->dl || t->DL)))
 		t->scroll = 1;
 	else if (baud < 38400)
 		mid = 1;
 
-/* Determine if we can ins/del within lines */
+	/* Determine if we can ins/del within lines */
 	if ((t->im || t->ic || t->IC) && (t->dc || t->DC))
 		t->insdel = 1;
 
-/* Adjust for high baud rates */
+	/* Adjust for high baud rates */
 	if (baud >= 38400) {
 		t->scroll = 0;
 		t->insdel = 0;
 	}
 
-/* Send out terminal initialization string */
+	/* Send out terminal initialisation string */
 	if (t->ti)
 		texec(t->cap, t->ti, 1, 0, 0, 0, 0);
 	if (!skiptop && t->cl)
 		texec(t->cap, t->cl, 1, 0, 0, 0, 0);
 
-/* Initialize variable screen size dependant vars */
-	t->htab = calloc(256, sizeof(struct hentry));
+	/* Initialise variable screen size-dependent vars */
+	t->htab = ralloc(256, sizeof(struct s_hentry));
 
 	nresize(t, t->co, t->li);
 
@@ -711,37 +557,49 @@ void nresize(SCRN *t, int w, int h)
 		w = 8;
 	t->li = h;
 	t->co = w;
+	if (notoktomul((size_t)t->li, (size_t)t->co)) {
+		/* who has THAT large screens? */
+		ttabrt(0, "screen too large");
+		exit(255);
+	}
 	if (t->sary)
-		joe_free(t->sary);
+		free(t->sary);
 	if (t->updtab)
-		joe_free(t->updtab);
+		free(t->updtab);
 	if (t->syntab)
-		joe_free(t->syntab);
+		free(t->syntab);
 	if (t->scrn)
-		joe_free(t->scrn);
+		free(t->scrn);
 	if (t->attr)
-		joe_free(t->attr);
+		free(t->attr);
 	if (t->compose)
-		joe_free(t->compose);
+		free(t->compose);
 	if (t->ofst)
-		joe_free(t->ofst);
+		free(t->ofst);
 	if (t->ary)
-		joe_free(t->ary);
-	t->scrn = calloc(t->li * t->co, sizeof(int));
-	t->attr = calloc(t->li * t->co, sizeof(int));
+		free(t->ary);
+	t->scrn = ralloc((size_t)t->li * (size_t)t->co, sizeof(int));
+	t->attr = ralloc((size_t)t->li * (size_t)t->co, sizeof(int));
 	t->sary = calloc(t->li, sizeof(int));
-	t->updtab = calloc(t->li, sizeof(int));
-	t->syntab = calloc(t->li, sizeof(int));
-	t->compose = calloc(t->co, sizeof(int));
-	t->ofst = calloc(t->co, sizeof(int));
-	t->ary = calloc(t->co, sizeof(struct hentry));
+	t->updtab = ralloc((size_t)t->li, sizeof(int));
+	t->syntab = ralloc((size_t)t->li, sizeof(int));
+	t->compose = ralloc((size_t)t->co, sizeof(int));
+	t->ofst = ralloc((size_t)t->co, sizeof(int));
+	t->ary = ralloc((size_t)t->co, sizeof(struct s_hentry));
+
+	if (!t->htab || !t->scrn || !t->attr || !t->sary || !t->updtab ||
+	    !t->syntab || !t->compose || !t->ofst || !t->ary) {
+		ttabrt(0, "screen allocation failed");
+		exit(255);
+	}
 
 	nredraw(t);
 }
 
-/* Calculate cost of positioning the cursor using only relative cursor
- * positioning functions: t->(lf, DO, up, UP, bs, LE, RI, ta, bt) and rewriting
- * characters (to move right)
+/*
+ * Calculate cost of positioning the cursor using only relative cursor
+ * positioning functions: t->(lf, DO, up, UP, bs, LE, RI, ta, bt) and
+ * rewriting characters (to move right)
  *
  * This doesn't use the am and bw capabilities although it probably could.
  */
@@ -750,11 +608,11 @@ static int relcost(register SCRN *t, register int x, register int y, register in
 {
 	int cost = 0;
 
-/* If we don't know the cursor position, force use of absolute positioning */
+	/* If we don't know the cursor position, force use of absolute positioning */
 	if (oy == -1 || ox == -1)
 		return 10000;
 
-/* First adjust row */
+	/* First adjust row */
 	if (y > oy) {
 		int dist = y - oy;
 
@@ -797,9 +655,9 @@ static int relcost(register SCRN *t, register int x, register int y, register in
 			return 10000;
 	}
 
-/* Now adjust column */
+	/* Now adjust column */
 
-/* Use tabs */
+	/* Use tabs */
 	if (x > ox && t->ta) {
 		int dist = x - ox;
 		int ntabs = (dist + ox % t->tw) / t->tw;
@@ -843,7 +701,7 @@ static int relcost(register SCRN *t, register int x, register int y, register in
 			return cost + cstover;
 	}
 
-/* Use simple motions */
+	/* Use simple motions */
 	if (x < ox) {
 		int dist = ox - x;
 
@@ -889,9 +747,10 @@ static void cposs(register SCRN *t, register int x, register int y)
 	int hy;
 	int hl;
 
-/* Home y position is usually 0, but it is 'top' if we have scrolling region
- * relative addressing
- */
+	/*
+	 * Home y position is usually 0, but it is 'top' if we have
+	 * scrolling region relative addressing
+	 */
 	if (t->rr) {
 		hy = t->top;
 		hl = t->bot - 1;
@@ -900,14 +759,16 @@ static void cposs(register SCRN *t, register int x, register int y)
 		hl = t->li - 1;
 	}
 
-/* Assume best way is with only using relative cursor positioning */
+	/* Assume best way is with only using relative cursor positioning */
 
 	bestcost = relcost(t, x, y, t->x, t->y);
 	bestway = 0;
 
-/* Now check if combinations of absolute cursor positioning functions are
- * better (or necessary in case one or both cursor positions are unknown)
- */
+	/*
+	 * Now check if combinations of absolute cursor positioning
+	 * functions are better (or necessary in case one or both cursor
+	 * positions are unknown)
+	 */
 
 	if (t->ccm < bestcost) {
 		cost = tcost(t->cap, t->cm, 1, y, x, 0, 0);
@@ -1001,9 +862,11 @@ static void cposs(register SCRN *t, register int x, register int y)
 		}
 	}
 
-/* Do absolute cursor positioning if we don't know the cursor position or
- * if it is faster than doing only relative cursor positioning
- */
+	/*
+	 * Do absolute cursor positioning if we don't know the cursor
+	 * position or if it is faster than doing only relative cursor
+	 * positioning
+	 */
 
 	switch (bestway) {
 	case 1:
@@ -1074,9 +937,9 @@ static void cposs(register SCRN *t, register int x, register int y)
 		break;
 	}
 
-/* Use relative cursor position functions if we're not there yet */
+	/* Use relative cursor position functions if we're not there yet */
 
-/* First adjust row */
+	/* First adjust row */
 	if (y > t->y) {
 		/* Have to go down */
 		if (!t->lf || t->cDO < (y - t->y) * t->clf) {
@@ -1099,7 +962,7 @@ static void cposs(register SCRN *t, register int x, register int y)
 			}
 	}
 
-/* Use tabs */
+	/* Use tabs */
 	if (x > t->x && t->ta) {
 		int ntabs = (x - t->x + t->x % t->tw) / t->tw;
 		int cstunder = x % t->tw + t->cta * ntabs;
@@ -1152,7 +1015,7 @@ static void cposs(register SCRN *t, register int x, register int y)
 		}
 	}
 
-/* Now adjust column */
+	/* Now adjust column */
 	if (x < t->x) {
 		/* Have to go left */
 		if (!t->bs || t->cLE < (t->x - x) * t->cbs) {
@@ -1175,28 +1038,6 @@ static void cposs(register SCRN *t, register int x, register int y)
 				++t->x;
 			}
 		}
-
-		/* if (t->cRI < x - t->x) { */
-/*		} else {
-			int *s = t->scrn + t->x + t->y * t->co;
-			int *a = t->attr + t->x + t->y * t->co;
-
-			if (t->ins)
-				clrins(t);
-			while (x > t->x) {
-				int atr, c;
-				if(*s==-1) c=' ', atr=0;
-				else c= *s, atr= *a;
-
-				if (atr != t->attrib)
-					set_attr(t, atr);
-				utf8_putc(c);
-				++s;
-				++a;
-				++t->x;
-			}
-		}
-*/
 	}
 }
 
@@ -1305,18 +1146,18 @@ static void dodelchr(SCRN *t, int x, int y, int n)
 
 void magic(SCRN *t, int y, int *cs, int *ca,int *s, int *a, int placex)
 {
-	struct hentry *htab = t->htab;
+	struct s_hentry *htab = t->htab;
 	int *ofst = t->ofst;
 	int aryx = 1;
 	int x;
 
 	if (!(t->im || t->ic || t->IC) || !(t->dc || t->DC))
 		return;
-	mset(htab, 0, 256 * sizeof(struct hentry));
+	mset(htab, 0, 256 * sizeof(struct s_hentry));
 
 	msetI(ofst, 0, t->co);
 
-/* Build hash table */
+	/* Build hash table */
 	for (x = 0; x != t->co - 1; ++x) {
 		t->ary[aryx].next = htab[cs[x] & 255].next;
 		t->ary[aryx].loc = x;
@@ -1324,7 +1165,7 @@ void magic(SCRN *t, int y, int *cs, int *ca,int *s, int *a, int placex)
 		htab[cs[x] & 255].next = aryx++;
 	}
 
-/* Build offset table */
+	/* Build offset table */
 	for (x = 0; x < t->co - 1;)
 		if (htab[s[x] & 255].loc >= 15)
 			ofst[x++] = t->co - 1;
@@ -1376,7 +1217,7 @@ void magic(SCRN *t, int y, int *cs, int *ca,int *s, int *a, int placex)
 			x += maxlen;
 		}
 
-/* Apply scrolling commands */
+	/* Apply scrolling commands */
 
 	for (x = 0; x != t->co - 1; ++x) {
 		int q = ofst[x];
@@ -1560,7 +1401,7 @@ void nscroll(SCRN *t)
 
 	for (y = 0; y != t->li; ++y) {
 		q = t->sary[y];
-		if (ifhave)
+		if (have)
 			return;
 		if (q && q != t->li) {
 			if (q > 0) {
@@ -1624,13 +1465,13 @@ void nclose(SCRN *t)
 		texec(t->cap, t->te, 1, 0, 0, 0, 0);
 	ttclose();
 	rmcap(t->cap);
-	joe_free(t->scrn);
-	joe_free(t->attr);
-	joe_free(t->sary);
-	joe_free(t->ofst);
-	joe_free(t->htab);
-	joe_free(t->ary);
-	joe_free(t);
+	free(t->scrn);
+	free(t->attr);
+	free(t->sary);
+	free(t->ofst);
+	free(t->htab);
+	free(t->ary);
+	free(t);
 }
 
 void nscrldn(SCRN *t, int top, int bot, int amnt)
@@ -1807,7 +1648,7 @@ void genfield(SCRN *t,int *scrn,int *attr,int x,int y,int ofst,unsigned char *s,
 			/* UTF-8 mode: decode character and determine its width */
 			c = utf8_decode(&sm,c);
 			if (c >= 0)
-				wid = joe_wcwidth(1,c);
+				wid = joe_wcwidth(c);
 		} else {
 			/* Byte mode: character is one column wide */
 			wid = 1;
@@ -1869,7 +1710,7 @@ int txtwidth(unsigned char *s,int len)
 		while(len--) {
 			int d = utf8_decode(&sm,*s++);
 			if (d >= 0)
-				col += joe_wcwidth(1,d);
+				col += joe_wcwidth(d);
 		}
 
 		return col;
@@ -1892,32 +1733,26 @@ void genfmt(SCRN *t, int x, int y, int ofst, const unsigned char *s, int flg)
 
 	while ((c = *s++) != '\0')
 		if (c == '\\') {
-			switch (c = *s++) {
+			switch ((c = *s++) | 0x20) {
 			case 'u':
-			case 'U':
 				atr ^= UNDERLINE;
 				break;
 			case 'i':
-			case 'I':
 				atr ^= INVERSE;
 				break;
 			case 'b':
-			case 'B':
 				atr ^= BOLD;
 				break;
 			case 'd':
-			case 'D':
 				atr ^= DIM;
 				break;
 			case 'f':
-			case 'F':
 				atr ^= BLINK;
 				break;
-			case 0:
-				--s;
-				break;
 			default: {
-				if (col++ >= ofst) {
+				if (!c)
+					--s;
+				else if (col++ >= ofst) {
 					outatr(locale_map, t, scrn, attr, x, y, (c&0x7F), atr);
 					++scrn;
 					++attr;
@@ -1932,7 +1767,7 @@ void genfmt(SCRN *t, int x, int y, int ofst, const unsigned char *s, int flg)
 				/* UTF-8 mode: decode character and determine its width */
 				c = utf8_decode(&sm,c);
 				if (c >= 0) {
-					wid = joe_wcwidth(1, c);
+					wid = joe_wcwidth(c);
 				}
 			} else {
 				/* Byte mode: character is one column wide */
@@ -2002,7 +1837,7 @@ int fmtlen(const unsigned char *s)
 			if(locale_map->type) {
 				c = utf8_decode(&sm,c);
 				if (c>=0)
-					wid = joe_wcwidth(1,c);
+					wid = joe_wcwidth(c);
 			} else {
 				wid = 1;
 			}
@@ -2053,7 +1888,7 @@ int fmtpos(unsigned char *s, int goal)
 			if(locale_map->type) {
 				c = utf8_decode(&sm,c);
 				if (c>=0)
-					wid = joe_wcwidth(1,c);
+					wid = joe_wcwidth(c);
 			} else {
 				wid = 1;
 			}
