@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/tab.h,v 1.2 2008/05/13 13:08:26 tg Exp $ */
 /*
  *	File selection menu
  *	Copyright
@@ -9,9 +8,10 @@
 #ifndef _JOE_TAB_H
 #define _JOE_TAB_H 1
 
-#include "config.h"
-#include "types.h"
+#ifdef EXTERN_UFILE_C
+__IDSTRING(rcsid_tab_h, "$MirOS: contrib/code/jupp/tab.h,v 1.5 2017/12/06 21:17:00 tg Exp $");
+#endif
 
-int cmplt PARAMS((BW *bw));
+int cmplt(BW *bw);
 
 #endif
