@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/termcap.c,v 1.14 2017/08/08 16:12:04 tg Exp $ */
 /*
  *	TERMCAP/TERMINFO database interface
  *	Copyright
@@ -9,20 +8,10 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h>
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
+__RCSID("$MirOS: contrib/code/jupp/termcap.c,v 1.25 2017/12/20 22:22:45 tg Exp $");
+
 #include <sys/stat.h>
-#endif
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
-#ifdef TERMINFO
-#ifdef HAVE_TERM_H
-#endif
-#endif
 
 #include "blocks.h"
 #include "path.h"
@@ -50,7 +39,8 @@ static const unsigned char defentry[] = "\
 
 /* Return true if termcap line matches name */
 
-static int match(unsigned char *s, unsigned char *name)
+static int
+match(const unsigned char *s, const unsigned char *name)
 {
 	if (s[0] == 0 || s[0] == '#')
 		return 0;
@@ -69,13 +59,14 @@ static int match(unsigned char *s, unsigned char *name)
 
 /* Find termcap entry in a file */
 
-static unsigned char *lfind(unsigned char *s, int pos, FILE *fd, unsigned char *name)
+static unsigned char *
+lfind(unsigned char *s, int pos, FILE *fd, const unsigned char *name)
 {
 	int c, x;
 
 	if (!s)
 		s = vsmk(1024);
-      loop:
+ loop:
 	while (c = getc(fd), c == ' ' || c == '\t' || c == '#')
 		do {
 			c = getc(fd);
@@ -98,7 +89,7 @@ static unsigned char *lfind(unsigned char *s, int pos, FILE *fd, unsigned char *
 			else
 				return vstrunc(s, x);
 		else if (c == '\r')
-			/* do nothing */;
+			/* nothing */;
 		else {
 			s = vsset(s, x, c);
 			++x;
@@ -111,7 +102,7 @@ static unsigned char *lfind(unsigned char *s, int pos, FILE *fd, unsigned char *
 			else
 				break;
 		else if (c == '\r')
-			/* do nothing */;
+			/* nothing */;
 		else {
 			s = vsset(s, x, c);
 			++x;
@@ -122,23 +113,22 @@ static unsigned char *lfind(unsigned char *s, int pos, FILE *fd, unsigned char *
 
 /* Lookup termcap entry in index */
 
-static off_t findidx(FILE *file, unsigned char *name)
+static off_t
+findidx(FILE *file, const unsigned char *name)
 {
 	unsigned char buf[80];
 	off_t addr = 0;
 
 	while (fgets((char *)buf, 80, file)) {
-		int x = 0, flg = 0, c, y, z;
+		int x = 0, flg = 0, c, y;
 
 		do {
 			for (y = x; buf[y] && buf[y] != ' ' && buf[y] != '\n'; ++y) ;
 			c = buf[y];
 			buf[y] = 0;
-			if (c == '\n' || !c) {
-				z = 0;
-				sscanf((char *)(buf + x), "%x", &z);
-				addr += z;
-			} else if (!strcmp(buf + x, name))
+			if (c == '\n' || !c)
+				addr += ustol(buf + x, NULL, USTOL_HEX);
+			else if (!strcmp(buf + x, name))
 				flg = 1;
 			x = y + 1;
 		} while (c && c != '\n');
@@ -161,7 +151,7 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 
 	if (!name && !(name = joeterm) && !(name = (unsigned char *)getenv("TERM")))
 		return NULL;
-	cap = (CAP *) joe_malloc(sizeof(CAP));
+	cap = malloc(sizeof(CAP));
 	cap->tbuf = vsmk(4096);
 	cap->abuf = NULL;
 	cap->sort = NULL;
@@ -174,18 +164,18 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 	}
 
 #ifdef TERMINFO
-	cap->abuf = (unsigned char *) joe_malloc(4096);
+	cap->abuf = malloc(4096);
 	cap->abufp = cap->abuf;
 	if (tgetent((char *)cap->tbuf, (char *)name) == 1)
 		return setcap(cap, baud, out, outptr);
 	else {
-		joe_free(cap->abuf);
+		free(cap->abuf);
 		cap->abuf = NULL;
 	}
 #endif
 
 	name = vsncpy(NULL, 0, sz(name));
-	cap->sort = (struct sortentry *) joe_malloc(sizeof(struct sortentry) * (sortsiz = 64));
+	cap->sort = malloc(sizeof(struct sortentry) * (sortsiz = 64));
 
 	cap->sortlen = 0;
 
@@ -225,16 +215,8 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 
 	cap->tbuf = vstrunc(cap->tbuf, 0);
 
-      nextfile:
+ nextfile:
 	if (!npbuf[y]) {
-/*
- varm(npbuf);
- vsrm(name);
- vsrm(cap->tbuf);
- joe_free(cap->sort);
- joe_free(cap);
- return 0;
-*/
 		fprintf(stderr, "Couldn't load termcap entry.  Using ansi default\n");
 		ti = 0;
 		cap->tbuf = vsncpy(cap->tbuf, 0, sc(defentry));
@@ -271,12 +253,12 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 	if (sLEN(cap->tbuf) == ti)
 		goto nextfile;
 
-      checktc:
+ checktc:
 	x = sLEN(cap->tbuf);
 	do {
 		cap->tbuf[x] = 0;
 		while (x && cap->tbuf[--x] != ':')
-			/* do nothing */;
+			/* nothing */;
 	} while (x && (!cap->tbuf[x + 1] || cap->tbuf[x + 1] == ':'));
 
 	if (cap->tbuf[x + 1] == 't' && cap->tbuf[x + 2] == 'c' && cap->tbuf[x + 3] == '=') {
@@ -290,19 +272,19 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 		goto nextfile;
 	}
 
-      doline:
+ doline:
 	pp = cap->tbuf + ti;
 
-/* Process line at pp */
+	/* Process line at pp */
 
-      loop:
+ loop:
 	while (*pp && *pp != ':')
 		++pp;
 	if (*pp) {
 		int q;
 
 		*pp++ = 0;
-	      loop1:
+ loop1:
 		if (pp[0] == ' ' || pp[0] == '\t')
 			goto loop;
 		for (q = 0; pp[q] && pp[q] != '#' && pp[q] != '=' && pp[q] != '@' && pp[q] != ':'; ++q) ;
@@ -317,10 +299,8 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 		x = 0;
 		y = cap->sortlen;
 		z = -1;
-		if (!y) {
-			/* dead store: z = 0; */
+		if (!y)
 			goto in;
-		}
 		while (z != (x + y) / 2) {
 			int found;
 
@@ -344,9 +324,9 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 					goto loop;
 			}
 		}
-	      in:
+ in:
 		if (cap->sortlen == sortsiz)
-			cap->sort = (struct sortentry *) joe_realloc(cap->sort, (sortsiz += 32) * sizeof(struct sortentry));
+			cap->sort = realloc(cap->sort, (sortsiz += 32) * sizeof(struct sortentry));
 		mmove(cap->sort + y + 1, cap->sort + y, (cap->sortlen++ - y) * sizeof(struct sortentry));
 
 		cap->sort[y].name = qq;
@@ -370,7 +350,7 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 	varm(npbuf);
 	vsrm(name);
 
-	cap->pad = jgetstr(cap, US "pc");
+	cap->pad = jgetstr(cap, UC "pc");
 	if (dopadding)
 		cap->dopadding = 1;
 	else
@@ -383,7 +363,8 @@ CAP *getcap(unsigned char *name, unsigned int baud, void (*out) (unsigned char *
 	return setcap(cap, baud, out, outptr);
 }
 
-static struct sortentry *findcap(CAP *cap, unsigned char *name)
+static struct sortentry *
+findcap(CAP *cap, const unsigned char *name)
 {
 	int x, y, z;
 	int found;
@@ -413,7 +394,8 @@ CAP *setcap(CAP *cap, unsigned int baud, void (*out) (unsigned char *, unsigned
 	return cap;
 }
 
-int getflag(CAP *cap, unsigned char *name)
+int
+getflag(CAP *cap, const unsigned char *name)
 {
 #ifdef TERMINFO
 	if (cap->abuf)
@@ -422,13 +404,15 @@ int getflag(CAP *cap, unsigned char *name)
 	return findcap(cap, name) != NULL;
 }
 
-unsigned char *jgetstr(CAP *cap, unsigned char *name)
+const unsigned char *
+jgetstr(CAP *cap, const unsigned char *name)
 {
 	struct sortentry *s;
 
 #ifdef TERMINFO
 	if (cap->abuf)
-		return (unsigned char *)tgetstr((char *)name, (char **)&cap->abufp);
+		return (const unsigned char *)tgetstr((char *)name,
+		    (char **)&cap->abufp);
 #endif
 	s = findcap(cap, name);
 	if (s)
@@ -437,7 +421,8 @@ unsigned char *jgetstr(CAP *cap, unsigned char *name)
 		return NULL;
 }
 
-int getnum(CAP *cap, unsigned char *name)
+int
+getnum(CAP *cap, const unsigned char *name)
 {
 	struct sortentry *s;
 
@@ -447,7 +432,7 @@ int getnum(CAP *cap, unsigned char *name)
 #endif
 	s = findcap(cap, name);
 	if (s && s->value)
-		return atoi((char *)(s->value));
+		return atoi((const char *)(s->value));
 	return -1;
 }
 
@@ -455,15 +440,17 @@ void rmcap(CAP *cap)
 {
 	vsrm(cap->tbuf);
 	if (cap->abuf)
-		joe_free(cap->abuf);
+		free(cap->abuf);
 	if (cap->sort)
-		joe_free(cap->sort);
-	joe_free(cap);
+		free(cap->sort);
+	free(cap);
 }
 
-static unsigned char escape(unsigned char **s)
+static unsigned char
+escape(const unsigned char **s)
 {
 	unsigned char c = *(*s)++;
+	int i;
 
 	if (c == '^' && **s)
 		if (**s != '?')
@@ -482,12 +469,9 @@ static unsigned char escape(unsigned char **s)
 		case '5':
 		case '6':
 		case '7':
-			c -= '0';
-			if (**s >= '0' && **s <= '7')
-				c = (c << 3) + *((*s)++) - '0';
-			if (**s >= '0' && **s <= '7')
-				c = (c << 3) + *((*s)++) - '0';
-			return c;
+			(*s)--;
+			*s += ustoc_oct(*s, &i, USTOC_MAX);
+			return i;
 		case 'e':
 		case 'E':
 			return 27;
@@ -519,14 +503,15 @@ static int outout(int c)
 }
 #endif
 
-void texec(CAP *cap, unsigned char *s, int l, int a0, int a1, int a2, int a3)
+void
+texec(CAP *cap, const unsigned char *s, int l, int a0, int a1, int a2, int a3)
 {
 	int c, tenth = 0, x;
 	int args[4];
-	int vars[128];
 	int *a = args;
+	int *vars = NULL;
 
-/* Do nothing if there is no string */
+	/* Do nothing if there is no string */
 	if (!s)
 		return;
 
@@ -535,19 +520,19 @@ void texec(CAP *cap, unsigned char *s, int l, int a0, int a1, int a2, int a3)
 		unsigned char *aa;
 
 		outcap = cap;
-		aa = (unsigned char *)tgoto((char *)s, a1, a0);
+		aa = (unsigned char *)tgoto((const char *)s, a1, a0);
 		tputs((char *)aa, l, outout);
 		return;
 	}
 #endif
 
-/* Copy args into array (yuk) */
+	/* Copy args into array (yuk) */
 	args[0] = a0;
 	args[1] = a1;
 	args[2] = a2;
 	args[3] = a3;
 
-/* Get tenths of MS of padding needed */
+	/* Get tenths of MS of padding needed */
 	while (*s >= '0' && *s <= '9')
 		tenth = tenth * 10 + *s++ - '0';
 	tenth *= 10;
@@ -556,13 +541,13 @@ void texec(CAP *cap, unsigned char *s, int l, int a0, int a1, int a2, int a3)
 		tenth += *s++ - '0';
 	}
 
-/* Check if we have to multiply by number of lines */
+	/* Check if we have to multiply by number of lines */
 	if (*s == '*') {
 		++s;
 		tenth *= l;
 	}
 
-/* Output string */
+	/* Output string */
 	while ((c = *s++) != '\0')
 		if (c == '%' && *s) {
 			switch (x = a[0], c = escape(&s)) {
@@ -648,10 +633,14 @@ void texec(CAP *cap, unsigned char *s, int l, int a0, int a1, int a2, int a3)
 					a[0] %= x;
 					break;
 				case 'l':
-					a[0] = vars[x];
+					if (vars)
+						a[0] = vars[x];
 					break;
 				case 's':
-					vars[x] = a[0];
+					if (!vars)
+						vars = calloc(256, sizeof(int));
+					if (vars)
+						vars[x] = a[0];
 					break;
 				default:
 					a[0] = x;
@@ -693,6 +682,8 @@ void texec(CAP *cap, unsigned char *s, int l, int a0, int a1, int a2, int a3)
 				tenth -= cap->div;
 			}
 	}
+
+	free(vars);
 }
 
 static int total;
@@ -702,9 +693,9 @@ static void cst(unsigned char *ptr, unsigned char c)
 	++total;
 }
 
-int tcost(CAP *cap, unsigned char *s, int l, int a0, int a1, int a2, int a3)
+int tcost(CAP *cap, const unsigned char *s, int l, int a0, int a1, int a2, int a3)
 {
-	void (*out) (unsigned char *, unsigned char) = cap->out;
+	void (*out)(unsigned char *, unsigned char) = cap->out;
 
 	if (!s)
 		return 10000;
@@ -721,7 +712,8 @@ static void cpl(unsigned char *ptr, unsigned char c)
 	ssp = vsadd(ssp, c);
 }
 
-unsigned char *tcompile(CAP *cap, unsigned char *s, int a0, int a1, int a2, int a3)
+unsigned char *
+tcompile(CAP *cap, const unsigned char *s, int a0, int a1, int a2, int a3)
 {
 	void (*out) (unsigned char *, unsigned char) = cap->out;
 	int divider = cap->div;
