@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/tty.c,v 1.24 2017/08/08 16:12:04 tg Exp $ */
 /*
  *	UNIX Tty and Process interface
  *	Copyright
@@ -9,23 +8,15 @@
 #include "config.h"
 #include "types.h"
 
-#include <sys/types.h>
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#ifdef HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
-#endif
-#include <stdio.h>
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
+__RCSID("$MirOS: contrib/code/jupp/tty.c,v 1.36 2018/01/06 00:28:33 tg Exp $");
+
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
+#include <sys/stat.h>
+#include <sys/ioctl.h>
+#include <sys/wait.h>
+#include <fcntl.h>
 
 #include <errno.h>
 #include <string.h>
@@ -203,6 +194,7 @@ static int ttymode = 0;
 /* Signal state flag.  1 for joe, 0 for normal */
 static int ttysig = 0;
 
+#if WANT_FORK
 /* Stuff for shell windows */
 
 static pid_t kbdpid;		/* PID of kbd client */
@@ -222,6 +214,7 @@ struct packet {
 } pack;
 
 MPX asyncs[NPROC];
+#endif
 
 /* Set signals for JOE */
 void sigjoe(void)
@@ -236,11 +229,12 @@ void sigjoe(void)
 }
 
 /* Restore signals for exiting */
-void signrm(void)
+void signrm(int inchild)
 {
 	if (!ttysig)
 		return;
-	ttysig = 0;
+	if (!inchild)
+		ttysig = 0;
 	joe_set_signal(SIGHUP, SIG_DFL);
 	joe_set_signal(SIGTERM, SIG_DFL);
 	joe_set_signal(SIGINT, SIG_DFL);
@@ -260,7 +254,7 @@ void ttopen(void)
 void ttclose(void)
 {
 	ttclsn();
-	signrm();
+	signrm(0);
 }
 
 static volatile sig_atomic_t winched = 0;
@@ -407,7 +401,7 @@ baud_reset(int bbaud)
 		Baud = baud;
 	upc = DIVIDEND / baud;
 	if (obuf)
-		joe_free(obuf);
+		free(obuf);
 	if ((TIMES * upc) == 0)
 		obufsiz = 4096;
 	else {
@@ -417,7 +411,7 @@ baud_reset(int bbaud)
 	}
 	if (!obufsiz)
 		obufsiz = 1;
-	obuf = (unsigned char *) joe_malloc(obufsiz);
+	obuf = malloc(obufsiz);
 }
 
 /* Close terminal */
@@ -547,6 +541,7 @@ int ttflsh(void)
 		obufp = 0;
 	}
 
+#if WANT_FORK
 	/* Ack previous packet */
 	if (ackkbd != -1 && tty_accept != NO_MORE_DATA && !have) {
 		unsigned char c = 0;
@@ -557,10 +552,12 @@ int ttflsh(void)
 			joe_write(ackkbd, &c, 1);
 		tty_accept = NO_MORE_DATA;
 	}
+#endif
 
 	/* Check for typeahead or next packet */
 
 	if (!have && !leave) {
+#if WANT_FORK
 		if (ackkbd != -1) {
 			fcntl(mpxfd, F_SETFL, O_NDELAY);
 			if (read(mpxfd, &pack, sizeof(struct packet) - 1024) > 0) {
@@ -570,7 +567,9 @@ int ttflsh(void)
 				tty_accept = pack.ch;
 			} else
 				fcntl(mpxfd, F_SETFL, 0);
-		} else {
+		} else
+#endif
+		  {
 			/* Set terminal input to non-blocking */
 			fcntl(fileno(termin), F_SETFL, O_NDELAY);
 
@@ -587,13 +586,17 @@ int ttflsh(void)
 
 /* Read next character from input */
 
-void mpxdied(MPX *m);
+#if WANT_FORK
+static void mpxdied(MPX *m);
+#endif
 
 static time_t last_time;
 
 int ttgetc(void)
 {
+#if WANT_FORK
 	int stat_;
+#endif
 	time_t new_time;
 
 	tickon();
@@ -616,6 +619,7 @@ int ttgetc(void)
 		ttflsh();
 		tickon();
 	}
+#if WANT_FORK
 	if (ackkbd != -1) {
 		if (!have) {	/* Wait for input */
 			stat_ = read(mpxfd, &pack, sizeof(struct packet) - 1024);
@@ -652,6 +656,7 @@ int ttgetc(void)
 			}
 		}
 	}
+#endif
 	if (have) {
 		have = 0;
 	} else {
@@ -707,30 +712,45 @@ void ttgtsz(int *x, int *y)
 #endif
 }
 
-void ttshell(unsigned char *cmd)
+#ifndef SIGTSTP
+/* void ttshell(char *s);  Run a shell command or if 's' is zero, run a
+ * sub-shell
+ */
+static void ttshell(unsigned char *cmd);
+static const char shmsg[] =
+    "You are at the command shell.  Type 'exit' to return\n";
+
+#if WANT_FORK
+#define v_or_fork() fork()
+#else
+#define v_or_fork() vfork()
+#endif
+
+static void ttshell(unsigned char *cmd)
 {
 	int x, omode = ttymode;
 	const char *sh;
 
 	sh = getushell();
 	ttclsn();
-	if ((x = fork()) != 0) {
-		if (x != -1)
-			wait(NULL);
-		if (omode)
-			ttopnn();
-	} else {
-		signrm();
+	if (!(x = v_or_fork())) {
+		signrm(1);
 		if (cmd)
 			execl(sh, sh, "-c", cmd, NULL);
 		else {
-			fprintf(stderr, "You are at the command shell.  Type 'exit' to return\n");
+			write(2, shmsg, sizeof(shmsg) - 1);
 			execl(sh, sh, NULL);
 		}
 		_exit(0);
 	}
+	if (x != -1)
+		wait(NULL);
+	if (omode)
+		ttopnn();
 }
+#endif
 
+#if WANT_FORK
 /* Create keyboard task */
 
 static int mpxresume(void)
@@ -776,6 +796,7 @@ static void mpxsusp(void)
 		close(ackkbd);
 	}
 }
+#endif
 
 /* We used to leave the keyboard copy task around during suspend, but
    Cygwin gets confused when two processes are waiting for input and you
@@ -790,19 +811,24 @@ void ttsusp(void)
 
 #ifdef SIGTSTP
 	omode = ttymode;
+#if WANT_FORK
 	mpxsusp();
+#endif
 	ttclsn();
 	fprintf(stderr, "You have suspended the program.  Type 'fg' to return\n");
 	kill(0, SIGTSTP);
 	if (omode)
 		ttopnn();
+#if WANT_FORK
 	if (ackkbd!= -1)
 		mpxresume();
+#endif
 #else
 	ttshell(NULL);
 #endif
 }
 
+#if WANT_FORK
 /* Stuff for asynchronous I/O multiplexing.  We do not use streams or
    select() because joe needs to work on versions of UNIX which predate
    these calls.  Instead, when there is multiple async sources, we use
@@ -887,9 +913,9 @@ static unsigned char *getpty(int *ptyfd)
 	static unsigned char name[32];
 	int ttyfd;
 
-        if (openpty(ptyfd, &ttyfd, name, NULL, NULL) == 0)
-           return(name);
-        else
+	if (openpty(ptyfd, &ttyfd, name, NULL, NULL) == 0)
+	   return(name);
+	else
 	   return (NULL);
 }
 
@@ -911,18 +937,20 @@ static unsigned char *getpty(int *ptyfd)
 	int x, fd;
 	unsigned char *orgpwd = pwd();
 	static unsigned char **ptys = NULL;
-	static unsigned char *ttydir;
-	static unsigned char *ptydir;
+	static const unsigned char *ttydir;
+	static const unsigned char *ptydir;
 	static unsigned char ttyname[32];
 
 	if (!ptys) {
-		ttydir = US "/dev/pty/";
-		ptydir = US "/dev/ptym/";	/* HPUX systems */
-		if (chpwd(ptydir) || !(ptys = rexpnd(US "pty*")))
+		/* HPUX systems */
+		ttydir = UC "/dev/pty/";
+		ptydir = UC "/dev/ptym/";
+		if (chpwd(ptydir) || !(ptys = rexpnd(UC "pty*")))
 			if (!ptys) {
-				ttydir = ptydir = US "/dev/";	/* Everyone else */
+				/* Everyone else */
+				ttydir = ptydir = UC "/dev/";
 				if (!chpwd(ptydir))
-					ptys = rexpnd(US "pty*");
+					ptys = rexpnd(UC "pty*");
 			}
 	}
 	chpwd(orgpwd);
@@ -979,13 +1007,15 @@ static RETSIGTYPE death(int unused)
 
 extern unsigned char **mainenv;
 
-static unsigned char **newenv(unsigned char **old, unsigned char *s)
+static unsigned char **
+newenv(unsigned char **old, const unsigned char *s)
 {
 	unsigned char **new;
 	int x, y, z;
 
-	for (x = 0; old[x]; ++x) ;
-	new = (unsigned char **) joe_malloc((x + 2) * sizeof(unsigned char *));
+	for (x = 0; old[x]; ++x)
+		/* nothing */;
+	new = malloc((x + 2) * sizeof(unsigned char *));
 
 	for (x = 0, y = 0; old[x]; ++x) {
 		for (z = 0; s[z] != '='; ++z)
@@ -993,19 +1023,22 @@ static unsigned char **newenv(unsigned char **old, unsigned char *s)
 				break;
 		if (s[z] == '=') {
 			if (s[z + 1])
-				new[y++] = s;
+				new[y++] = (void *)strdup((const char *)s);
 		} else
 			new[y++] = old[x];
 	}
 	if (x == y)
-		new[y++] = s;
+		new[y++] = (void *)strdup((const char *)s);
 	new[y] = 0;
 	return new;
 }
 
 /* Create a shell process */
 
-MPX *mpxmk(int *ptyfd, const unsigned char *cmd, unsigned char **args, void (*func) (/* ??? */), void *object, void (*die) (/* ??? */), void *dieobj)
+MPX *
+mpxmk(int *ptyfd, const unsigned char *cmd, unsigned char **args,
+    void (*func)(B*, unsigned char *, int), void *object,
+    void (*die)(B*), void *dieobj)
 {
 	unsigned char buf[80];
 	int fds[2];
@@ -1086,7 +1119,7 @@ MPX *mpxmk(int *ptyfd, const unsigned char *cmd, unsigned char **args, void (*fu
 
 		if (!(pid = fork())) {
 			/* This process becomes the shell */
-			signrm();
+			signrm(0);
 
 			/* Close pty (we only need tty) */
 			close(*ptyfd);
@@ -1120,7 +1153,7 @@ MPX *mpxmk(int *ptyfd, const unsigned char *cmd, unsigned char **args, void (*fu
 
 			/* Open the TTY */
 			if ((x = open((char *)name, O_RDWR)) != -1) {	/* Standard input */
-				unsigned char **env = newenv(mainenv, US "TERM=");
+				unsigned char **env = newenv(mainenv, UC "TERM=");
 
 #ifdef HAVE_LOGIN_TTY
 				login_tty(x);
@@ -1183,7 +1216,7 @@ MPX *mpxmk(int *ptyfd, const unsigned char *cmd, unsigned char **args, void (*fu
 		   mode.  This allows us to read any remaining data- then
 		   read returns 0 and we know we're done. */
 
-	      loop:
+ loop:
 		pack.who = m;
 		pack.ch = 0;
 
@@ -1220,7 +1253,7 @@ MPX *mpxmk(int *ptyfd, const unsigned char *cmd, unsigned char **args, void (*fu
 	return m;
 }
 
-void mpxdied(MPX *m)
+static void mpxdied(MPX *m)
 {
 	if (!--nmpx)
 		mpxend();
@@ -1231,6 +1264,7 @@ void mpxdied(MPX *m)
 	m->func = NULL;
 	edupd(1);
 }
+#endif
 
 void
 tty_xonoffbaudrst(void)
