@@ -1,5 +1,4 @@
-/* $MirOS: contrib/code/jupp/tw.c,v 1.12 2017/03/19 18:53:02 tg Exp $ */
-/* 
+/*
  *	Text editing windows
  *	Copyright
  *		(C) 1992 Joseph H. Allen
@@ -9,7 +8,8 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h>
+__RCSID("$MirOS: contrib/code/jupp/tw.c,v 1.21 2018/01/06 00:28:33 tg Exp $");
+
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_TIME_H
@@ -33,6 +33,7 @@
 #include "vs.h"
 #include "syntax.h"
 #include "charmap.h"
+#include "tw.h"
 #include "w.h"
 
 #if !HAVE_DECL_CTIME
@@ -46,8 +47,9 @@ int keepup = 0;
 
 /* Move text window */
 
-static void movetw(BW *bw, int x, int y)
+static void movetw(jobject jO, int x, int y)
 {
+	BW *bw = jO.bw;
 	TW *tw = (TW *) bw->object;
 
 	if (y || !staen) {
@@ -67,8 +69,10 @@ static void movetw(BW *bw, int x, int y)
 
 /* Resize text window */
 
-static void resizetw(BW *bw, int wi, int he)
+static void resizetw(jobject jO, int wi, int he)
 {
+	BW *bw = jO.bw;
+
 	if (bw->parent->ny || !staen)
 		bwresz(bw, wi - (bw->o.linums ? LINCOLS : 0), he - 1);
 	else
@@ -83,7 +87,7 @@ static void resizetw(BW *bw, int wi, int he)
  *
  * There are actually two possibilities:
  *
- * We want the first line- 
+ * We want the first line-
  *
  * int
  * foo(int x,int y) {
@@ -98,7 +102,8 @@ static void resizetw(BW *bw, int wi, int he)
  *
  */
 
-unsigned char *get_context(BW *bw)
+static unsigned char *
+get_context(BW *bw)
 {
 	P *p = pdup(bw->cursor);
 	static unsigned char buf1[stdsiz];
@@ -110,7 +115,7 @@ unsigned char *get_context(BW *bw)
 		p_goto_bol(p);
 		if (!pisindent(p) && !pisblank(p)) {
 			/* Uncomment to get the last line instead of the first line (see above)
-			next:
+ next:
 			*/
 			brzs(p,stdbuf,stdsiz-1);
 			/* Ignore comment and block structuring lines */
@@ -119,21 +124,20 @@ unsigned char *get_context(BW *bw)
 			    (stdbuf[0]=='\f') ||
 			    (stdbuf[0]=='/' && stdbuf[1]=='/') ||
 			    (stdbuf[0]=='#') ||
-			    (stdbuf[0]=='b' && stdbuf[1]=='e' && stdbuf[2]=='g' && stdbuf[3]=='i' && stdbuf[4]=='n') ||
-			    (stdbuf[0]=='B' && stdbuf[1]=='E' && stdbuf[2]=='G' && stdbuf[3]=='I' && stdbuf[4]=='N') ||
+			    ((stdbuf[0] | 0x20) == 'b' && (stdbuf[1] | 0x20) == 'e' && (stdbuf[2] | 0x20) == 'g' && (stdbuf[3] | 0x20) == 'i' && (stdbuf[4] | 0x20) == 'n') ||
 			    (stdbuf[0]=='-' && stdbuf[1]=='-') ||
 			    stdbuf[0]==';')) {
-			    	strlcpy(buf1,stdbuf,stdsiz);
+				strlcpy(buf1,stdbuf,stdsiz);
 				/* Uncomment to get the last line instead of the first line (see above)
-			    	if (pprevl(p)) {
-			    		p_goto_bol(p);
-			    		if (!pisindent(p) && !pisblank(p))
-			    			goto next;
-			    	}
-			    	*/
-			    	break;
+				if (pprevl(p)) {
+					p_goto_bol(p);
+					if (!pisindent(p) && !pisblank(p))
+						goto next;
+				}
+				*/
+				break;
 			}
-			
+
 		}
 	} while (!buf1[0] && pprevl(p));
 
@@ -142,30 +146,32 @@ unsigned char *get_context(BW *bw)
 	return buf1;
 }
 
-static unsigned char *stagen(unsigned char *stalin, BW *bw, unsigned char *s, int fill)
+static unsigned char *
+stagen(unsigned char *stalin, BW *bw, const unsigned char *s, int fill)
 {
-	unsigned char *cp, *cp2, uc;
+	const unsigned char *ccp;
+	unsigned char *cp, uc;
 	unsigned char buf[80];
 	int x;
 	W *w = bw->parent;
 	time_t tt;
 	int special_aA = 0;
 
-	cp = s;
-	while ((cp2 = strstr(cp, "%a")) != NULL) {
-		cp2 += /* %a */ 2;
-		if (cp2[1] == '%')
-			++cp2;
-		if (cp2[0] == '%' && cp2[1] == 'A') {
+	ccp = s;
+	while ((cp = strstr(ccp, "%a")) != NULL) {
+		cp += /* %a */ 2;
+		if (cp[1] == '%')
+			++cp;
+		if (cp[0] == '%' && cp[1] == 'A') {
 			special_aA = 1;
 			break;
 		}
 	}
-	if (!special_aA) while ((cp2 = strstr(cp, "%A")) != NULL) {
-		cp2 += /* %A */ 2;
-		if (cp2[1] == '%')
-			++cp2;
-		if (cp2[0] == '%' && cp2[1] == 'a') {
+	if (!special_aA) while ((cp = strstr(ccp, "%A")) != NULL) {
+		cp += /* %A */ 2;
+		if (cp[1] == '%')
+			++cp;
+		if (cp[0] == '%' && cp[1] == 'a') {
 			special_aA = 1;
 			break;
 		}
@@ -178,8 +184,8 @@ static unsigned char *stagen(unsigned char *stalin, BW *bw, unsigned char *s, in
 			case 'x':
 				/* Context (but only if autoindent is enabled) */
 				if (bw->o.autoindent) {
-					cp = get_context(bw);
-					stalin = vsncpy(sv(stalin), sz(cp));
+					ccp = get_context(bw);
+					stalin = vsncpy(sv(stalin), sz(ccp));
 				}
 				break;
 
@@ -191,21 +197,21 @@ static unsigned char *stagen(unsigned char *stalin, BW *bw, unsigned char *s, in
 				break;
 			case 't':
 				tt = time(NULL);
-				cp = (unsigned char *)ctime(&tt);
+				ccp = (unsigned char *)ctime(&tt);
 
-				x = (cp[11] - '0') * 10 + cp[12] - '0';
+				x = (ccp[11] - '0') * 10 + ccp[12] - '0';
 				if (x > 12)
 					x -= 12;
 				joe_snprintf_1((char *)buf, sizeof(buf), "%2.2d", x);
 				if (buf[0] == '0')
 					buf[0] = fill;
 				stalin = vsncpy(sv(stalin), buf, 2);
-				stalin = vsncpy(sv(stalin), cp + 13, 3);
+				stalin = vsncpy(sv(stalin), ccp + 13, 3);
 				break;
 			case 'u':
 				tt = time(NULL);
-				cp = (unsigned char *)ctime(&tt);
-				stalin = vsncpy(sv(stalin), cp + 11, 5);
+				ccp = (unsigned char *)ctime(&tt);
+				stalin = vsncpy(sv(stalin), ccp + 11, 5);
 				break;
 			case 'T':
 				if (bw->o.overtype)
@@ -236,9 +242,9 @@ static unsigned char *stagen(unsigned char *stalin, BW *bw, unsigned char *s, in
 					stalin = vsncpy(sv(stalin), sc("Unnamed"));
 					break;
 				}
-				cp = bw->b->name;
+				ccp = bw->b->name;
  escape_loop:
-				switch ((uc = *cp++)) {
+				switch ((uc = *ccp++)) {
 				case '\\':
 					stalin = vsadd(stalin, uc);
 					/* FALLTHROUGH */
@@ -393,16 +399,17 @@ static unsigned char *stagen(unsigned char *stalin, BW *bw, unsigned char *s, in
 	return stalin;
 }
 
-static void disptw(BW *bw, int flg)
+static void disptw(jobject jO, int flg)
 {
+	BW *bw = jO.bw;
 	W *w = bw->parent;
 	TW *tw = (TW *) bw->object;
 
 	if (bw->o.linums != bw->linums) {
 		bw->linums = bw->o.linums;
-		resizetw(bw, w->w, w->h);
-		movetw(bw, w->x, w->y);
-		bwfllw(bw);
+		resizetw(jO, w->w, w->h);
+		movetw(jO, w->x, w->y);
+		bwfllw(jO);
 	}
 
 	if (bw->o.hex) {
@@ -474,10 +481,10 @@ int usplitw(BW *bw)
 	if (!new)
 		return -1;
 	wfit(new->t);
-	new->object = (void *) (newbw = bwmk(new, bw->b, 0));
+	new->object.bw = newbw = bwmk(new, bw->b, 0);
 	++bw->b->count;
 	newbw->offset = bw->offset;
-	newbw->object = (void *) (newtw = (TW *) joe_malloc(sizeof(TW)));
+	newbw->object = newtw = malloc(sizeof(TW));
 	iztw(newtw, new->y);
 	pset(newbw->top, bw->top);
 	pset(newbw->cursor, bw->cursor);
@@ -500,10 +507,10 @@ int uduptw(BW *bw)
 		return -1;
 	if (demotegroup(w))
 		new->t->topwin = new;
-	new->object = (void *) (newbw = bwmk(new, bw->b, 0));
+	new->object.bw = newbw = bwmk(new, bw->b, 0);
 	++bw->b->count;
 	newbw->offset = bw->offset;
-	newbw->object = (void *) (newtw = (TW *) joe_malloc(sizeof(TW)));
+	newbw->object = newtw = malloc(sizeof(TW));
 	iztw(newtw, new->y);
 	pset(newbw->top, bw->top);
 	pset(newbw->cursor, bw->cursor);
@@ -526,7 +533,7 @@ static void deltw(BW *bw, B *b, long int l, long int n, int flg)
 }
 
 WATOM watomtw = {
-	US "main",
+	UC "main",
 	disptw,
 	bwfllw,
 	NULL,
@@ -559,15 +566,15 @@ int abortit(BW *bw)
 			   any prompt windows? */
 
 			bwrm(bw);
-			w->object = (void *) (bw = bwmk(w, b, 0));
+			w->object.bw = bw = bwmk(w, b, 0);
 			wredraw(bw->parent);
 			bw->object = object;
 			return 0;
 		}
 	bwrm(bw);
 	vsrm(tw->stalin);
-	joe_free(tw);
-	w->object = NULL;
+	free(tw);
+	w->object.base = NULL;
 	wabort(w);	/* Eliminate this window and it's children */
 	return 0;
 }
@@ -578,7 +585,7 @@ static int naborttw(BW *bw, int k, void *object, int *notify)
 {
 	if (notify)
 		*notify = 1;
-	if (k != 'y' && k != 'Y')
+	if ((k | 0x20) != 'y')
 		return -1;
 
 	genexmsg(bw, 0, NULL);
@@ -589,7 +596,7 @@ static int naborttw1(BW *bw, int k, void *object, int *notify)
 {
 	if (notify)
 		*notify = 1;
-	if (k != 'y' && k != 'Y')
+	if ((k | 0x20) != 'y')
 		return -1;
 
 	if (!exmsg) genexmsg(bw, 0, NULL);
@@ -657,7 +664,7 @@ int uabortbuf(BW *bw)
 		void *object = bw->object;
 
 		bwrm(bw);
-		w->object = (void *) (bw = bwmk(w, b, 0));
+		w->object.bw = bw = bwmk(w, b, 0);
 		wredraw(bw->parent);
 		bw->object = object;
 		return 0;
@@ -670,7 +677,7 @@ int uabortbuf(BW *bw)
 
 int utw0(BASE *b)
 {
-	BW *bw = b->parent->main->object;
+	BW *bw = b->parent->main->object.bw;
 
 	if (countmain(b->parent->t) == 1)
 		return -1;
@@ -690,12 +697,12 @@ int utw1(BASE *b)
 
 	do {
 		yn = 0;
-	      loop:
+ loop:
 		do {
 			wnext(t);
 		} while (t->curwin->main == mainw && t->curwin != starting);
 		if (t->curwin->main != mainw) {
-			BW *bw = t->curwin->main->object;
+			BW *bw = t->curwin->main->object.bw;
 			utw0((BASE *)bw);
 			yn = 1;
 			goto loop;
@@ -710,7 +717,7 @@ void setline(B *b, long int line)
 
 	do {
 		if (w->watom->what == TYPETW) {
-			BW *bw = w->object;
+			BW *bw = w->object.bw;
 
 			if (bw->b == b) {
 				long oline = bw->top->line;
@@ -736,8 +743,8 @@ BW *wmktw(SCREEN *t, B *b)
 
 	w = wcreate(t, &watomtw, NULL, NULL, NULL, t->h, NULL, NULL);
 	wfit(w->t);
-	w->object = (void *) (bw = bwmk(w, b, 0));
-	bw->object = (void *) (tw = (TW *) joe_malloc(sizeof(TW)));
+	w->object.bw = bw = bwmk(w, b, 0);
+	bw->object = tw = malloc(sizeof(TW));
 	iztw(tw, w->y);
 	return bw;
 }
