@@ -1,31 +1,42 @@
-/* $MirOS: contrib/code/jupp/types.h,v 1.15 2017/01/11 22:56:49 tg Exp $ */
-
 #ifndef _JOE_TYPES_H
 #define _JOE_TYPES_H
 
-#include "config.h"
+#ifdef EXTERN
+__IDSTRING(rcsid_types_h, "$MirOS: contrib/code/jupp/types.h,v 1.35 2018/02/14 17:51:49 tg Exp $");
+#endif
+
+/*-
+ * Copyright © 2004, 2005, 2006, 2007, 2008, 2011, 2012, 2013,
+ *	       2014, 2016, 2017, 2018
+ *	Thorsten “mirabilos” Glaser <tg@mirbsd.org>
+ *
+ * Provided that these terms and disclaimer and all copyright notices
+ * are retained or reproduced in an accompanying document, permission
+ * is granted to deal in this work without restriction, including un‐
+ * limited rights to use, publicly perform, distribute, sell, modify,
+ * merge, give away, or sublicence.
+ *
+ * This work is provided “AS IS” and WITHOUT WARRANTY of any kind, to
+ * the utmost extent permitted by applicable law, neither express nor
+ * implied; without malicious intent or gross negligence. In no event
+ * may a licensor, author or contributor be held liable for indirect,
+ * direct, other damage, loss, or other issues arising in any way out
+ * of dealing in the work, even if advised of the possibility of such
+ * damage or existence of a defect, except proven that it results out
+ * of said person’s immediate fault when using the work as intended.
+ */
 
 /* Prefix to make string constants unsigned */
+#define UC (const unsigned char *)
 #define US (unsigned char *)
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>				/* we need pid_t */
-#endif
-
-/* from mksh */
-#if defined(DEBUG) || defined(__COVERITY__)
-#define mkssert(e)	do { if (!(e)) exit(255); } while (/* CONSTCOND */ 0)
-#else
-#define mkssert(e)	do { } while (/* CONSTCOND */ 0)
-#endif
-
-/* from mksh */
-#define NELEM(a)	(sizeof(a) / sizeof((a)[0]))
-
 #define LINK(type) struct { type *next; type *prev; }
 
-#define KEYS		256
+#ifdef SMALL
+#define stdsiz		4096
+#else
 #define stdsiz		8192
+#endif
 #define FITHEIGHT	4		/* Minimum text window height */
 #define LINCOLS		6
 #define NPROC		8		/* Number of processes we keep track of */
@@ -37,6 +48,74 @@
 #define TYPEMENU	0x0800
 #define TYPEQW		0x1000
 
+/* polymorph function pointers, which do not use compiler type checking */
+#ifndef GCC_Wstrict_prototypes
+typedef int jpoly_int();
+typedef void jpoly_void();
+#else
+/* same content as above, but system header */
+#include <jupp.tmp.h>
+#endif
+
+struct jalloc_common {
+	/*XXX these must be size_t not int */
+	/*
+	 * size part: number of elements that can be fit,
+	 * not counting the terminator or space needed for the header
+	 */
+	int esiz;
+	/*
+	 * length part: number of elements currently in the array,
+	 * not counting the terminator
+	 */
+	int elen;
+};
+
+#ifdef MKSH_ALLOC_CATCH_UNDERRUNS
+struct jalloc_item {
+	struct jalloc_common enfo;
+	size_t len;
+	char dummy[8192 - sizeof(struct jalloc_common) - sizeof(size_t)];
+};
+#define ALLOC_INFO(f)	enfo.f
+#define ALLOC_ITEM	struct jalloc_item
+#else
+#define ALLOC_INFO(f)	f
+#define ALLOC_ITEM	struct jalloc_common
+#endif
+
+#define jalloc_krnl(i)	((ALLOC_ITEM *)((char *)(i) - sizeof(ALLOC_ITEM)))
+#define jalloc_user(i)	((void *)((char *)(i) + sizeof(ALLOC_ITEM)))
+#define jalloc_siz(i)	(jalloc_krnl(i)->ALLOC_INFO(esiz))
+#define jalloc_len(i)	(jalloc_krnl(i)->ALLOC_INFO(elen))
+
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#endif
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+
+#ifndef SIZE_MAX
+#ifdef SIZE_T_MAX
+#define SIZE_MAX	SIZE_T_MAX
+#else
+#define SIZE_MAX	((size_t)-1)
+#endif
+#endif
+
+#define notok2mul(max, val, c)	(((val) != 0) && ((c) != 0) && \
+				    (((max) / (c)) < (val)))
+#define notok2add(max, val, c)	((val) > ((max) - (c)))
+#define notoktomul(val, cnst)	notok2mul(SIZE_MAX, (val), (cnst))
+#define notoktoadd(val, cnst)	notok2add(SIZE_MAX, (val), (cnst))
+
+void jalloc_init(void);
+void *jalloc(void *, size_t, size_t);
+void jfree(void *);
+
+#define ralloc(nmemb, size)	(notoktomul(nmemb, size) ? NULL : \
+				    malloc((nmemb) * (size)))
 
 typedef struct header H;
 typedef struct buffer B;
@@ -44,7 +123,7 @@ typedef struct point P;
 typedef struct options OPTIONS;
 typedef struct macro MACRO;
 typedef struct cmd CMD;
-typedef struct entry HENTRY;
+typedef struct hentry HENTRY;
 typedef struct hash HASH;
 typedef struct kmap KMAP;
 typedef struct kbd KBD;
@@ -69,6 +148,13 @@ typedef struct srchrec SRCHREC;
 typedef struct vpage VPAGE;
 typedef struct vfile VFILE;
 
+/* window.object* */
+typedef union {
+	BASE *base;
+	MENU *menu;
+	BW *bw;
+	QW *qw;
+} jobject;
 
 struct header {
 	LINK(H)	link;		/* LINK ??? */
@@ -108,9 +194,9 @@ struct options {
 	int	tab;
 	int	indentc;
 	int	istep;
-	unsigned char	*context;
-	unsigned char	*lmsg;
-	unsigned char	*rmsg;
+	unsigned char *context;
+	const unsigned char *lmsg;
+	const unsigned char *rmsg;
 	char	*hmsg;
 	int	linums;
 	int	readonly;
@@ -154,12 +240,12 @@ struct recmac {
 /* Command entry */
 
 struct cmd {
-	unsigned char	*name;		/* Command name */
-	int	flag;		/* Execution flags */
-	int	(*func) ();	/* Function bound to name */
+	const unsigned char *name;	/* Command name */
+	const unsigned char *negarg;	/* Command to use if arg was negative */
+	jpoly_int *func;	/* Function bound to name */
 	MACRO	*m;		/* Macro bound to name */
+	unsigned int flag;	/* Execution flags */
 	int	arg;		/* 0= arg is meaningless, 1= ok */
-	unsigned char	*negarg;	/* Command to use if arg was negative */
 };
 
 
@@ -188,8 +274,8 @@ struct buffer {
 };
 
 
-struct entry {
-	unsigned char	*name;
+struct hentry {
+	const unsigned char *name;
 	HENTRY	*next;
 	void	*val;
 };
@@ -201,11 +287,11 @@ struct hash {
 
 
 struct help {
-	unsigned char	*text;		/* help text with attributes */
-	unsigned int	lines;		/* number of lines */
 	struct help	*prev;		/* previous help screen */
 	struct help	*next;		/* nex help screen */
 	unsigned char	*name;		/* context name for context sensitive help */
+	unsigned char	*text;		/* help text with attributes */
+	unsigned int	lines;		/* number of lines */
 };
 
 /* A key binding */
@@ -217,9 +303,9 @@ struct key {
 	} value;
 };
 
-/* A map of keycode to command/sub-map bindings */
+/* A map of keycode (octet) to command/sub-map bindings */
 struct kmap {
-	KEY	keys[KEYS];	/* KEYs */
+	KEY	keys[256];	/* KEYs */
 };
 
 /** A keyboard handler **/
@@ -232,16 +318,17 @@ struct kbd {
 
 
 struct watom {
-	unsigned char	*context;	/* Context name */
-	void	(*disp) ();	/* Display window */
-	void	(*follow) ();	/* Called to have window follow cursor */
-	int	(*abort) ();	/* Common user functions */
-	int	(*rtn) ();
-	int	(*type) ();
-	void	(*resize) ();	/* Called when window changed size */
-	void	(*move) ();	/* Called when window moved */
-	void	(*ins) ();	/* Called on line insertions */
-	void	(*del) ();	/* Called on line deletions */
+	const unsigned char *context;	/* Context name */
+	void (*disp)(jobject, int);	/* Display window */
+	void (*follow)(jobject);	/* Display window */
+	int (*abort)(jobject);		/* Common user functions */
+	int (*rtn)(jobject);
+	int (*type)(jobject, int);
+					/* Called when… */
+	void (*resize)(jobject, int, int);	/* window changed size */
+	void (*move)(jobject, int, int);	/* window moved */
+	void (*ins)(BW *, B *, long, long, int);	/* on line insertions */
+	void (*del)(BW *, B *, long, long, int);	/* on line deletions */
 	int	what;		/* Type of this thing */
 };
 
@@ -286,17 +373,7 @@ struct window {
 	int	curx, cury;	/* Cursor position within window */
 	KBD	*kbd;		/* Keyboard handler for this window */
 	WATOM	*watom;		/* The type of this window */
-	void	*object;	/* Object which inherits this */
-#if 0
-	union {			/* FIXME: instead of void *object we should */
-		BW	*bw;	/* use this union to get strict type checking */
-		PW	*pw;	/* from C compiler (need to check and change */
-		QW	*qw;	/* all of the occurrencies of ->object) */
-		TW	*tw;
-		MENU	*menu;
-		BASE	*base;
-	} object;
-#endif
+	jobject	object;		/* Object which inherits this */
 
 	const unsigned char *msgt;	/* Message at top of window */
 	const unsigned char *msgb;	/* Message at bottom of window */
@@ -336,46 +413,29 @@ struct menu {
 	int	saved_co;	/* Saved #columns of screen */
 	SCREEN	*t;		/* Screen we're on */
 	int	h, w, x, y;
-	int	(*abrt) ();	/* Abort callback function */
-	int	(*func) ();	/* Return callback function */
-	int	(*backs) ();	/* Backspace callback function */
+	jpoly_int *abrt;	/* Abort callback function */
+	jpoly_int *func;	/* Return callback function */
+	jpoly_int *backs;	/* Backspace callback function */
 	void	*object;
 };
 
-struct hentry {
+struct s_hentry {
 	int	next;
 	int	loc;
 };
 
 /* Each terminal has one of these */
-
-#ifdef __MSDOS__
-
-struct scrn {
-	int	li;		/* Height of screen */
-	int	co;		/* Width of screen */
-	short	*scrn;		/* Buffer */
-	int	scroll;
-	int	insdel;
-	int	*updtab;	/* Lines which need to be updated */
-	/* HIGHLIGHT_STATE *syntab; */ /* Syntax highlight state at start of each line */
-	int	*syntab;
-	int	*compose;
-	int	*sary;
-};
-
-#else
 struct scrn {
 	CAP	*cap;		/* Termcap/Terminfo data */
 
 	int	li;		/* Screen height */
 	int	co;		/* Screen width */
 
-	unsigned char	*ti;		/* Initialization string */
-	unsigned char	*cl;		/* Home and clear screen... really an
-				   init. string */
-	unsigned char	*cd;		/* Clear to end of screen */
-	unsigned char	*te;		/* Restoration string */
+	const unsigned char *ti;	/* Initialisation string */
+	const unsigned char *cl;	/* Home and clear screen... really an
+					   init. string */
+	const unsigned char *cd;	/* Clear to end of screen */
+	const unsigned char *te;	/* Restoration string */
 
 	int	haz;		/* Terminal can't print ~s */
 	int	os;		/* Terminal overstrikes */
@@ -384,73 +444,73 @@ struct scrn {
 	int	am;		/* Terminal has autowrap, but not magicwrap */
 	int	xn;		/* Terminal has magicwrap */
 
-	unsigned char	*so;		/* Enter standout (inverse) mode */
-	unsigned char	*se;		/* Exit standout mode */
+	const unsigned char *so;	/* Enter standout (inverse) mode */
+	const unsigned char *se;	/* Exit standout mode */
 
-	unsigned char	*us;		/* Enter underline mode */
-	unsigned char	*ue;		/* Exit underline mode */
-	unsigned char	*uc;		/* Single time underline character */
+	const unsigned char *us;	/* Enter underline mode */
+	const unsigned char *ue;	/* Exit underline mode */
+	const unsigned char *uc;	/* Single time underline character */
 
 	int	ms;		/* Ok to move when in standout/underline mode */
 
-	unsigned char	*mb;		/* Enter blinking mode */
-	unsigned char	*md;		/* Enter bold mode */
-	unsigned char	*mh;		/* Enter dim mode */
-	unsigned char	*mr;		/* Enter inverse mode */
-	unsigned char	*me;		/* Exit above modes */
+	const unsigned char *mb;	/* Enter blinking mode */
+	const unsigned char *md;	/* Enter bold mode */
+	const unsigned char *mh;	/* Enter dim mode */
+	const unsigned char *mr;	/* Enter inverse mode */
+	const unsigned char *me;	/* Exit above modes */
 
-	unsigned char	*Sb;		/* Set background color */
-	unsigned char	*Sf;		/* Set foregrond color */
+	const unsigned char *Sb;	/* Set background color */
+	const unsigned char *Sf;	/* Set foregrond color */
 	int	ut;		/* Screen erases with background color */
 
 	int	da, db;		/* Extra lines exist above, below */
-	unsigned char	*al, *dl, *AL, *DL;	/* Insert/delete lines */
-	unsigned char	*cs;		/* Set scrolling region */
+	const unsigned char *al, *dl, *AL, *DL;	/* Insert/delete lines */
+	const unsigned char *cs;		/* Set scrolling region */
 	int	rr;		/* Set for scrolling region relative addressing */
-	unsigned char	*sf, *SF, *sr, *SR;	/* Scroll */
+	const unsigned char *sf, *SF, *sr, *SR;	/* Scroll */
 
-	unsigned char	*dm, *dc, *DC, *ed;	/* Delete characters */
-	unsigned char	*im, *ic, *IC, *ip, *ei;	/* Insert characters */
+	const unsigned char *dm, *dc, *DC, *ed;	/* Delete characters */
+	const unsigned char *im, *ic, *IC, *ip, *ei;	/* Insert characters */
 	int	mi;		/* Set if ok to move while in insert mode */
 
-	unsigned char	*bs;		/* Move cursor left 1 */
+	const unsigned char *bs;	/* Move cursor left 1 */
 	int	cbs;
-	unsigned char	*lf;		/* Move cursor down 1 */
+	const unsigned char *lf;	/* Move cursor down 1 */
 	int	clf;
-	unsigned char	*up;		/* Move cursor up 1 */
+	const unsigned char *up;	/* Move cursor up 1 */
 	int	cup;
-	unsigned char	*nd;		/* Move cursor right 1 */
+	const unsigned char *nd;	/* Move cursor right 1 */
 
-	unsigned char	*ta;		/* Move cursor to next tab stop */
+	const unsigned char *ta;	/* Move cursor to next tab stop */
 	int	cta;
-	unsigned char	*bt;		/* Move cursor to previous tab stop */
+	const unsigned char *bt;	/* Move cursor to previous tab stop */
 	int	cbt;
-	int	tw;		/* Tab width */
+	int	tw;			/* Tab width */
 
-	unsigned char	*ho;		/* Home cursor to upper left */
+	const unsigned char *ho;	/* Home cursor to upper left */
 	int	cho;
-	unsigned char	*ll;		/* Home cursor to lower left */
+	const unsigned char *ll;	/* Home cursor to lower left */
 	int	cll;
-	unsigned char	*cr;		/* Move cursor to left edge */
+	const unsigned char *cr;	/* Move cursor to left edge */
 	int	ccr;
-	unsigned char	*RI;		/* Move cursor right n */
+	const unsigned char *RI;	/* Move cursor right n */
 	int	cRI;
-	unsigned char	*LE;		/* Move cursor left n */
+	const unsigned char *LE;	/* Move cursor left n */
 	int	cLE;
-	unsigned char	*UP;		/* Move cursor up n */
+	const unsigned char *UP;	/* Move cursor up n */
 	int	cUP;
-	unsigned char	*DO;		/* Move cursor down n */
+	const unsigned char *DO;	/* Move cursor down n */
 	int	cDO;
-	unsigned char	*ch;		/* Set cursor column */
+	const unsigned char *ch;	/* Set cursor column */
 	int	cch;
-	unsigned char	*cv;		/* Set cursor row */
+	const unsigned char *cv;	/* Set cursor row */
 	int	ccv;
-	unsigned char	*cV;		/* Goto beginning of specified line */
+	const unsigned char *cV;	/* Goto beginning of specified line */
 	int	ccV;
-	unsigned char	*cm;		/* Set cursor row and column */
+	const unsigned char *cm;	/* Set cursor row and column */
 	int	ccm;
 
-	unsigned char	*ce;		/* Clear to end of line */
+	const unsigned char *ce;	/* Clear to end of line */
 	int	cce;
 
 	/* Basic abilities */
@@ -472,10 +532,9 @@ struct scrn {
 
 	int	*compose;	/* Line compose buffer */
 	int	*ofst;		/* stuff for magic */
-	struct hentry	*htab;
-	struct hentry	*ary;
+	struct s_hentry	*htab;
+	struct s_hentry	*ary;
 };
-#endif
 
 
 struct sortentry {
@@ -494,7 +553,7 @@ struct cap {
 
 	int	div;		/* tenths of MS per char */
 	int	baud;		/* Baud rate */
-	unsigned char	*pad;		/* Padding string or NULL to use NUL */
+	const unsigned char *pad;	/* Padding string or NULL to use NUL */
 	void	(*out) (unsigned char *, unsigned char);		/* Character output routine */
 	void	*outptr;	/* First arg passed to output routine.  Second
 				   arg is character to write */
@@ -505,10 +564,10 @@ struct cap {
 
 
 struct pw {
-	int	(*pfunc) ();	/* Func which gets called when RTN is hit */
-	int	(*abrt) ();	/* Func which gets called when window is aborted */
-	int	(*tab) ();	/* Func which gets called when TAB is hit */
-	unsigned char	*prompt;	/* Prompt string */
+	jpoly_int *pfunc;	/* Func which gets called when RTN is hit */
+	jpoly_int *abrt;	/* Func which gets called when window is aborted */
+	jpoly_int *tab;		/* Func which gets called when TAB is hit */
+	unsigned char *prompt;	/* Prompt string */
 	int	promptlen;	/* Width of prompt string */
 	int	promptofst;	/* Prompt scroll offset */
 	B	*hist;		/* History buffer */
@@ -521,8 +580,8 @@ struct stditem {
 
 struct query {
 	W	*parent;	/* Window we're in */
-	int	(*func) ();	/* Func. which gets called when key is hit */
-	int	(*abrt) ();
+	jpoly_int *func;	/* Func. which gets called when key is hit */
+	jpoly_int *abrt;
 	void	*object;
 	unsigned char	*prompt;	/* Prompt string */
 	int	promptlen;	/* Width of prompt string */
@@ -535,9 +594,9 @@ struct mpx {
 	int	ackfd;		/* Packetizer response descriptor */
 	int	kpid;		/* Packetizer process id */
 	int	pid;		/* Client process id */
-	void	(*func) ();	/* Function to call when read occures */
+	jpoly_void *func;	/* Function to call when read occures */
 	void	*object;	/* First arg to pass to function */
-	void	(*die) ();	/* Function: call when client dies or closes */
+	jpoly_void *die;	/* Function: call when client dies or closes */
 	void	*dieobj;
 };
 
@@ -578,7 +637,7 @@ struct undorec {
 	long	len;		/* Length of insert or delete */
 	int	del;		/* Set if this is a delete */
 	B	*big;		/* Set to buffer containing a large amount of deleted data */
-	unsigned char	*small;		/* Set to malloc block containg a small amount of deleted data */
+	unsigned char *small;	/* Set to malloc block containg a small amount of deleted data */
 };
 
 struct undo {
@@ -607,7 +666,7 @@ struct search {
 	int	replace;	/* Set if this is search & replace */
 	int	rest;		/* Set to do remainder of search & replace w/o query */
 	unsigned char	*entire;	/* Entire matched string */
-	unsigned char	*pieces[26];	/* Peices of the matched string */
+	unsigned char	*pieces[26];	/* Pieces of the matched string */
 	int	flg;		/* Set after prompted for first replace */
 	SRCHREC	recs;		/* Search & replace position history */
 	P	*markb, *markk;	/* Original marks */
