@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/ublock.c,v 1.10 2013/08/19 22:04:15 tg Exp $ */
 /*
  * 	Highlighted block functions
  *	Copyright
@@ -9,16 +8,11 @@
 #include "config.h"
 #include "types.h"
 
-#include <unistd.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_WAIT_H
+__RCSID("$MirOS: contrib/code/jupp/ublock.c,v 1.31 2018/01/07 20:32:47 tg Exp $");
+
 #include <sys/wait.h>
-#endif
+#include <stdlib.h>
+#include <unistd.h>
 
 #include "b.h"
 #include "pw.h"
@@ -32,7 +26,6 @@
 #include "path.h"
 #include "poshist.h"
 #include "ushell.h"
-#include "utf8.h"
 #include "charmap.h"
 #include "w.h"
 
@@ -55,8 +48,8 @@ typedef struct marksav MARKSAV;
 struct marksav {
 	LINK(MARKSAV) link;
 	P *markb, *markk;
-} markstack = { { &markstack, &markstack} };
-MARKSAV markfree = { {&markfree, &markfree} };
+} markstack = { { &markstack, &markstack}, NULL, NULL };
+MARKSAV markfree = { {&markfree, &markfree}, NULL, NULL };
 int nstack = 0;
 
 int upsh(BW *bw)
@@ -306,7 +299,7 @@ int utoggle_marking(BW *bw)
 		prm(markk); markk=0;
 		updall();
 		marking = 0;
-		msgnw(bw->parent, US "Selection cleared.");
+		msgnw(bw->parent, UC "Selection cleared.");
 		return 0;
 	} else if (markk) {
 		/* Clear selection and start new one */
@@ -314,7 +307,7 @@ int utoggle_marking(BW *bw)
 		prm(markk); markk=0;
 		updall();
 		marking = 1;
-		msgnw(bw->parent, US "Selection started.");
+		msgnw(bw->parent, UC "Selection started.");
 		return umarkb(bw);
 	} else if (markb && markb->b==bw->cursor->b) {
 		marking = 0;
@@ -331,7 +324,7 @@ int utoggle_marking(BW *bw)
 		return 0;
 	} else {
 		marking = 1;
-		msgnw(bw->parent, US "Selection started.");
+		msgnw(bw->parent, UC "Selection started.");
 		return umarkb(bw);
 	}
 }
@@ -438,7 +431,7 @@ int ublkdel(BW *bw)
 		if (lightoff)
 			unmark(bw);
 	} else {
-		msgnw(bw->parent, US "No block");
+		msgnw(bw->parent, UC "No block");
 		return -1;
 	}
 	return 0;
@@ -475,7 +468,7 @@ int ublkmove(BW *bw)
 {
 	if (markv(1)) {
 		if (markb->b->rdonly) {
-			msgnw(bw->parent, US "Read only");
+			msgnw(bw->parent, UC "Read only");
 			return -1;
 		}
 		if (square) {
@@ -526,7 +519,7 @@ int ublkmove(BW *bw)
 			return 0;
 		}
 	}
-	msgnw(bw->parent, US "No block");
+	msgnw(bw->parent, UC "No block");
 	return -1;
 }
 
@@ -582,7 +575,7 @@ int ublkcpy(BW *bw)
 			return 0;
 		}
 	} else {
-		msgnw(bw->parent, US "No block");
+		msgnw(bw->parent, UC "No block");
 		return -1;
 	}
 }
@@ -592,43 +585,34 @@ int ublkcpy(BW *bw)
 
 int dowrite(BW *bw, unsigned char *s, void *object, int *notify)
 {
+	int fl;
+	int ret = 0;
+
 	if (notify)
 		*notify = 1;
-	if (markv(1)) {
-		if (square) {
-			int fl;
-			int ret = 0;
-			B *tmp = pextrect(markb,
-					  markk->line - markb->line + 1,
-					  markk->xcol);
-
-			if ((fl = bsave(tmp->bof, s, tmp->eof->byte, 0)) != 0) {
-				msgnw(bw->parent, msgs[-fl]);
-				ret = -1;
-			}
-			brm(tmp);
-			if (lightoff)
-				unmark(bw);
-			vsrm(s);
-			return ret;
-		} else {
-			int fl;
-			int ret = 0;
-
-			if ((fl = bsave(markb, s, markk->byte - markb->byte, 0)) != 0) {
-				msgnw(bw->parent, msgs[-fl]);
-				ret = -1;
-			}
-			if (lightoff)
-				unmark(bw);
-			vsrm(s);
-			return ret;
-		}
-	} else {
+	if (!markv(1)) {
 		vsrm(s);
-		msgnw(bw->parent, US "No block");
-		return -1;
+		msgnw(bw->parent, UC "No block");
+		return (-1);
 	}
+	if (square) {
+		B *tmp = pextrect(markb,
+				  markk->line - markb->line + 1,
+				  markk->xcol);
+
+		fl = bsave(tmp->bof, s, tmp->eof->byte, 0);
+		brm(tmp);
+	} else {
+		fl = bsave(markb, s, markk->byte - markb->byte, 0);
+	}
+	if (fl != 0) {
+		msgnw(bw->parent, msgs[-fl]);
+		ret = -1;
+	}
+	if (lightoff)
+		unmark(bw);
+	vsrm(s);
+	return (ret);
 }
 
 /* Set highlighted block on a program block */
@@ -653,7 +637,7 @@ void setindent(BW *bw)
 	} while (pisindent(p) >= indent || pisblank(p));
 	pnextl(p);
 	/* Maybe skip blank lines at beginning */
-      done:
+ done:
 	p_goto_bol(p);
 	p->xcol = piscol(p);
 	if (markb)
@@ -679,7 +663,8 @@ void setindent(BW *bw)
 /* Verifies that at least n indentation characters (for non-blank lines) match c */
 /* If n is 0 (for urindent), this fails if c is space but indentation begins with tab */
 
-int purity_check(int c, int n)
+static int
+purity_check(int c, int n)
 {
 	P *p = pdup(markb);
 	while (p->byte < markk->byte) {
@@ -703,7 +688,8 @@ int purity_check(int c, int n)
 /* Left indent check */
 /* Verify that there is enough whitespace to do the left indent */
 
-int lindent_check(int c, int n)
+static int
+lindent_check(int c, int n)
 {
 	P *p = pdup(markb);
 	int indwid;
@@ -779,7 +765,7 @@ int urindent(BW *bw)
 			prm(p);
 		} else {
 			/* Purity failure */
-			msgnw(bw->parent,US "Selected lines not properly indented");
+			msgnw(bw->parent,UC "Selected lines not properly indented");
 			return 1;
 		}
 	}
@@ -862,7 +848,7 @@ int ulindent(BW *bw)
 			prm(q);
 		} else {
 			/* Purity failure */
-			msgnw(bw->parent,US "Selected lines not properly indented");
+			msgnw(bw->parent, UC "Selected lines not properly indented");
 			return 1;
 		}
 	}
@@ -875,7 +861,7 @@ int doinsf(BW *bw, unsigned char *s, void *object, int *notify)
 {
 	if (notify)
 		*notify = 1;
-	if (square)
+	if (square) {
 		if (markv(1)) {
 			B *tmp;
 			long width = markk->xcol - markb->xcol;
@@ -888,6 +874,7 @@ int doinsf(BW *bw, unsigned char *s, void *object, int *notify)
 			if (error) {
 				msgnw(bw->parent, msgs[-error]);
 				brm(tmp);
+				vsrm(s);
 				return -1;
 			}
 			if (piscol(tmp->eof))
@@ -908,10 +895,13 @@ int doinsf(BW *bw, unsigned char *s, void *object, int *notify)
 			}
 			brm(tmp);
 			updall();
+			vsrm(s);
 			return 0;
 		} else {
-			msgnw(bw->parent, US "No block");
+			vsrm(s);
+			msgnw(bw->parent, UC "No block");
 			return -1;
+		}
 	} else {
 		int ret = 0;
 		B *tmp = bload(s);
@@ -939,6 +929,12 @@ int doinsf(BW *bw, unsigned char *s, void *object, int *notify)
 
 static int filtflg = 0;
 
+#if WANT_FORK
+#define v_or_fork() fork()
+#else
+#define v_or_fork() vfork()
+#endif
+
 /*
  * This isn't optimal, but until the home-brewn VM system is removed
  * it is the best we can do: we cannot use bsavefd() in a concurrent
@@ -953,8 +949,12 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 {
 	int fr[2];
 	int fw;
-	int flg = 0;
+	volatile int flg = 0;
 	unsigned char *tf;
+	const char *sh;
+#if defined(HAVE_PUTENV) && (WANT_FORK || defined(HAVE_UNSETENV))
+	unsigned char *fname;
+#endif
 
 	if (notify)
 		*notify = 1;
@@ -962,19 +962,21 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 		flg = 1;
 		goto ok;
 	} if (!markv(1)) {
-		msgnw(bw->parent, US "No block");
+		vsrm(s);
+		msgnw(bw->parent, UC "No block");
 		return -1;
 	}
-      ok:
-
+ ok:
 	if (pipe(fr)) {
-		msgnw(bw->parent, US "Pipe error");
+		vsrm(s);
+		msgnw(bw->parent, UC "Pipe error");
 		return (-1);
 	}
 	if ((tf = mktmp(NULL, &fw)) == NULL) {
 		close(fr[0]);
 		close(fr[1]);
-		msgnw(bw->parent, US "Cannot create temporary file");
+		vsrm(s);
+		msgnw(bw->parent, UC "Cannot create temporary file");
 		return (-1);
 	}
 	unlink((char *)tf);
@@ -990,13 +992,20 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 	} else
 		bsavefd(markb, fw, markk->byte - markb->byte);
 	lseek(fw, (off_t)0, SEEK_SET);
-	if (!fork()) {
-		const char *sh;
-#ifdef HAVE_PUTENV
-		unsigned char		*fname, *name;
-		unsigned	len;
+#if defined(HAVE_PUTENV) && (WANT_FORK || defined(HAVE_UNSETENV))
+	fname = vsncpy(NULL, 0, sc("JOE_FILENAME="));
+	tf = bw->b->name ? bw->b->name : (unsigned char *)"Unnamed";
+	fname = vsncpy(sv(fname), sz(tf));
+#if !WANT_FORK
+	putenv((char *)fname);
 #endif
-		signrm();
+#endif
+	sh = getushell();
+	if (!v_or_fork()) {
+#if defined(HAVE_PUTENV) && WANT_FORK
+		putenv((char *)fname);
+#endif
+		signrm(1);
 		close(0);
 		close(1);
 		close(2);
@@ -1007,20 +1016,17 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 		close(fw);
 		close(fr[1]);
 		close(fr[0]);
-#ifdef HAVE_PUTENV
-		fname = vsncpy(NULL, 0, sc("JOE_FILENAME="));
-		name = bw->b->name ? bw->b->name : (unsigned char *)"Unnamed";
-		if((len = slen(name)) >= 512)	/* limit filename length */
-			len = 512;
-		fname = vsncpy(sv(fname), name, len);
-		putenv((char *)fname);
-#endif
-		sh = getushell();
 		execl(sh, sh, "-c", s, NULL);
 		_exit(0);
 	}
 	close(fr[1]);
 	close(fw);
+#if defined(HAVE_PUTENV) && (WANT_FORK || defined(HAVE_UNSETENV))
+#if !WANT_FORK
+	unsetenv("JOE_FILENAME");
+#endif
+	vsrm(fname);
+#endif
 	if (square) {
 		B *tmp;
 		long width = markk->xcol - markb->xcol;
@@ -1029,7 +1035,7 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 				       markk->line - markb->line + 1,
 				       markk->xcol);
 
-		tmp = bread(fr[0], MAXLONG);
+		tmp = bread(fr[0], LONG_MAX);
 		if (piscol(tmp->eof))
 			height = tmp->eof->line + 1;
 		else
@@ -1056,7 +1062,7 @@ static int dofilt(BW *bw, unsigned char *s, void *object, int *notify)
 		if (!flg)
 			prgetc(p);
 		bdel(markb, p);
-		binsb(p, bread(fr[0], MAXLONG));
+		binsb(p, bread(fr[0], LONG_MAX));
 		if (!flg) {
 			pset(p,markk);
 			prgetc(p);
@@ -1104,27 +1110,22 @@ static int checkmark(BW *bw)
 
 int ufilt(BW *bw)
 {
-#ifdef __MSDOS__
-	msgnw(bw->parent, "Sorry, no sub-processes in DOS (yet)");
-	return -1;
-#else
 	switch (checkmark(bw)) {
 	case 0:
-		if (wmkpw(bw->parent, US "Command to filter block through (^C to abort): ", &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL, locale_map))
+		if (wmkpw(bw->parent, UC "Command to filter block through (^C to abort): ", &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL, locale_map))
 			return 0;
 		else
 			return -1;
 	case 1:
-		if (wmkpw(bw->parent, US "Command to filter file through (^C to abort): ", &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL, locale_map))
+		if (wmkpw(bw->parent, UC "Command to filter file through (^C to abort): ", &filthist, dofilt, NULL, NULL, utypebw, NULL, NULL, locale_map))
 			return 0;
 		else
 			return -1;
 	case 2:
 	default:
-		msgnw(bw->parent, US "No block");
+		msgnw(bw->parent, UC "No block");
 		return -1;
 	}
-#endif
 }
 
 /* Force region to lower case */
@@ -1133,8 +1134,8 @@ int ulower(BW *bw)
 {
 	if (markv(1)) {
 		P *q;
-	        P *p;
-	        int c;
+		P *p;
+		int c;
 		B *b = bcpy(markb,markk);
 		/* Leave one character in buffer to keep pointers set properly... */
 		q = pdup(markk);
@@ -1163,8 +1164,8 @@ int uupper(BW *bw)
 {
 	if (markv(1)) {
 		P *q;
-	        P *p;
-	        int c;
+		P *p;
+		int c;
 		B *b = bcpy(markb,markk);
 		q = pdup(markk);
 		prgetc(q);
