@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/uedit.c,v 1.17 2017/03/19 19:19:51 tg Exp $ */
 /*
  *	Basic user edit functions
  *	Copyright
@@ -9,7 +8,8 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h>
+__RCSID("$MirOS: contrib/code/jupp/uedit.c,v 1.34 2018/02/14 22:28:12 tg Exp $");
+
 #include <string.h>
 
 #ifdef HAVE_BSD_STRING_H
@@ -24,11 +24,11 @@
 #include "qw.h"
 #include "scrn.h"
 #include "ublock.h"
+#include "uedit.h"
 #include "uformat.h"
 #include "umath.h"
 #include "utils.h"
 #include "vs.h"
-#include "utf8.h"
 #include "charmap.h"
 #include "w.h"
 
@@ -192,15 +192,15 @@ int u_goto_prev(BW *bw)
 	struct charmap *map=bw->b->o.charmap;
 	int c = prgetc(p);
 
-	if (joe_isalnum_(map,c)) {
-		while (joe_isalnum_(map,(c=prgetc(p))))
+	if (joe_isalnux(map,c)) {
+		while (joe_isalnux(map,(c=prgetc(p))))
 			/* Do nothing */;
 		if (c != NO_MORE_DATA)
 			pgetc(p);
 	} else if (joe_isspace(map,c) || joe_ispunct(map,c)) {
 		while ((c=prgetc(p)), (joe_isspace(map,c) || joe_ispunct(map,c)))
 			/* Do nothing */;
-		while(joe_isalnum_(map,(c=prgetc(p))))
+		while(joe_isalnux(map,(c=prgetc(p))))
 			/* Do nothing */;
 		if (c != NO_MORE_DATA)
 			pgetc(p);
@@ -229,14 +229,14 @@ int u_goto_next(BW *bw)
 	int c = brch(p);
 	int rtn = -1;
 
-	if (joe_isalnum_(map,c)) {
+	if (joe_isalnux(map,c)) {
 		rtn = 0;
-		while (joe_isalnum_(map,(c = brch(p))))
+		while (joe_isalnux(map,(c = brch(p))))
 			pgetc(p);
 	} else if (joe_isspace(map,c) || joe_ispunct(map,c)) {
 		while (joe_isspace(map, (c = brch(p))) || joe_ispunct(map,c))
 			pgetc(p);
-		while (joe_isalnum_(map,(c = brch(p)))) {
+		while (joe_isalnux(map,(c = brch(p)))) {
 			rtn = 0;
 			pgetc(p);
 		}
@@ -328,7 +328,7 @@ utomatch_i(BW *bw, int dir)
 	P *p;
 	int cnt = 0;		/* delimiter depth */
 
-	switch (c = brch(bw->cursor)) {
+	switch (f = c = brch(bw->cursor)) {
 	case '(':
 		f = ')';
 		dir = 1;
@@ -361,13 +361,6 @@ utomatch_i(BW *bw, int dir)
 		f = '<';
 		dir = -1;
 		break;
-	case '"':
-	case '\'':
-	case '`':
-		f = c;
-		break;
-	default:
-		return -1;
 	}
 
 	p = pdup(bw->cursor);
@@ -606,7 +599,7 @@ void scrdn(BW *bw, int n, int flg)
 
 int upgup(BW *bw)
 {
-	bw = (BW *) bw->parent->main->object;
+	bw = bw->parent->main->object.bw;
 	if (bw->o.hex ? bw->cursor->byte < 16 : !bw->cursor->line)
 		return -1;
 	if (pgamnt < 0)
@@ -622,7 +615,7 @@ int upgup(BW *bw)
 
 int upgdn(BW *bw)
 {
-	bw = (BW *) bw->parent->main->object;
+	bw = bw->parent->main->object.bw;
 	if (bw->o.hex ? bw->cursor->byte/16 == bw->b->eof->byte/16 : bw->cursor->line == bw->b->eof->line)
 		return -1;
 	if (pgamnt < 0)
@@ -638,7 +631,7 @@ int upgdn(BW *bw)
 
 int uupslide(BW *bw)
 {
-	bw = (BW *) bw->parent->main->object;
+	bw = bw->parent->main->object.bw;
 	if (bw->o.hex ? bw->top->byte/16 : bw->top->line) {
 		if (bw->o.hex ? bw->top->byte/16 + bw->h -1 != bw->cursor->byte/16 : bw->top->line + bw->h - 1 != bw->cursor->line)
 			udnarw(bw);
@@ -650,7 +643,7 @@ int uupslide(BW *bw)
 
 int udnslide(BW *bw)
 {
-	bw = (BW *) bw->parent->main->object;
+	bw = bw->parent->main->object.bw;
 	if (bw->o.hex ? bw->top->line/16 + bw->h <= bw->top->b->eof->byte/16 : bw->top->line + bw->h <= bw->top->b->eof->line) {
 		if (bw->o.hex ? bw->top->byte/16 != bw->cursor->byte/16 : bw->top->line != bw->cursor->line)
 			uuparw(bw);
@@ -666,7 +659,7 @@ static B *linehist = NULL;	/* History of previously entered line numbers */
 
 static int doline(BW *bw, unsigned char *s, void *object, int *notify)
 {
-	long num = calc(bw, s);
+	long num = calcl(bw, s);
 
 	if (notify)
 		*notify = 1;
@@ -685,14 +678,14 @@ static int doline(BW *bw, unsigned char *s, void *object, int *notify)
 		if (merrf)
 			msgnw(bw->parent, merrt);
 		else
-			msgnw(bw->parent, US "Invalid line number");
+			msgnw(bw->parent, UC "Invalid line number");
 		return -1;
 	}
 }
 
 int uline(BW *bw)
 {
-	if (wmkpw(bw->parent, US "Go to line (^C to abort): ", &linehist, doline, NULL, NULL, NULL, NULL, NULL, locale_map))
+	if (wmkpw(bw->parent, UC "Go to line (^C to abort): ", &linehist, doline, NULL, NULL, NULL, NULL, NULL, locale_map))
 		return 0;
 	else
 		return -1;
@@ -704,7 +697,7 @@ static B *colhist = NULL;	/* History of previously entered column numbers */
 
 static int docol(BW *bw, unsigned char *s, void *object, int *notify)
 {
-	long num = calc(bw, s);
+	long num = calcl(bw, s);
 
 	if (notify)
 		*notify = 1;
@@ -721,14 +714,14 @@ static int docol(BW *bw, unsigned char *s, void *object, int *notify)
 		if (merrf)
 			msgnw(bw->parent, merrt);
 		else
-			msgnw(bw->parent, US "Invalid column number");
+			msgnw(bw->parent, UC "Invalid column number");
 		return -1;
 	}
 }
 
 int ucol(BW *bw)
 {
-	if (wmkpw(bw->parent, US "Go to column (^C to abort): ", &colhist, docol, NULL, NULL, NULL, NULL, NULL, locale_map))
+	if (wmkpw(bw->parent, UC "Go to column (^C to abort): ", &colhist, docol, NULL, NULL, NULL, NULL, NULL, locale_map))
 		return 0;
 	else
 		return -1;
@@ -740,7 +733,7 @@ static B *bytehist = NULL;	/* History of previously entered byte numbers */
 
 static int dobyte(BW *bw, unsigned char *s, void *object, int *notify)
 {
-	long num = calc(bw, s);
+	long num = calcl(bw, s);
 
 	if (notify)
 		*notify = 1;
@@ -757,14 +750,14 @@ static int dobyte(BW *bw, unsigned char *s, void *object, int *notify)
 		if (merrf)
 			msgnw(bw->parent, merrt);
 		else
-			msgnw(bw->parent, US "Invalid byte number");
+			msgnw(bw->parent, UC "Invalid byte number");
 		return -1;
 	}
 }
 
 int ubyte(BW *bw)
 {
-	if (wmkpw(bw->parent, US "Go to byte (^C to abort): ", &bytehist, dobyte, NULL, NULL, NULL, NULL, NULL, locale_map))
+	if (wmkpw(bw->parent, UC "Go to byte (^C to abort): ", &bytehist, dobyte, NULL, NULL, NULL, NULL, NULL, locale_map))
 		return 0;
 	else
 		return -1;
@@ -870,8 +863,8 @@ int u_word_delete(BW *bw)
 	struct charmap *map=bw->b->o.charmap;
 	int c = brch(p);
 
-	if (joe_isalnum_(map,c))
-		while (joe_isalnum_(map,(c = brch(p))))
+	if (joe_isalnux(map,c))
+		while (joe_isalnux(map,(c = brch(p))))
 			pgetc(p);
 	else if (joe_isspace(map,c))
 		while (joe_isspace(map,(c = brch(p))))
@@ -898,8 +891,8 @@ int ubackw(BW *bw)
 	int c = prgetc(bw->cursor);
 	struct charmap *map=bw->b->o.charmap;
 
-	if (joe_isalnum_(map,c)) {
-		while (joe_isalnum_(map,(c = prgetc(bw->cursor))))
+	if (joe_isalnux(map,c)) {
+		while (joe_isalnux(map,(c = prgetc(bw->cursor))))
 			/* do nothing */;
 		if (c != NO_MORE_DATA)
 			pgetc(bw->cursor);
@@ -979,7 +972,8 @@ int uinsc(BW *bw)
 
 /* Move p backwards to first non-blank line and return its indentation */
 
-int find_indent(P *p)
+static int
+find_indent(P *p)
 {
 	int x;
 	for (x=0; x != 10; ++x) {
@@ -1063,7 +1057,6 @@ int utypebw_raw(BW *bw, int k, int no_decode)
 		while (n--)
 			utypebw_raw(bw, ' ', 0);
 	} else {
-		int upd;
 		int simple;
 		int x;
 
@@ -1081,7 +1074,6 @@ int utypebw_raw(BW *bw, int k, int no_decode)
 				return 0;
 		}
 
-		upd = bw->parent->t->t->updtab[bw->y + bw->cursor->line - bw->top->line];
 		simple = 1;
 
 		if (pisblank(bw->cursor))
@@ -1127,7 +1119,6 @@ int utypebw_raw(BW *bw, int k, int no_decode)
 		}
 
 		bw->cursor->xcol = piscol(bw->cursor);
-#ifndef __MSDOS__
 		if (x < 0 || x >= bw->w)
 			simple = 0;
 		if (bw->cursor->line < bw->top->line || bw->cursor->line >= bw->top->line + bw->h)
@@ -1152,7 +1143,8 @@ int utypebw_raw(BW *bw, int k, int no_decode)
 			int *attr = t->attr + y * t->co;
 			x += bw->x;
 
-			if (!upd && piseol(bw->cursor) && !bw->o.highlight)
+			if (!bw->parent->t->t->updtab[bw->y + bw->cursor->line - bw->top->line] &&
+			    piseol(bw->cursor) && !bw->o.highlight)
 				t->updtab[y] = 0;
 			if (markb &&
 			    markk &&
@@ -1163,14 +1155,13 @@ int utypebw_raw(BW *bw, int k, int no_decode)
 				atr = INVERSE;
 			outatr(bw->b->o.charmap, t, screen + x, attr + x, x, y, no_decode == 2 ? 0xFFFD : k, atr);
 		}
-#endif
 	}
 	return 0;
 }
 
-int utypebw(BW *bw, int k)
+int utypebw(jobject jO, int k)
 {
-	return utypebw_raw(bw, k, 0);
+	return utypebw_raw(jO.bw, k, 0);
 }
 
 /* Quoting */
@@ -1181,7 +1172,7 @@ static int dounicode(BW *bw, unsigned char *s, void *object, int *notify)
 {
 	int num;
 
-	sscanf((char *)s,"%x",&num);
+	num = ustolb(s, NULL, 0, 0x10FFFF, USTOL_HEX | USTOL_TRIM | USTOL_EOS);
 	if (notify)
 		*notify = 1;
 	vsrm(s);
@@ -1199,10 +1190,23 @@ static int dounicode(BW *bw, unsigned char *s, void *object, int *notify)
 	return 0;
 }
 
+static void
+doquote0(BW *bw, int c, int meta)
+{
+	if (c == '?')
+		c = 127;
+	else if ((c >= 0x40 && c <= 0x5F) || (c >= 'a' && c <= 'z'))
+		c &= 0x1F;
+	c |= meta;
+	utypebw_raw(bw, c, 1);
+	bw->cursor->xcol = piscol(bw->cursor);
+}
+
 int quotestate;
 int quoteval;
 
-static int doquote(BW *bw, int c, void *object, int *notify)
+static int
+doquote(BW *bw, int c, void *object, int *notify)
 {
 	unsigned char buf[40];
 
@@ -1220,16 +1224,16 @@ static int doquote(BW *bw, int c, void *object, int *notify)
 				return -1;
 			else
 				return 0;
-		} else if (c == 'u' || c == 'U') {
+		} else if ((c | 0x20) == 'u') {
 			if (bw->b->o.charmap->type)
 				goto unopoo;
  uhex_uni:
-			if (!wmkpw(bw->parent, US "Unicode (ISO-10646) character in hex (^C to abort): ", &unicodehist, dounicode,
-			           NULL, NULL, NULL, NULL, NULL, locale_map))
+			if (!wmkpw(bw->parent, UC "Unicode (ISO-10646) character in hex (^C to abort): ", &unicodehist, dounicode,
+				   NULL, NULL, NULL, NULL, NULL, locale_map))
 				return 0;
 			else
 				return -1;
-		} else if (c == 'r' || c == 'R') {
+		} else if ((c | 0x20) == 'r') {
 			if (!bw->b->o.charmap->type)
 				goto unopoo;
  uhex_raw:
@@ -1238,12 +1242,12 @@ static int doquote(BW *bw, int c, void *object, int *notify)
 				return -1;
 			else
 				return 0;
-		} else if (c == 'x' || c == 'X') {
+		} else if ((c | 0x20) == 'x') {
 			if (bw->b->o.charmap->type)
 				goto uhex_uni;
 			else
 				goto uhex_raw;
-		} else if (c == 'o' || c == 'O') {
+		} else if ((c | 0x20) == 'o') {
 			quotestate = 5;
 			if (!mkqwna(bw->parent, sc("ASCII 0---"), doquote, NULL, NULL, notify))
 				return -1;
@@ -1251,12 +1255,7 @@ static int doquote(BW *bw, int c, void *object, int *notify)
 				return 0;
 		} else {
  unopoo:
-			if ((c >= 0x40 && c <= 0x5F) || (c >= 'a' && c <= 'z'))
-				c &= 0x1F;
-			if (c == '?')
-				c = 127;
-			utypebw_raw(bw, c, 1);
-			bw->cursor->xcol = piscol(bw->cursor);
+			doquote0(bw, c, 0);
 		}
 		break;
 	case 1:
@@ -1279,30 +1278,20 @@ static int doquote(BW *bw, int c, void *object, int *notify)
 		break;
 	case 3:
 		if (c >= '0' && c <= '9') {
-			joe_snprintf_1((char *)buf, sizeof(buf), "ASCII 0x%c-", c);
 			quoteval = c - '0';
-			quotestate = 4;
-			if (!mkqwna(bw->parent, sz(buf), doquote, NULL, NULL, notify))
-				return -1;
-			else
-				return 0;
-		} else if (c >= 'a' && c <= 'f') {
-			joe_snprintf_1((char *)buf, sizeof(buf), "ASCII 0x%c-", c + 'A' - 'a');
-			quoteval = c - 'a' + 10;
-			quotestate = 4;
-			if (!mkqwna(bw->parent, sz(buf), doquote, NULL, NULL, notify))
-				return -1;
-			else
-				return 0;
-		} else if (c >= 'A' && c <= 'F') {
+ uhex_3:
 			joe_snprintf_1((char *)buf, sizeof(buf), "ASCII 0x%c-", c);
-			quoteval = c - 'A' + 10;
 			quotestate = 4;
 			if (!mkqwna(bw->parent, sz(buf), doquote, NULL, NULL, notify))
 				return -1;
 			else
 				return 0;
 		}
+		c &= ~0x20;
+		if (c >= 'A' && c <= 'F') {
+			quoteval = c - 'A' + 10;
+			goto uhex_3;
+		}
 		break;
 	case 4:
 		if (c >= '0' && c <= '9') {
@@ -1353,32 +1342,28 @@ static int doquote(BW *bw, int c, void *object, int *notify)
 	return 0;
 }
 
-int uquote(BW *bw)
+static char uquote_txt[] =
+    "Ctrl- (or 0-9 for dec. x for hex, o for octal ASCII, u for hex UTF-8)";
+int
+uquote(BW *bw)
 {
-	const char *qs;
-
-	if (bw->b->o.charmap->type)
-		qs = "Ctrl- (or 0-9 for dec. r for hex, o for octal ASCII, x for hex UTF-8)";
-	else
-		qs = "Ctrl- (or 0-9 for dec. x for hex, o for octal ASCII, u for hex UTF-8)";
 	quotestate = 0;
-	if (mkqwna(bw->parent, US qs, strlen(qs), doquote, NULL, NULL, NULL))
-		return 0;
-	else
-		return -1;
+	if (bw->b->o.charmap->type) {
+		uquote_txt[23] = 'r';
+		uquote_txt[53] = 'x';
+	} else {
+		uquote_txt[23] = 'x';
+		uquote_txt[53] = 'u';
+	}
+	return (mkqwna(bw->parent, sc(uquote_txt),
+	    doquote, NULL, NULL, NULL) ? 0 : -1);
 }
 
 static int doquote9(BW *bw, int c, void *object, int *notify)
 {
 	if (notify)
 		*notify = 1;
-	if ((c >= 0x40 && c <= 0x5F) || (c >= 'a' && c <= 'z'))
-		c &= 0x1F;
-	if (c == '?')
-		c = 127;
-	c |= 128;
-	utypebw_raw(bw, c, 1);
-	bw->cursor->xcol = piscol(bw->cursor);
+	doquote0(bw, c, 128);
 	return 0;
 }
 
@@ -1416,8 +1401,8 @@ static int doctrl(BW *bw, int c, void *object, int *notify)
 		*notify = 1;
 	bw->o.overtype = 0;
 	if (bw->parent->huh == srchstr && c == '\n') {
-		utypebw(bw, '\\');
-		utypebw(bw, 'n');
+		utypebw_raw(bw, '\\', 0);
+		utypebw_raw(bw, 'n', 0);
 	} else
 		utypebw_raw(bw, c, 1);
 	bw->o.overtype = org;
@@ -1436,8 +1421,10 @@ int uctrl(BW *bw)
 /* User hit Return.  Deal with autoindent.
  */
 
-int rtntw(BW *bw)
+int rtntw(jobject jO)
 {
+	BW *bw = jO.bw;
+
 	if (bw->o.overtype) {
 		p_goto_eol(bw->cursor);
 		if (piseof(bw->cursor))
@@ -1558,7 +1545,7 @@ static int dofwrdc(BW *bw, int k, void *object, int *notify)
 				break;
 	}
 	if (c == NO_MORE_DATA) {
-		msgnw(bw->parent, US "Not found");
+		msgnw(bw->parent, UC "Not found");
 		prm(q);
 		return -1;
 	} else {
@@ -1605,7 +1592,7 @@ static int domsg(BASE *b, unsigned char *s, void *object, int *notify)
 
 int umsg(BASE *b)
 {
-	if (wmkpw(b->parent, US "Msg (^C to abort): ", NULL, domsg, NULL, NULL, NULL, NULL, NULL, locale_map))
+	if (wmkpw(b->parent, UC "Msg (^C to abort): ", NULL, domsg, NULL, NULL, NULL, NULL, NULL, locale_map))
 		return 0;
 	else
 		return -1;
@@ -1620,14 +1607,14 @@ static int dotxt(BW *bw, unsigned char *s, void *object, int *notify)
 	if (notify)
 		*notify = 1;
 	for (x = 0; x != sLEN(s); ++x)
-		utypebw(bw, s[x]);
+		utypebw_raw(bw, s[x], 0);
 	vsrm(s);
 	return 0;
 }
 
 int utxt(BW *bw)
 {
-	if (wmkpw(bw->parent, US "Insert (^C to abort): ", NULL, dotxt, NULL, NULL, utypebw, NULL, NULL, bw->b->o.charmap))
+	if (wmkpw(bw->parent, UC "Insert (^C to abort): ", NULL, dotxt, NULL, NULL, utypebw, NULL, NULL, bw->b->o.charmap))
 		return 0;
 	else
 		return -1;
