@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/uisrch.c,v 1.7 2017/01/10 19:16:28 tg Exp $ */
 /*
  *	Incremental search
  *	Copyright
@@ -9,6 +8,8 @@
 #include "config.h"
 #include "types.h"
 
+__RCSID("$MirOS: contrib/code/jupp/uisrch.c,v 1.15 2017/12/08 02:28:08 tg Exp $");
+
 #include <stdlib.h>
 
 #include "b.h"
@@ -17,10 +18,10 @@
 #include "queue.h"
 #include "qw.h"
 #include "tty.h"
+#include "uisrch.h"
 #include "usearch.h"
 #include "utils.h"
 #include "charmap.h"
-#include "utf8.h"
 #include "vs.h"
 
 extern int smode;
@@ -32,7 +33,7 @@ unsigned char *lastpat = NULL;	/* Previous pattern */
 
 extern SRCH *globalsrch;	/* Existing SRCH structure */
 
-IREC fri = { {&fri, &fri} };	/* Free-list of irecs */
+IREC fri = { {&fri, &fri}, 0, 0, 0, 0 };	/* Free-list of irecs */
 
 static IREC *alirec(void)
 {				/* Allocate an IREC */
@@ -50,7 +51,7 @@ static void rmisrch(struct isrch *isrch)
 		vsrm(isrch->pattern);
 		vsrm(isrch->prompt);
 		frchn(&fri, &isrch->irecs);
-		joe_free(isrch);
+		free(isrch);
 	}
 }
 
@@ -109,6 +110,7 @@ static int itype(BW *bw, int c, struct isrch *isrch, int *notify)
 {
 	IREC *i;
 	int omid;
+	int tc;
 
 	if (isrch->quote) {
 		goto in;
@@ -195,9 +197,7 @@ static int itype(BW *bw, int c, struct isrch *isrch, int *notify)
 		unsigned char buf[16];
 		int buf_len;
 		/* Search */
-
-		in:
-
+ in:
 		/* Convert to/from utf-8 */
 		if (locale_map->type && !bw->b->o.charmap->type) {
 			utf8_encode(buf,c);
@@ -212,7 +212,7 @@ static int itype(BW *bw, int c, struct isrch *isrch, int *notify)
 		} else {
 			buf[0] = c;
 			buf_len = 1;
-		}		
+		}
 
 		isrch->quote = 0;
 		iappend(bw, isrch, buf, buf_len);
@@ -229,20 +229,21 @@ static int itype(BW *bw, int c, struct isrch *isrch, int *notify)
 		/* Translate bytes to utf-8 */
 		unsigned char buf[16];
 		int x;
-		for (x=0; x!=sLEN(isrch->pattern); ++x) {
-			int c_ = to_uni(bw->b->o.charmap, isrch->pattern[x]);
-			utf8_encode(buf,c_);
-			isrch->prompt = vsncpy(sv(isrch->prompt),sz(buf));
+
+		for (x = 0; x != sLEN(isrch->pattern); ++x) {
+			tc = to_uni(bw->b->o.charmap, isrch->pattern[x]);
+			utf8_encode(buf, tc);
+			isrch->prompt = vsncpy(sv(isrch->prompt), sz(buf));
 		}
 	} else if (!locale_map->type && bw->b->o.charmap->type) {
 		/* Translate utf-8 to bytes */
 		unsigned char *p = isrch->pattern;
 		int len = sLEN(isrch->pattern);
+
 		while (len) {
-			int c_ = utf8_decode_fwrd(&p, &len);
-			if (c_ >= 0) {
-				c_ = from_uni(locale_map, c_);
-				isrch->prompt = vsadd(isrch->prompt, c_);
+			if ((tc = utf8_decode_fwrd(&p, &len)) >= 0) {
+				tc = from_uni(locale_map, tc);
+				isrch->prompt = vsadd(isrch->prompt, tc);
 			}
 		}
 	} else {
@@ -258,9 +259,11 @@ static int itype(BW *bw, int c, struct isrch *isrch, int *notify)
 	}
 }
 
-static int doisrch(BW *bw, int dir)
-{				/* Create a struct isrch */
-	struct isrch *isrch = (struct isrch *) joe_malloc(sizeof(struct isrch));
+/* Create a struct isrch */
+static int
+doisrch(BW *bw, int dir)
+{
+	struct isrch *isrch = malloc(sizeof(struct isrch));
 
 	izque(IREC, link, &isrch->irecs);
 	isrch->pattern = vsncpy(NULL, 0, NULL, 0);
