@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/uisrch.h,v 1.2 2008/05/13 13:08:28 tg Exp $ */
 /*
  *	Incremental search
  *	Copyright
@@ -9,10 +8,11 @@
 #ifndef _JOE_UISRCH_H
 #define _JOE_UISRCH_H 1
 
-#include "config.h"
-#include "types.h"
+#ifdef EXTERN_CMD_C
+__IDSTRING(rcsid_uisrch_h, "$MirOS: contrib/code/jupp/uisrch.h,v 1.5 2017/12/06 21:17:02 tg Exp $");
+#endif
 
-int uisrch PARAMS((BW *bw));
-int ursrch PARAMS((BW *bw));
+int uisrch(BW *bw);
+int ursrch(BW *bw);
 
 #endif
