@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/umath.h,v 1.5 2017/03/19 19:19:51 tg Exp $ */
 /*
  *	Math
  *	Copyright
@@ -9,19 +8,27 @@
 #ifndef _JOE_UMATH_H
 #define _JOE_UMATH_H 1
 
-#include "config.h"
-#include "types.h"
-
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
+#ifdef EXTERN_CMD_C
+__IDSTRING(rcsid_umath_h, "$MirOS: contrib/code/jupp/umath.h,v 1.11 2017/12/06 21:17:03 tg Exp $");
 #endif
 
-extern volatile sig_atomic_t merrf;       
+#include <signal.h>
+
+extern volatile sig_atomic_t merrf;
 extern const unsigned char *merrt;
 
-double calc(BW *bw, unsigned char *s);
+long calcl(BW *bw, unsigned char *s);
+
+#if WANT_MATH
 int umath(BW *bw);
 int umathins(BW *bw);
 int umathres(BW *bw);
+#else
+int unomath(BW *bw);
+
+#define umath		unomath
+#define umathins	unomath
+#define umathres	unomath
+#endif
 
 #endif
