@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/usearch.c,v 1.10 2017/01/11 22:56:50 tg Exp $ */
 /*
  *	Search & Replace system
  *	Copyright
@@ -9,7 +8,8 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h>
+__RCSID("$MirOS: contrib/code/jupp/usearch.c,v 1.21 2017/12/16 22:10:55 tg Exp $");
+
 #include <stdlib.h>
 
 #include "b.h"
@@ -42,14 +42,15 @@ B *replhist = NULL;		/* Replacement string history */
 
 SRCH *globalsrch = NULL;	/* Most recent completed search data */
 
-SRCHREC fsr = { {&fsr, &fsr} };
+SRCHREC fsr = { {&fsr, &fsr}, 0, 0, 0 };
 
 /* Completion stuff: should go somewhere else */
 
 unsigned char **word_list;
 
 #define MAX_WORD_SIZE 64
-unsigned char **get_word_list(B *b,int ignore)
+static unsigned char **
+get_word_list(B *b, int ignore)
 {
 	unsigned char buf[MAX_WORD_SIZE];
 	unsigned char *s;
@@ -67,7 +68,7 @@ unsigned char **get_word_list(B *b,int ignore)
 	idx = 0;
 	while ((c=pgetc(p))!=NO_MORE_DATA)
 		if (idx) {
-			if (joe_isalnum_(b->o.charmap, c)) {
+			if (joe_isalnux(b->o.charmap, c)) {
 				if (idx!=MAX_WORD_SIZE)
 					buf[idx++] = c;
 			} else {
@@ -82,30 +83,31 @@ unsigned char **get_word_list(B *b,int ignore)
 			}
 		} else {
 			start=p->byte-1;
-			if (joe_isalpha_(b->o.charmap, c))
+			if (joe_isalphx(b->o.charmap, c))
 				buf[idx++] = c;
 		}
 	prm(p);
 
 	for (idx = 0;idx != h->len;++idx)
 		for (t = h->tab[idx];t;t=t->next)
-			list = vaadd(list, t->name);
+			list = vaadd(list, /* checked */ US t->name);
 	if (list)
-		vasort(list,sLEN(list));	
+		vasort(list, sLEN(list));
 
 	htrm(h);
 
 	return list;
 }
 
-void fcmplt_ins(BW *bw, unsigned char *line)
+static void
+fcmplt_ins(BW *bw, unsigned char *line)
 {
 	P *p;
 	int c;
 
 	if (!piseol(bw->cursor)) {
 		c = brch(bw->cursor);
-		if (joe_isalnum_(bw->b->o.charmap,c))
+		if (joe_isalnux(bw->b->o.charmap,c))
 			return;
 	}
 
@@ -114,7 +116,7 @@ void fcmplt_ins(BW *bw, unsigned char *line)
 	p = pdup(bw->cursor);
 	do
 		c = prgetc(p);
-		while (joe_isalnum_(bw->b->o.charmap,c));
+		while (joe_isalnux(bw->b->o.charmap,c));
 	if (c!=NO_MORE_DATA)
 		pgetc(p);
 
@@ -130,7 +132,8 @@ void fcmplt_ins(BW *bw, unsigned char *line)
 	}
 }
 
-int fcmplt_abrt(BW *bw, int x, unsigned char *line)
+static int
+fcmplt_abrt(BW *bw, int x, unsigned char *line)
 {
 	if (line) {
 		fcmplt_ins(bw, line);
@@ -139,9 +142,10 @@ int fcmplt_abrt(BW *bw, int x, unsigned char *line)
 	return -1;
 }
 
-int fcmplt_rtn(MENU *m, int x, unsigned char *line)
+static int
+fcmplt_rtn(MENU *m, int x, unsigned char *line)
 {
-	fcmplt_ins(m->parent->win->object, m->list[x]);
+	fcmplt_ins(m->parent->win->object.bw, m->list[x]);
 	vsrm(line);
 	m->object = NULL;
 	wabort(m->parent);
@@ -161,7 +165,7 @@ int ufinish(BW *bw)
 
 	if (!piseol(bw->cursor)) {
 		c = brch(bw->cursor);
-		if (joe_isalnum_(bw->b->o.charmap,c))
+		if (joe_isalnux(bw->b->o.charmap,c))
 			return -1;
 	}
 
@@ -170,7 +174,7 @@ int ufinish(BW *bw)
 	p = pdup(bw->cursor);
 	do
 		c = prgetc(p);
-		while (joe_isalnum_(bw->b->o.charmap,c));
+		while (joe_isalnux(bw->b->o.charmap,c));
 	if (c!=NO_MORE_DATA)
 		pgetc(p);
 
@@ -232,8 +236,11 @@ int ufinish(BW *bw)
 
 static int srch_cmplt(BW *bw)
 {
-	utypebw(bw, 9);
-		return 0;
+	jobject jO;
+
+	jO.bw = bw;
+	utypebw(jO, 9);
+	return 0;
 }
 
 /* Search forward.
@@ -262,7 +269,7 @@ static P *searchf(BW *bw,SRCH *srch, P *p)
 	for (x = 0; x != sLEN(pattern) && pattern[x] != '\\' && (pattern[x]<128 || !p->b->o.charmap->type); ++x)
 		if (srch->ignore)
 			pattern[x] = joe_tolower(p->b->o.charmap,pattern[x]);
-	wrapped:
+ wrapped:
 	while (srch->ignore ? pifind(start, pattern, x) : pfind(start, pattern, x)) {
 		pset(end, start);
 		pfwrd(end, (long) x);
@@ -280,7 +287,7 @@ static P *searchf(BW *bw,SRCH *srch, P *p)
 			break;
 	}
 	if (wrap && !srch->wrap_flag && srch->wrap_p) {
-		msgnw(bw->parent, US "Wrapped");
+		msgnw(bw->parent, UC "Wrapped");
 		srch->wrap_flag = 1;
 		p_goto_bof(start);
 		goto wrapped;
@@ -319,7 +326,7 @@ static P *searchb(BW *bw,SRCH *srch, P *p)
 		if (srch->ignore)
 			pattern[x] = joe_tolower(p->b->o.charmap,pattern[x]);
 
-	wrapped:
+ wrapped:
 	while (pbkwd(start, 1L)
 	       && (srch->ignore ? prifind(start, pattern, x) : prfind(start, pattern, x))) {
 		pset(end, start);
@@ -337,7 +344,7 @@ static P *searchb(BW *bw,SRCH *srch, P *p)
 	}
 
 	if (wrap && !srch->wrap_flag && srch->wrap_p) {
-		msgnw(bw->parent, US "Wrapped");
+		msgnw(bw->parent, UC "Wrapped");
 		srch->wrap_flag = 1;
 		p_goto_eof(start);
 		goto wrapped;
@@ -370,7 +377,7 @@ static SRCH *setmark(SRCH *srch)
 
 SRCH *mksrch(unsigned char *pattern, unsigned char *replacement, int ignore, int backwards, int repeat, int replace, int rest)
 {
-	SRCH *srch = (SRCH *) joe_malloc(sizeof(SRCH));
+	SRCH *srch = malloc(sizeof(SRCH));
 	int x;
 
 	srch->pattern = pattern;
@@ -420,7 +427,7 @@ void rmsrch(SRCH *srch)
 	vsrm(srch->pattern);
 	vsrm(srch->replacement);
 	vsrm(srch->entire);
-	joe_free(srch);
+	free(srch);
 	updall();
 }
 
@@ -440,8 +447,8 @@ static P *insert(SRCH *srch, P *p, unsigned char *s, int len)
 			len -= x;
 			s += x;
 		} else if (len >= 2) {
-			if (((s[1] >= 'a' && s[1] <= 'z') || (s[1] >= 'A' && s[1] <= 'Z'))
-				 && srch->pieces[(s[1] & 0x1f) - 1]) {
+			if (((s[1] | 0x20) >= 'a' && (s[1] | 0x20) <= 'z') &&
+			    srch->pieces[(s[1] & 0x1f) - 1]) {
 				binsm(p, sv(srch->pieces[(s[1] & 0x1f) - 1]));
 				pfwrd(p, (long) sLEN(srch->pieces[(s[1] & 0x1f) - 1]));
 				s += 2;
@@ -529,25 +536,20 @@ static int set_options(BW *bw, unsigned char *s, SRCH *srch, int *notify)
 	srch->ignore = icase;
 
 	for (x = 0; s[x]; ++x) {
-		switch (s[x]) {
+		switch (s[x] | 0x20) {
 		case 'r':
-		case 'R':
 			srch->replace = 1;
 			break;
 		case 'b':
-		case 'B':
 			srch->backwards = 1;
 			break;
 		case 'i':
-		case 'I':
 			srch->ignore = 1;
 			break;
 		case 's':
-		case 'S':
 			srch->ignore = 0;
 			break;
 		case 'k':
-		case 'K':
 			srch->block_restrict = 1;
 			break;
 		case '0':
@@ -568,7 +570,7 @@ static int set_options(BW *bw, unsigned char *s, SRCH *srch, int *notify)
 	}
 	vsrm(s);
 	if (srch->replace) {
-		if (wmkpw(bw->parent, US "Replace with (^C to abort): ", &replhist, set_replace, srchstr, pfabort, srch_cmplt, srch, notify, bw->b->o.charmap))
+		if (wmkpw(bw->parent, UC "Replace with (^C to abort): ", &replhist, set_replace, srchstr, pfabort, srch_cmplt, srch, notify, bw->b->o.charmap))
 			return 0;
 		else
 			return -1;
@@ -579,12 +581,12 @@ static int set_options(BW *bw, unsigned char *s, SRCH *srch, int *notify)
 static int set_pattern(BW *bw, unsigned char *s, SRCH *srch, int *notify)
 {
 	BW *pbw;
-	unsigned char *p;
+	const unsigned char *p;
 
 	if (icase)
-		p = US "case (S)ensitive (R)eplace (B)ackwards Bloc(K) NNN (^C to abort): ";
+		p = UC "case (S)ensitive (R)eplace (B)ackwards Bloc(K) NNN (^C to abort): ";
 	else
-		p = US "(I)gnore (R)eplace (B)ackwards Bloc(K) NNN (^C to abort): ";
+		p = UC "(I)gnore (R)eplace (B)ackwards Bloc(K) NNN (^C to abort): ";
 
 	vsrm(srch->pattern);
 	srch->pattern = s;
@@ -623,19 +625,21 @@ static int dofirst(BW *bw, int back, int repl)
 	}
 	if (bw->parent->huh == srchstr) {
 		long byte;
+		jobject jO;
 
 		p_goto_eol(bw->cursor);
 		byte = bw->cursor->byte;
 		p_goto_bol(bw->cursor);
 		if (byte == bw->cursor->byte)
 			prgetc(bw->cursor);
-		return urtn((BASE *)bw, -1);
+		jO.bw = bw;
+		return urtn(jO, -1);
 	}
 	srch = setmark(mksrch(NULL, NULL, 0, back, -1, repl, 0));
 	srch->addr = bw->cursor->byte;
 	srch->wrap_p = pdup(bw->cursor);
 	srch->wrap_p->owner = &srch->wrap_p;
-	if (wmkpw(bw->parent, US "Find (^C to abort): ", &findhist, set_pattern, srchstr, pfabort, srch_cmplt, srch, NULL, bw->b->o.charmap))
+	if (wmkpw(bw->parent, UC "Find (^C to abort): ", &findhist, set_pattern, srchstr, pfabort, srch_cmplt, srch, NULL, bw->b->o.charmap))
 		return 0;
 	else {
 		rmsrch(srch);
@@ -665,7 +669,7 @@ static int doreplace(BW *bw, SRCH *srch)
 	P *q;
 
 	if (bw->b->rdonly) {
-		msgnw(bw->parent, US "Read only");
+		msgnw(bw->parent, UC "Read only");
 		return -1;
 	}
 	if (markk)
@@ -718,19 +722,19 @@ static void goback(SRCH *srch, BW *bw)
 static int dopfrepl(BW *bw, int c, SRCH *srch, int *notify)
 {
 	srch->addr = bw->cursor->byte;
-	if (c == 'N' || c == 'n')
+	if ((c | 0x20) == 'n')
 		return dopfnext(bw, srch, notify);
-	else if (c == 'Y' || c == 'y' || c == ' ' || c == 'L' || c == 'l') {
+	else if ((c | 0x20) == 'y' || (c | 0x20) == 'l' || c == ' ') {
 		srch->recs.link.prev->yn = 1;
 		/* why do I return -1 on 'L' here? */
-		return ((doreplace(bw, srch) || c == 'L' || c == 'l') ?
+		return ((doreplace(bw, srch) || (c | 0x20) == 'l') ?
 		    pfsave(bw, srch) : dopfnext(bw, srch, notify));
-	} else if (c == 'R' || c == 'r') {
+	} else if ((c | 0x20) == 'r') {
 		if (doreplace(bw, srch))
 			return -1;
 		srch->rest = 1;
 		return dopfnext(bw, srch, notify);
-	} else if (c == 8 || c == 127 || c == 'b' || c == 'B') {
+	} else if (c == 8 || c == 127 || (c | 0x20) == 'b') {
 		goback(srch, bw);
 		goback(srch, bw);
 		return dopfnext(bw, srch, notify);
@@ -797,14 +801,15 @@ static int fnext(BW *bw, SRCH *srch)
 {
 	P *sta;
 
-      next:
+ next:
 	if (srch->repeat != -1) {
 		if (!srch->repeat)
 			return 0;
 		else
 			--srch->repeat;
 	}
-      again:if (srch->backwards)
+ again:
+	if (srch->backwards)
 		sta = searchb(bw, srch, bw->cursor);
 	else
 		sta = searchf(bw, srch, bw->cursor);
@@ -850,15 +855,17 @@ int dopfnext(BW *bw, SRCH *srch, int *notify)
 		smode = 2;	/* We have started a search mode */
 	if (srch->replace)
 		visit(srch, bw, 0);
-again:	switch (fnext(bw, srch)) {
+ again:
+	switch (fnext(bw, srch)) {
 	case 0:
 		break;
 	case 1:
-bye:		if (!srch->flg && !srch->rest) {
+ bye:
+		if (!srch->flg && !srch->rest) {
 			if (srch->valid && srch->block_restrict)
-				msgnw(bw->parent, US "Not found (search restricted to marked block)");
+				msgnw(bw->parent, UC "Not found (search restricted to marked block)");
 			else
-				msgnw(bw->parent, US "Not found");
+				msgnw(bw->parent, UC "Not found");
 			ret = -1;
 		}
 		break;
@@ -927,20 +934,22 @@ bye:		if (!srch->flg && !srch->rest) {
 
 int pfnext(BW *bw)
 {
-	if (!globalsrch)	/* Query for search string if there isn't any */
-		return pffirst(bw);
-	else {
-		SRCH *srch = globalsrch;
+	SRCH *srch;
 
-		globalsrch = NULL;
-		srch->addr = bw->cursor->byte;
-		if (!srch->wrap_p || srch->wrap_p->b!=bw->b) {
-			prm(srch->wrap_p);
-			srch->wrap_p = pdup(bw->cursor);
-			srch->wrap_p->owner = &srch->wrap_p;
-			srch->wrap_flag = 0;
-		}
-		srch->valid = 0;
-		return dopfnext(bw, setmark(srch), NULL);
+	if (!globalsrch) {
+		/* Query for search string if there isn't any */
+		return pffirst(bw);
 	}
+
+	srch = globalsrch;
+	globalsrch = NULL;
+	srch->addr = bw->cursor->byte;
+	if (!srch->wrap_p || srch->wrap_p->b!=bw->b) {
+		prm(srch->wrap_p);
+		srch->wrap_p = pdup(bw->cursor);
+		srch->wrap_p->owner = &srch->wrap_p;
+		srch->wrap_flag = 0;
+	}
+	srch->valid = 0;
+	return dopfnext(bw, setmark(srch), NULL);
 }
