@@ -1,5 +1,4 @@
-/* $MirOS: contrib/code/jupp/utag.h,v 1.2 2008/05/13 13:08:30 tg Exp $ */
-/* 
+/*
  *	tags file symbol lookup
  *	Copyright
  *		(C) 1992 Joseph H. Allen
@@ -9,9 +8,10 @@
 #ifndef _JOE_UTAG_H
 #define _JOE_UTAG_H 1
 
-#include "config.h"
-#include "types.h"
+#ifdef EXTERN_CMD_C
+__IDSTRING(rcsid_utag_h, "$MirOS: contrib/code/jupp/utag.h,v 1.6 2017/12/06 23:17:36 tg Exp $");
+#endif
 
-int utag PARAMS((BW *bw));
+int utag(BW *bw);
 
 #endif
