@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/utf8.c,v 1.19 2017/01/11 21:48:58 tg Exp $ */
 /*
  *	UTF-8 Utilities
  *	Copyright
@@ -11,12 +10,10 @@
 #include "config.h"
 #include "types.h"
 
-#include <stdio.h>
-#include <string.h>
+__RCSID("$MirOS: contrib/code/jupp/utf8.c,v 1.23 2017/12/20 23:53:29 tg Exp $");
 
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
+#include <string.h>
 
 #ifdef __CYGWIN__
 #include <cygwin/version.h>
@@ -62,7 +59,6 @@
 #endif
 
 #include "rc.h"
-#include "utf8.h"
 #include "charmap.h"
 
 /* UTF-8 Encoder
@@ -269,9 +265,7 @@ joe_locale(void)
 #ifdef USE_LOCALE
 	if (!locale_map) {
 		setlocale(LC_ALL,"");
-		s = (unsigned char *)strdup(nl_langinfo(CODESET));
-
-		locale_map = find_charmap(s);
+		locale_map = find_charmap((const void *)nl_langinfo(CODESET));
 	}
 #else
 	if (!locale_map && s) {
@@ -287,8 +281,8 @@ joe_locale(void)
 	}
 #endif
 	if (!locale_map)
-		locale_map = find_charmap(US "ascii");
-	utf8_map = find_charmap(US "utf-8");
+		locale_map = find_charmap(UC "ascii");
+	utf8_map = find_charmap(UC "utf-8");
 
 #ifndef TEST
 #ifdef defutf8
