@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/utils.c,v 1.4 2017/01/10 19:05:17 tg Exp $ */
 /*
  *	Various utilities
  *	Copyright
@@ -9,16 +8,12 @@
  */
 #include "config.h"
 
+__RCSID("$MirOS: contrib/code/jupp/utils.c,v 1.10 2018/01/06 00:28:34 tg Exp $");
+
 #include <errno.h>
-#ifdef HAVE_UNISTD_H
+#include <stdlib.h>
 #include <unistd.h>
-#endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
 
-#include "i18n.h"
-#include "utf8.h"
 #include "charmap.h"
 #include "blocks.h"
 #include "utils.h"
@@ -106,7 +101,8 @@ int joe_set_signal(int signum, sighandler_t handler)
 
 /* Skip whitespace and return first non-whitespace character */
 
-int parse_ws(unsigned char **pp,int cmt)
+int
+parse_ws(unsigned char **pp, int cmt)
 {
 	unsigned char *p = *pp;
 	while (*p==' ' || *p=='\t')
@@ -119,14 +115,15 @@ int parse_ws(unsigned char **pp,int cmt)
 
 /* Parse an identifier into a buffer.  Identifier is truncated to a maximum of len chars. */
 
-int parse_ident(unsigned char **pp, unsigned char *buf, int len)
+int
+parse_ident(unsigned char **pp, unsigned char *buf, int len)
 {
 	unsigned char *p = *pp;
-	if (joe_isalpha_(locale_map,*p)) {
-		while(len && joe_isalnum_(locale_map,*p))
+	if (joe_isalphx(locale_map,*p)) {
+		while(len && joe_isalnux(locale_map,*p))
 			*buf++= *p++, --len;
 		*buf=0;
-		while(joe_isalnum_(locale_map,*p))
+		while(joe_isalnux(locale_map,*p))
 			++p;
 		*pp = p;
 		return 0;
@@ -136,7 +133,8 @@ int parse_ident(unsigned char **pp, unsigned char *buf, int len)
 
 /* Parse to next whitespace */
 
-int parse_tows(unsigned char **pp, unsigned char *buf)
+int
+parse_tows(unsigned char **pp, unsigned char *buf)
 {
 	unsigned char *p = *pp;
 	while (*p && *p!=' ' && *p!='\t' && *p!='\n' && *p!='\r' && *p!='#')
@@ -149,12 +147,13 @@ int parse_tows(unsigned char **pp, unsigned char *buf)
 
 /* Parse a keyword */
 
-int parse_kw(unsigned char **pp, unsigned char *kw)
+int
+parse_kw(unsigned char **pp, const unsigned char *kw)
 {
 	unsigned char *p = *pp;
 	while(*kw && *kw==*p)
 		++kw, ++p;
-	if(!*kw && !joe_isalnum_(locale_map,*p)) {
+	if(!*kw && !joe_isalnux(locale_map,*p)) {
 		*pp = p;
 		return 0;
 	} else
@@ -163,7 +162,8 @@ int parse_kw(unsigned char **pp, unsigned char *kw)
 
 /* Parse a field */
 
-int parse_field(unsigned char **pp, unsigned char *kw)
+int
+parse_field(unsigned char **pp, const unsigned char *kw)
 {
 	unsigned char *p = *pp;
 	while(*kw && *kw==*p)
@@ -177,7 +177,8 @@ int parse_field(unsigned char **pp, unsigned char *kw)
 
 /* Parse a character */
 
-int parse_char(unsigned char **pp, unsigned char c)
+int
+parse_char(unsigned char **pp, unsigned char c)
 {
 	unsigned char *p = *pp;
 	if (*p == c) {
@@ -187,29 +188,14 @@ int parse_char(unsigned char **pp, unsigned char c)
 		return -1;
 }
 
-/* Parse an integer.  Returns 0 for success. */
-
-int parse_int(unsigned char **pp, int *buf)
+/*
+ * Parse a string into a buffer.  Returns 0 for success.
+ * Leaves escape sequences in string.
+ */
+int
+parse_string(unsigned char **pp, unsigned char *buf, int len)
 {
 	unsigned char *p = *pp;
-	if ((*p>='0' && *p<='9') || *p=='-') {
-		*buf = atoi((char *)p);
-		if(*p=='-')
-			++p;
-		while(*p>='0' && *p<='9')
-			++p;
-		*pp = p;
-		return 0;
-	} else
-		return -1;
-}
-
-/* Parse a string into a buffer.  Returns 0 for success.
-   Leaves escape sequences in string. */
-
-int parse_string(unsigned char **pp, unsigned char *buf, int len)
-{
-	unsigned char *p= *pp;
 	if(*p=='\"') {
 		++p;
 		while(len && *p && *p!='\"')
@@ -236,10 +222,10 @@ int parse_string(unsigned char **pp, unsigned char *buf, int len)
 }
 
 /* Parse a character range: a-z */
-
-int parse_range(unsigned char **pp, int *first, int *second)
+int
+parse_range(unsigned char **pp, int *first, int *second)
 {
-	unsigned char *p= *pp;
+	unsigned char *p = *pp;
 	int a, b;
 	if(!*p)
 		return -1;
