@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/va.h,v 1.3 2012/06/08 16:55:29 tg Exp $ */
 /*
  *	Variable length arrays of strings
  *	Copyright
@@ -9,7 +8,9 @@
 #ifndef _JOE_VA_H
 #define _JOE_VA_H 1
 
-#include "config.h"
+#ifdef EXTERN_B_C
+__IDSTRING(rcsid_va_h, "$MirOS: contrib/code/jupp/va.h,v 1.8 2018/01/07 20:32:47 tg Exp $");
+#endif
 
 #include "vs.h"
 
@@ -37,13 +38,13 @@ typedef unsigned char *aELEMENT;
 /* aELEMENT *vamk(int len);
  * Create a variable length array.  Space for 'len' elements is preallocated.
  */
-aELEMENT *vamk PARAMS((int len));
+aELEMENT *vamk(int len);
 
 /* void varm(aELEMENT *vary);
  * Free an array and everything which is in it.  Does nothing if 'vary' is
  * 0.
  */
-void varm PARAMS((aELEMENT *vary));
+void varm(aELEMENT *vary);
 
 /********************/
 /* Space management */
@@ -58,8 +59,8 @@ void varm PARAMS((aELEMENT *vary));
  * aSIZ returns 0 if you pass it 0.  aSiz does not do this checking,
  * but can be used as an lvalue.
  */
-#define aSIZ(a) ((a) ? *((int *)(a) - 2) : 0)
-#define aSiz(a) (*((int *)(a) - 2))
+#define aSiz(a) jalloc_siz(a)
+#define aSIZ(a) ((a) ? aSiz(a) : 0)
 
 /* int aLEN(aELEMENT *vary);
  * int aLen(aELEMENT *vary);
@@ -73,14 +74,14 @@ void varm PARAMS((aELEMENT *vary));
  * aLEN return a length of zero if 'vary' is 0.
  * aLen doesn't do this checking, but can be used as an lvalue
  */
-#define aLEN(a) ((a) ? *((int *)(a) - 1) : 0)
-#define aLen(a) (*((int *)(a) - 1))
+#define aLen(a) jalloc_len(a)
+#define aLEN(a) ((a) ? aLen(a) : 0)
 
 /* int alen(aELEMENT *ary);
  * Compute length of char or variable length array by searching for termination
  * element.  Returns 0 if 'vary' is 0.
  */
-int alen PARAMS((aELEMENT *ary));
+int alen(aELEMENT *ary);
 
 /* aELEMENT *vaensure(aELEMENT *vary, int len);
  * Make sure there's enough space in the array for 'len' elements.  Whenever
@@ -88,7 +89,7 @@ int alen PARAMS((aELEMENT *ary));
  * minimum space in anticipation of future expansion.  If 'vary' is 0,
  * it creates a new array.
  */
-aELEMENT *vaensure PARAMS((aELEMENT *vary, int len));
+aELEMENT *vaensure(aELEMENT *vary, int len);
 
 /* aELEMENT *vazap(aELEMENT *vary, int pos, int n);
  * Destroy n elements from an array beginning at pos.  Is ok if pos/n go
@@ -97,13 +98,13 @@ aELEMENT *vaensure PARAMS((aELEMENT *vary, int len));
  * function does not actually write to the array.  This does not stop if
  * a aterm is encountered.
  */
-aELEMENT *vazap PARAMS((aELEMENT *vary, int pos, int n));
+aELEMENT *vazap(aELEMENT *vary, int pos, int n);
 
 /* aELEMENT *vatrunc(aELEMENT *vary, int len);
  * Truncate array to indicated size.  This zaps or expands with blank elements
  * and sets the LEN() of the array.  A new array is created if 'vary' is 0.
  */
-aELEMENT *vatrunc PARAMS((aELEMENT *vary, int len));
+aELEMENT *vatrunc(aELEMENT *vary, int len);
 
 /************************************/
 /* Function which write to an array */
@@ -117,14 +118,14 @@ aELEMENT *vatrunc PARAMS((aELEMENT *vary, int len));
  * This does not zap previous values.  If you need that to happen, call
  * vazap first.  It does move the terminator around properly though.
  */
-aELEMENT *vafill PARAMS((aELEMENT *vary, int pos, aELEMENT el, int len));
+aELEMENT *vafill(aELEMENT *vary, int pos, aELEMENT el, int len);
 
 /* aELEMENT *vandup(aELEMENT *vary, int pos, aELEMENT *array, int len);
  * Duplicate 'len' elements from 'array' onto 'vary' beginning at position
  * 'pos'.  'array' can be a char array since its length is passed seperately.  A
  * new array is created if 'vary' is 0.
  */
-aELEMENT *vandup PARAMS((aELEMENT *vary, int pos, aELEMENT *array, int len));
+aELEMENT *vandup(aELEMENT *vary, int pos, aELEMENT *array, int len);
 
 /* aELEMENT *vadup(aELEMENT *vary);
  * Duplicate array.  This is just a functionalized version of:
@@ -139,7 +140,7 @@ aELEMENT *vandup PARAMS((aELEMENT *vary, int pos, aELEMENT *array, int len));
  * functionalized instances of the ones you use most often - especially since
  * the macros aren't safe).
  */
-aELEMENT *vadup PARAMS((aELEMENT *vary));
+aELEMENT *vadup(aELEMENT *vary);
 
 /* aELEMENT *vaset(aELEMENT *vary, int pos, aELEMENT element);
  * Set an element in an array.  Any value of 'pos' is valid.  A new array
@@ -147,7 +148,7 @@ aELEMENT *vadup PARAMS((aELEMENT *vary));
  * deleted.    This does not duplicate 'element'.  If you need 'element'
  * duplicated, call: vaset(vary,pos,adup(element));
  */
-aELEMENT *_vaset PARAMS((aELEMENT *vary, int pos, aELEMENT el));
+aELEMENT *_vaset(aELEMENT *vary, int pos, aELEMENT el);
 
 #define vaset(v,p,el)  \
  (!(v) || (p) > aLen(v) || ((p) == aLen(v) && (p) == aSiz(v)) ?  \
@@ -225,12 +226,12 @@ aELEMENT *_vaset PARAMS((aELEMENT *vary, int pos, aELEMENT el));
 /* aELEMENT *vasort(aELEMENT *ary, int len)
  * Sort the elements of an array (char or variable length) using qsort().
  */
-aELEMENT *vasort PARAMS((aELEMENT *ary, int len));
+aELEMENT *vasort(aELEMENT *ary, int len);
 
 /* aELEMENT *vawords(aELEMENT *a, char *s, int len, char *sep, int seplen);
  * Generate list of strings out of words in 's' seperated with the characters
  * in 'sep'.  The characters in 'sep' must be sorted.
  */
-aELEMENT *vawords PARAMS((aELEMENT *a, unsigned char *s, int len, unsigned char *sep, int seplen));
+aELEMENT *vawords(aELEMENT *a, unsigned char *s, int len, const unsigned char *sep, int seplen);
 
 #endif
