@@ -1,4 +1,3 @@
-/* $MirOS: contrib/code/jupp/w.c,v 1.7 2017/01/11 22:56:50 tg Exp $ */
 /*
  *	Window system
  *	Copyright
@@ -9,9 +8,9 @@
 #include "config.h"
 #include "types.h"
 
-#ifdef HAVE_STDLIB_H
+__RCSID("$MirOS: contrib/code/jupp/w.c,v 1.13 2017/12/20 22:52:11 tg Exp $");
+
 #include <stdlib.h>
-#endif
 
 #include "blocks.h"
 #include "kbd.h"
@@ -168,7 +167,7 @@ SCREEN *scr;
 
 SCREEN *screate(SCRN *scrn)
 {
-	SCREEN *t = (SCREEN *) joe_malloc(sizeof(SCREEN));
+	SCREEN *t = malloc(sizeof(SCREEN));
 
 	t->t = scrn;
 	t->w = scrn->co;
@@ -218,8 +217,8 @@ void scrins(B *b, long l, long n, int flg)
 	if ((w = scr->topwin) != NULL) {
 		do {
 			if (w->y >= 0) {
-				if (w->object && w->watom->ins)
-					w->watom->ins(w->object, b, l, n, flg);
+				if (w->object.base && w->watom->ins)
+					w->watom->ins(w->object.bw, b, l, n, flg);
 			}
 		w = w->link.next;
 		} while (w != scr->topwin);
@@ -233,8 +232,8 @@ void scrdel(B *b, long l, long n, int flg)
 	if ((w = scr->topwin) != NULL) {
 		do {
 			if (w->y >= 0) {
-				if (w->object && w->watom->del)
-					w->watom->del(w->object, b, l, n, flg);
+				if (w->object.base && w->watom->del)
+					w->watom->del(w->object.bw, b, l, n, flg);
 			}
 		w = w->link.next;
 		} while (w != scr->topwin);
@@ -264,7 +263,7 @@ void wfit(SCREEN *t)
 
 	dostaupd = 1;
 
-      tryagain:
+ tryagain:
 	y = t->wind;
 	left = t->h - y;
 	pw = NULL;
@@ -335,7 +334,7 @@ void wfit(SCREEN *t)
 						l = pw;
 				}
 				/* Scroll windows between l and w */
-			      loop1:
+ loop1:
 				if (l->ny >= 0 && l->y >= 0)
 					nscrldn(t->t, l->y, l->ny + uns_min(l->h, l->nh), l->ny - l->y);
 				if (w != l) {
@@ -352,7 +351,7 @@ void wfit(SCREEN *t)
 						l = pw;
 				}
 				/* Scroll windows between l and w */
-			      loop0:
+ loop0:
 				if (w->ny >= 0 && w->y >= 0)
 					nscrlup(t->t, w->ny, w->y + uns_min(w->h, w->nh), w->y - w->ny);
 				if (w != l) {
@@ -370,7 +369,7 @@ void wfit(SCREEN *t)
 	w = t->topwin;
 	do {
 		if (w->ny >= 0) {
-			if (w->object) {
+			if (w->object.base) {
 				if (w->watom->move)
 					w->watom->move(w->object, w->x, w->ny);
 				if (w->watom->resize)
@@ -576,7 +575,7 @@ W *wcreate(SCREEN *t, WATOM *watom, W *where, W *target, W *original, int height
 		return NULL;
 
 	/* Create the window */
-	new = (W *) joe_malloc(sizeof(W));
+	new = malloc(sizeof(W));
 	new->notify = notify;
 	new->t = t;
 	new->w = t->w - 1;
@@ -589,7 +588,7 @@ W *wcreate(SCREEN *t, WATOM *watom, W *where, W *target, W *original, int height
 	new->huh = huh;
 	new->orgwin = original;
 	new->watom = watom;
-	new->object = NULL;
+	new->object.base = NULL;
 	new->msgb = NULL;
 	new->msgt = NULL;
 	/* Set window's target and family */
@@ -606,7 +605,7 @@ W *wcreate(SCREEN *t, WATOM *watom, W *where, W *target, W *original, int height
 	if (original) {
 		if (original->h - height <= 2) {
 			/* Not enough space for window */
-			joe_free(new);
+			free(new);
 			return NULL;
 		} else
 			seth(original, original->h - height);
@@ -643,7 +642,7 @@ static int doabort(W *w, int *ret)
 	w->y = -2;
 	if (w->t->topwin == w)
 		w->t->topwin = w->link.next;
-      loop:
+ loop:
 	z = w->t->topwin;
 	do {
 		if (z->orgwin == w)
@@ -668,7 +667,7 @@ static int doabort(W *w, int *ret)
 		amnt = 0;
 	}
 	deque(W, link, w);
-	if (w->watom->abort && w->object) {
+	if (w->watom->abort && w->object.base) {
 		*ret = w->watom->abort(w->object);
 		if (w->notify)
 			*w->notify = -1;
@@ -678,8 +677,8 @@ static int doabort(W *w, int *ret)
 			*w->notify = 1;
 	}
 	rmkbd(w->kbd);
-	joe_free(w);
 	windie(w);
+	free(w);
 	return amnt;
 }
 
@@ -751,18 +750,22 @@ void msgnwt(W *w, const unsigned char *s)
 	w->msgt = s;
 }
 
-int urtn(BASE *b, int k)
+int urtn(jobject jO, int k)
 {
+	BASE *b = jO.base;
+
 	if (b->parent->watom->rtn)
-		return b->parent->watom->rtn(b, k);
+		return b->parent->watom->rtn(jO);
 	else
 		return -1;
 }
 
-int utype(BASE *b, int k)
+int utype(jobject jO, int k)
 {
+	BASE *b = jO.base;
+
 	if (b->parent->watom->type)
-		return b->parent->watom->type(b, k);
+		return b->parent->watom->type(jO, k);
 	else
 		return -1;
 }
